//+--------------------------------------------------------------------------------------------+
//|                                                                                            |
//|                                SonicR Chart Panel.mq4                                      | 
//|                                                                                            |
//+--------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2010 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview:

This indicator places a colored panel in the upper left corner of the chart.  Three panel sizes
are available (sm, med, lg).  Within the large panel are five labels: (1) the pair symbol and 
time frame, (2) the spread, (3) the range (average for the period selected, and current for the 
session), (4) the time to the next candle, (5) the market bid price, and (6) the "SonicR" label 
in an extension panel.  If the medium panel is selected the range item is not displayed.  If 
the small panel is selected both the range and candle time items are not displayed.  You can 
turn off the "SonicR" extension panel so that it does not appear with the Chart Panel.  You can
hide the chart OHLC data so that it does not show.  You can select default colors for white and 
black charts, which override the External Inputs colors.  In this manner, you can use external 
input colors to store customized colors, and still have immediate access to colors for white 
and black charts. 

The indicator can be used for both forex and non-forex items. In the Forex mode, you can show 
either 4 digit or 5 digits.  If your platform is 4 digit and you select the 5 digit display, 
a "0" is added to displayed numbers. If it is a 5 digit platform and you select to display 4 
digits, numbers are rounded off to 4 digits.  

This indicator optionally charts: (1) a bid ray line, (2) a background of level lines, 
(3) range high/low lines, and (4) vertical time lines and day separators.

Optional Bid Ray Line-
You can select to place your own Bid Ray Line on the chart, which you can vary in thickness and
color.  The Bid Ray Line ends at the current candle, or at your specified number of TFs beyond.
It can terminate with an optional Bid Dot that changes color(grn/red) as the bid price changes
(up/dn), and can be sized. Or, you can display just the Bid Dot without the Bid Ray Line.  To
display only your own Bid Ray Line, right click on the chart, select Chart Properties/Colors 
and make the Grid color = "None".  Otherwise, the MT4 bid line is beneath your own, and will be 
visible from the end of your line to the chart edge.
 
Optional Level Lines-
This indicator will place level lines on your chart at key levels between whole numbers (00, 12,
25, 38, 50, 62, 75, 88 and 00 again).  The number of "sets" of these lines above and below the
current price range "set" is determined by hard code.  But "Override_Line_Sets_Displayed" allows
for change.  For example, if "1" is entered, then one set of lines is displayed for the
current range.  If "2" is entered, then one set of lines is displayed for the current range
and one set is displayed both above and below the current range.  An input of "0" or less will
default to the hard code.  

The lines can individually be of any available color and style, and of any thickness for solid
lines.  And line catagories other than whole number can be turned On/Off individually while 
the other catagories remain displayed.

Optional Range Lines-
You can select to display two horizontal lines, one for the computed range high target for 
the current session, and the other for the computed range low target for the current session.
The computed range is based on an averaging period you select (default is 14 days).  The 
range averaging process eliminates brief Sunday sessions, replacing them with another full
session day.  So, no matter the averaging period, there are no short Sunday sessions to drag 
the average down, and no days missing from the averaging process.

The optional range H/L lines are displayed in two different ways, according to two different
conditions.  Condition #1 is that the session range has not exceeded the computed average
range.  The RgH line is the computed average range distance above the session Low, and the
RgL line is the computed average range distance below the session high.  The lines will move
as new highs/lows are achieved during the session.  This display shows how far price can move 
in either direction before exceeding the computed average range.  Condition #2 occurs if PA 
exceeds the computed average range.  Then, if price is in the upper half of the new increased 
range the RgL line is at the session low and the RgH line is at the computed average range 
distance above the RgL line.  And if price is in the lower half of the new increased range the 
RgH line is at the session high and the RgL line is at the computed average range distance 
below RgH line.  This display shows the relationship between the computed average range limit
and the current session PA (Price Activity), which has moved beyond that limit.

Optional Vertical Time Lines-
You can select to display vertical time lines for day separators, for SonicR approved trading 
hours, and for many forex market open times; Sydney, Tokyo, Beijing, Moscow, Frankfurt, London 
and New York, and even the open of the New York Stock Exchange.  Setting up the times for 
these vertical lines is more fully commented on as follows.
 
All vertical lines are set by default for a broker/server based upon the GMT time zone.  You 
should not have to mess with changing the individual inputs for these lines if you properly
complete two important inputs.  1) What is your broker/server "DST_Server_GMT_Offset" 
in +/- hours?  2) What is your broker/server "DST_Zone"? 

If your broker/server is at GMT time all year, then your "DST_Server_GMT_Offset" is zero.
Otherwise, determine your "DST_Server_GMT_Offset" by using the SonicR Clock Panel.  Set it
to display in 24 hour format, showing "GMT" and "Broker".  If it is during the DST time of
year that you are setting this up, the hours difference (+/-) between "Broker" and "GMT" is 
your "DST_Server_GMT_Offset".  If it is the non-DST time of year that you are setting this up, 
then add one hour to the difference.  For example, London is at GMT+1 during DST, but during
the non-DST time of year London is at GMT+0. So, if you were setting this up during the
non-DST time of year you would add "+1" to the zero difference to get the proper input "1". 

Four DST zones are preset for you to choose from (US, EU, AUS and NZ), hard coded to make
an hour shift for DST changeovers at the appropriate time twice yearly.  If your broker
does not implement DST changeovers on their server per one of these 4 zones, then you must 
customize a 5th DST zone based on when your broker does make DST changeovers.  Be warned
that you cannot reliably assume that your broker/server is in the "eu" zone just because 
the chart time scale is based on what time it is somewhere in Europe.  Nor can you reliably 
assume that your broker/server is in the "us" zone just because that is where the broker is 
located.  You are therefore advised to ignore your chart time scale and broker location and
find out from your broker exactly when do they implement DST changes on their server.

For "DST_Zone" enter the number (1, 2, 3, 4) for the DST zone that represents when your 
broker/server makes the DST changeovers.  If your broker/server makes DST changeovers not 
covered by these DST zones, enter "5" and proceed to customize a 5th DST zone accordingly.

To customize your DST Zone settings just follow the instructions provided by Caveman in the 
External Inputs Window.  You are to input the correct Month, which Sunday of the Month, and 
at what hour (24 hour format) your broker/server starts DST.  And repeat the same information 
for when your broker/server ends DST.  If you do not know when these times are, obtain this 
information from your broker.

Here is some "background" information that helps to explain what is going on.  The location
of the vLines is determined by the number of hours these events occur after the session start,
per your broker/server time.  For example, if the London Open starts 7 hours into the broker 
server day, then you would enter "7" for the London_Open_Time.  If the New Your Open starts 
5 hours later, you would enter "12" for the NewYork_Open_Time.  In other words, all times are
based on when your broker/server day starts, and when during that day the individual events
occur.  From this, you can easily understand that for broker/servers in different time zones,
the hours to input for the various events will be different.  It is somewhat simplified by
defaulting to hours appropriate to a GMT based broker/server and then using the user's actual
broker/server GMT offset, together with CaveMan's DST code, to automatically make the twice
yearly necessary changes.

A final note on the vertical lines (vlines) settings.  They can be set to 1/2 hour times.
To do this simply add ".5" to the number.  Only whole hour inputs, or whole hour and a half
inputs will result in properly placed lines.

Some servers/brokers have a very brief Sunday session.  This causes some or all of the vlines
for that session to "stack" up.  When the next session starts, the new current session vlines 
will display properly, but the vlines for that brief previous session will still be "stacked".   
There is no known easy coding solution available. 
                                                                    - Traderathome, 12-01-2010                                                                    
------------------------------------------------------------------------------------------------
Acknowlegements:

CaveMan - who generously devoted his time and programming skills to create his benchmark DST 
          code which automates applying an hour time shift as appropriate due to DST/non-DST
          changes, and provides for customizing your own DST_Zone setting in case the times of
          your broker/server DST/non-DST changes is not covered by the preconfigured DST zones.   
Domas4  - "THANKS!" are owed to Domas4 for providing price labeling code compatible with non-
          forex items. 

------------------------------------------------------------------------------------------------
Suggested Settings:             White Chart       Black Chart
  
Chart_Background_Color          White             C'10,10,10'        
Panel_Background_Color          C'233,233,252'    C'25,25,25'
Symbol_And_TF_Color             Black             Gray
Spread_Color                    Black             DarkOrange
Range_Color                     Black             DarkOrange
Candle_Time_Color               Black             DarkOrange
PriceLabel_UP_Color             ForestGreen       C'46,188,46'
PriceLabel_DOWN_Color           Crimson           C'234,0,0' 
PriceLabel_SAME_Color           MediumBlue        C'0,115,230'
sonicS                          C'44,44,255'      C'0,121,230'
sonicM                          Black             WhiteSmoke
sonicR                          Red               Red
Bid_Ray_Color                   MidnightBlue      Gray
Bid_Dot_Up_Color                Lime              Lime
Bid_Dot_Dn_Color                OrangeRed         OrangeRed
RangeHigh_Color                 Tomato            Red
RangeLow_Color                  C'106,106,255'    C'0,115,230'
RangeLabels_Color               MidnightBlue      DarkGray
vLines_Color                    Black             C'74,74,0'
vLabels_Color                   BlueViolet        DarkOrange
Whole_Number_Color              Lavender          C'65,51,119'
Half_Number_Color               C'207,250,207'    C'0,90,0'
Quarter_Number_Color            C'242,238,227'    C'77,56,36'
Eighth_Number_Color             C'230,233,236'    C'45,45,45'
  
 
GMT Based MktOpen vLines/Times:         -Time At GMT When Event Occurs-
                                        DST        Non-DST    Hrs Shift

Sydney                                  -3           -3            0    During non-DST, some
Tokyo                                   -1           -1            0    events occur an hour 
Beijing                                  0            0            0    later.  Use DST hrs.
Moscow                                   4            5           -1    Code adds an hr. to
1am EST (SonicR Trading Start)           5            6           -1    the start times of 
Frankfurt                                6            7           -1    these certain events
London                                   7            8           -1    to make them occur
New York                                 12           13          -1    an hour later during
Stock_Exchange_Open_Time                 13.5         14.5        -1    non-DST than they do
11am EST  (SonicR Trading End)           15           16          -1    during DST.
                                                                                                                                                                
----------------------------------------------------------------------------------------------*/


//+--------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                    |                                                        
//+--------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

//Global External Inputs------------------------------------------------------------------------ 
extern string Part_1                          = "Indicator Display Controls:";
extern bool   Indicator_On                    = true;
extern bool   Main_Panel_On                   = true;
extern int    __Apply_Panel_Sm_Med_Lg_123     = 3;
extern bool   __Show_SonicR_Extension         = true;
extern bool   __Obscure_Chart_OHLC_Info       = true;
extern bool   Bid_On                          = true;
extern bool   Levels_On                       = true;
extern int    Levels_Max_TF                   = 60;
extern bool   Range_H\L_Lines_On              = true;  
extern int    Range_H\L_Lines_Max_Display_TF  = 60;
extern bool   vLines_On                       = true;
extern int    vLines_Max_Display_TF           = 60;
extern int    DST_Server_GMT_Offset           = 0;
extern bool   Apply_Default_Color_Selections  = true;
extern bool   Apply_On_White_vs_Black_Chart   = true;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string __                              = "";
extern string Part_2                          = "Market Panel Settings:";
extern bool   Use_For_Forex                   = true;
extern bool   Display_5_Digits_Forex          = false;
extern int    Digits_To_Show_In_Spread        = 0;
extern color  Chart_Background_Color          = White; 
extern color  Panel_Background_Color          = C'233,233,252';
extern color  Symbol_And_TF_Color             = Black; 
extern color  Spread_Color                    = Black;
extern color  Range_Color                     = Black;
extern color  Candle_Time_Color               = Black;
extern color  PriceLabel_UP_Color             = ForestGreen;
extern color  PriceLabel_DOWN_Color           = Crimson;
extern color  sonicS                          = C'44,44,255';
extern color  sonicM                          = Black;
extern color  sonicR                          = Red;

extern string ___                             = "";
extern string Part_3                          = "Bid Settings:";
extern color  Bid_Ray_Color                   = MidnightBlue; 
extern int    Bid_Ray_Thickness               = 1;
extern int    Bid_Ray_TFs_Past_Last_Candle    = 4;
extern bool   Bid_Dot_Included                = true;
extern bool   Bid_Dot_Only                    = false;
extern color  Bid_Dot_Up_Color                = Lime;  
extern color  Bid_Dot_Dn_Color                = OrangeRed;     
extern int    Bid_Dot_Size                    = 7;

extern string ____                            = "";
extern string Part_4                          = "Range Lines Settings:";
extern int    Days_For_Range_Average          = 14;  
extern string note1                           = "1 = Full screen";
extern string note2                           = "2 = Current Session";
extern string note3                           = "3 = Session start";
extern string note4                           = "4 = Last candle start";
extern int    Line_Selection_Number           =  2; 
extern color  RangeHigh_Color                 = Tomato;
extern color  RangeLow_Color                  = C'106,106,255';
extern int    RG_LineStyle_01234              = 0;    
extern int    RG_SolidLineThickness           = 4;
extern bool   Subodinate_Range_Lines          = false;

extern string _____                           = "";
extern string Part_5                          = "Range Labels Settings:";
extern color  RangeLabels_Color               = MidnightBlue;
extern bool   RangeLabels_Shown               = true;
extern string RangeLabels_FontStyle           = "Verdana"; 
extern int    RangeLabels_FontSize            = 8;  
extern bool   RangeLabels_MaxRight            = false;
extern bool   Subodinate_Range_Labels         = false;

extern string ______                          = "";
extern string Part_6                          = "vLines Selections:";
extern color  vLines_Color                    = Silver; 
extern int    vLines_Style_01234              = 2;    
extern int    vLines_Solid_Thickness          = 1;
extern bool   vLines_Extend_To_SubWindows     = false;
extern bool   vLines_Current_Session_On       = true;
extern int    vLines_Prior_Sessions_To_Show   = 1;
extern bool   Day_Separators_On               = true;
extern color  Day_Separators_Color            = BlueViolet; 
extern bool   Trading_Start_On                = true;
extern bool   Trading_Stop_On                 = true;
extern bool   Sydney_Open_On                  = false;
extern bool   Tokyo_Open_On                   = false;
extern bool   Beijing_Open_On                 = false;
extern bool   Moscow_Open_On                  = false;
extern bool   FrankFurt_Open_On               = false;
extern bool   London_Open_On                  = true;
extern bool   NewYork_Open_On                 = true; 
extern bool   Stock_Exchange_Open_On          = false;

extern string _______                         = "";
extern string Part_7                          = "vLines Time Settings:";
extern double Sydney_Open_Time                = -3;   
extern double Tokyo_Open_Time                 = -1;   
extern double Beijing_Open_Time               = 0;   
extern double Moscow_Open_Time                = 4;    
extern double Trading_Start_Time              = 5;   
extern double Frankfurt_Open_Time             = 6;    
extern double London_Open_Time                = 7;    
extern double NewYork_Open_Time               = 12;   
extern double Stock_Exchange_Open_Time        = 13.5; 
extern double Trading_Stop_Time               = 15;  

extern string ________                        = "";
extern string Part_8                          = "vLabels Settings:";
extern color  vLabels_Color                   = BlueViolet; 
extern string vLabels_FontStyle               = "Verdana Bold";
extern int    vLabels_FontSize                = 8;
extern bool   vLabels_Chart_Top               = true;
extern int    vLabels_Dist_To_Border          = 1;
extern string Label_Trading_Start             = "   >";
extern string Label_Trading_Stop              = "<  ";
extern string Label_Sydney_Open               = "    S";
extern string Label_Tokyo_Open                = "    T";
extern string Label_Beijing_Open              = "    B";
extern string Label_Moscow_Open               = "    M";
extern string Label_Frankfurt_Open            = "    F";
extern string Label_London_Open               = "    L";
extern string Label_NewYork_Open              = "    N";
extern string Label_Stock_Exchange_Open       = "    E";

extern string _________                       = "";
extern string Part_9                          = "Levels Settings:";
extern bool   Half_Number_Lines_On            = true;
extern bool   Quarter_Number_Lines_On         = true;
extern bool   Eighth_Number_Lines_On          = true;
extern int    Override_Line_Sets_Displayed    = 0;
extern color  Whole_Number_Color              = Lavender;  
extern int    Line1_Style_01234               = 0;
extern int    Line1_SolidThickness            = 3;
extern color  Half_Number_Color               = C'207,250,207'; 
extern int    Line2_Style_01234               = 0;
extern int    Line2_SolidThickness            = 2;
extern color  Quarter_Number_Color            = C'242,238,227';   
extern int    Line3_Style_01234               = 0;
extern int    Line3_SolidThickness            = 2;
extern color  Eighth_Number_Color             = C'230,233,236';
extern int    Line4_Style_01234               = 0;
extern int    Line4_SolidThickness            = 1;
  
// Start - Added by CaveMan to cater for DST
extern string __________                      = "";
extern string Part_10                         = "DST Zone Settings:";
extern string note_5                          = "Enter preset DST zone number 1 - 4";
extern string note_6                          = "or enter ' 5 ' for a custom DST zone.";
extern string note_7                          = "1 US   2 EU   3 AUS   4 NZ   5 Cust.";
extern int    DST_Zone                        = 2;
extern string note_8                          = "To customized A DST setting -";  
extern string note_9                          = "Enter # month DST starts,";
extern int    Custom_DST_Start_Month          = 3; //March
extern string note_10                         = "Enter # Sunday DST starts,";
extern string note_11                         = "1-1st Sunday, 2-2nd ....7-Last Sunday";
extern int    Custom_DST_Start_Sunday         = 2; //2nd Sunday of March start DST
extern string note_12                         = "Enter 24 hour time DST starts,";
extern string Custom_DST_Start_Time           = "02:00"; //DST starting time
extern string note_13                         = "Repeat for DST end,";
extern int    Custom_DST_End_Month            = 11; //November
extern int    Custom_DST_End_Sunday           = 1;  //1st Sunday of November end DST
extern string Custom_DST_End_Time             = "02:00"; //DST ending time
// End - Added by CaveMan to cater for DST

//Buffers, Constants and Variables-------------------------------------------------------------
bool     FLAG_deinit;
color    Static_Price_Color, Static_Bid_Color, Bid_Dot_Color, Static_Bid_Dot_Color;
int      obj_total,fxDigits,modifier,Color,Factor,Thickness,A,a,b,c,k,h,i,m,s,t,Z,F,T,T1,T2,TodayBar,bps,MF;
double   Old_Price, Spread, ARg, Range, RangeAvg, RangeHigh, RangeLow, RH, RL;
double   HiToday,LoToday,startToday,endToday,TodayRange;
double   top, bottom, scale,YadjustTop,YadjustBot,level;
datetime startline, stopline, startlabel, startrangeday, shift;  
string   item0  = "[Chart Panel] Box 1"; 
string   item1  = "[Chart Panel] Box 2";  
string   item2  = "[Chart Panel] Box 3"; 
string   item3  = "[Chart Panel] Box 4";
string   item4  = "[Chart Panel] Box 5";
string   item6  = "[Chart Panel] ID&TF";
string   item7  = "[Chart Panel] Spread"; 
string   item8  = "[Chart Panel] Range";     
string   item9  = "[Chart Panel] Time"; 
string   item10 = "[Chart Panel] Price";
string   item11 = "[Chart Panel] Logo1 S"; 
string   item12 = "[Chart Panel] Logo1 o"; 
string   item13 = "[Chart Panel] Logo1 n";     
string   item14 = "[Chart Panel] Logo1 i"; 
string   item15 = "[Chart Panel] Logo1 c";
string   item16 = "[Chart Panel] Logo1 R"; 
string   item17 = "[Chart Panel] Bid Line";
string   item18 = "[Chart Panel] Bid Dot";
string   sub,name,Price,C,rangeline,rangelabel,spc,tab,tab2,dt,vline,vlabel; 

// Start - Added by CaveMan/TAH to cater for DST
int      dst, Non_DST_Server_GMT_Offset, Server_GMT_Offset, LastRefresh, First_Time;
// End - Added by CaveMan/TAH to cater for DST

int      p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,
         p21,p22,p23,p24,p25,p26,p27,p28,p29,p30,p31;
string   G;
string   Label_Day_Separator = "day";
static int Day_Start_Time = 0;
static int Day_End_Time   = 24;

color    linecolor;
double   Poin, linelevel, linestyle, linewidth, BL;
int      Line1_Color, Line2_Color, Line3_Color, Line4_Color; 
int      u1=00, u2=50, u3=25, u4=75, u5=12, u6=38, u7=62, u8=88;        
int      c1, ssp, ssp1, NumberOfRanges;
string   linename;

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
   {
   FLAG_deinit = false;
      
   // Start - Added by CaveMan/modified by TAH to cater for DST
   Server_GMT_Offset = DST_Server_GMT_Offset;
   if (DST_Server_GMT_Offset != 0) {Non_DST_Server_GMT_Offset = DST_Server_GMT_Offset - 1;} 
   LastRefresh = TimeHour(iTime(NULL,0,0));
   First_Time = true;
   // End - Added by CaveMan/modified by TAH to cater for DST  
                
   if (Use_For_Forex)
      {
      sub=StringSubstr(Symbol(), 3, 3);
      if (Display_5_Digits_Forex) {if (sub == "JPY") {fxDigits = 3;} else {fxDigits = 5;}}
      else {if (sub == "JPY") {fxDigits = 2;} else {fxDigits = 4;}}
      } 
   if (Digits == 5 || Digits == 3) Factor = 10; else Factor = 1; //cater for 5 digits  
   
   if (__Apply_Panel_Sm_Med_Lg_123 == 1)
      {
      G= "gg";
      p1= 48; p2= 145; p3= 178; p4= 151; p5= 36; p6= 166; p7= 28; p8= 178; p9= 21;
      p10= 191; p11= 16; p12= 197; p13= 9;   
      p21= 48; p22= 82; p23= 2; p24= 46; p25= 19;
      p30= 11; p31= 33;
      }      
   if (__Apply_Panel_Sm_Med_Lg_123 == 2)
      {
      G= "g";
      p1=56; p2= 145; p3= 168; p4= 152; p5= 45; p6= 166; p7= 36; p8= 177; p9= 28; 
      p10= 191; p11= 21; p12= 199; p13= 11;
      p21= 56; p22= 71; p23= 2; p24= 34; p25= 18; p28= 34; p29= 31; 
      p30= 11; p31= 43;
      } 
   if (__Apply_Panel_Sm_Med_Lg_123 == 3)
      {
      G= "g";
      p1= 69; p2= 149; p3= 150; p4= 152; p5= 63; p6= 166; p7= 49; p8= 177; p9= 38;
      p10= 191; p11= 28; p12= 199; p13= 16; 
      p21= 69; p22= 58; p23= 2; p24= 34; p25= 19; p26= 34; p27= 33; p28= 34; p29= 47;
      p30= 11; p31= 61;
      }  
      
   if (Apply_Default_Color_Selections)
      {         
      if (Apply_On_White_vs_Black_Chart)
         {
         Chart_Background_Color          = White;         
         Panel_Background_Color          = C'233,233,252';
         Symbol_And_TF_Color             = Black; 
         Spread_Color                    = Black;
         Range_Color                     = Black;
         Candle_Time_Color               = Black;
         PriceLabel_UP_Color             = ForestGreen;
         PriceLabel_DOWN_Color           = Crimson;
         sonicS                          = C'44,44,255';
         sonicM                          = Black;
         sonicR                          = Red;
         Bid_Ray_Color                   = MidnightBlue; 
         Bid_Ray_Thickness               = 2;
         Bid_Ray_TFs_Past_Last_Candle    = 4;
         Bid_Dot_Included                = true; 
         Bid_Dot_Up_Color                = Lime; 
         Bid_Dot_Dn_Color                = OrangeRed;
         Bid_Dot_Size                    = 7;                                           
         RangeHigh_Color                 = Tomato;
         RangeLow_Color                  = C'106,106,255';
         RangeLabels_Color               = MidnightBlue;       
         vLines_Color                    = Silver;
         vLines_Style_01234              = 2;    
         vLines_Solid_Thickness          = 1;
         Day_Separators_Color            = BlueViolet;                       
         vLabels_Color                   = BlueViolet; 
         vLabels_FontStyle               = "Verdana Bold";
         vLabels_FontSize                = 8; 
         Whole_Number_Color              = Lavender;
         Line1_Style_01234               = 0;
         Line1_SolidThickness            = 3;         
         Half_Number_Color               = C'207,250,207';
         Line2_Style_01234               = 0;
         Line2_SolidThickness            = 2;            
         Quarter_Number_Color            = C'242,238,227';
         Line3_Style_01234               = 0;
         Line3_SolidThickness            = 2;            
         Eighth_Number_Color             = C'230,233,236'; 
         Line4_Style_01234               = 0;
         Line4_SolidThickness            = 1;  
         } 
      else
         {
         Chart_Background_Color          = C'10,10,10';          
         Panel_Background_Color          = C'25,25,25';
         Symbol_And_TF_Color             = Gray; 
         Spread_Color                    = DarkOrange;
         Range_Color                     = DarkOrange;
         Candle_Time_Color               = DarkOrange;
         PriceLabel_UP_Color             = C'46,188,46';
         PriceLabel_DOWN_Color           = C'234,0,0';
         sonicS                          = C'0,121,230';
         sonicM                          = WhiteSmoke;
         sonicR                          = Red;
         Bid_Ray_Color                   = Gray; 
         Bid_Ray_Thickness               = 2;
         Bid_Ray_TFs_Past_Last_Candle    = 4;
         Bid_Dot_Included                = true;
         Bid_Dot_Up_Color                = Lime; 
         Bid_Dot_Dn_Color                = Red; 
         Bid_Dot_Size                    = 7;                             
         RangeHigh_Color                 = Red;
         RangeLow_Color                  = C'0,115,230';
         RangeLabels_Color               = DarkGray;        
         vLines_Color                    = C'74,74,0'; 
         vLines_Style_01234              = 2;    
         vLines_Solid_Thickness          = 1;
         Day_Separators_Color            = C'73,73,73';                 
         vLabels_Color                   = Olive;  
         vLabels_FontStyle               = "Verdana Bold";
         vLabels_FontSize                = 8;
         Whole_Number_Color              = C'65,51,119';
         Line1_Style_01234               = 2;
         Line1_SolidThickness            = 1;         
         Half_Number_Color               = C'0,90,0';
         Line2_Style_01234               = 2;
         Line2_SolidThickness            = 1;           
         Quarter_Number_Color            = C'77,56,36';
         Line3_Style_01234               = 2;
         Line3_SolidThickness            = 1;          
         Eighth_Number_Color             = C'45,45,45'; 
         Line4_Style_01234               = 2;
         Line4_SolidThickness            = 1;      
         }
      }     

   //calculate line levels for user specified # of ranges or use preset number of ranges to show
   if (Override_Line_Sets_Displayed >= 1) {NumberOfRanges = Override_Line_Sets_Displayed-1;}
   
   else if(Period() == 1) {NumberOfRanges = 4;}
   else if(Period() == 5) {NumberOfRanges = 7;}
   else if(Period() == 15) {NumberOfRanges = 10;}
   else if(Period() == 30) {NumberOfRanges = 10;}   
   else if(Period() == 60) {NumberOfRanges = 10;}  
   else if(Period() == 240) {NumberOfRanges = 20;}  
   else if(Period() == 1440) {NumberOfRanges = 30;}    
     
   //Checking for unconventional Point digits number
   if (Point == 0.00001) Poin = 0.0001; //5 digits     
   else if (Point == 0.001) Poin = 0.01; //3 digits
   else Poin = Point; //Normal 4 digits
                         
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+       
int deinit()
   {   
   obj_total= ObjectsTotal();  
   for (k= obj_total; k>=0; k--){
      name= ObjectName(k); 
      if (StringSubstr(name,0,13)=="[Chart Panel]"){ObjectDelete(name);}}
   Comment("");     
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+         
int start()
   {
   //If Indicator is "Off" deinitialize only once, not every tick------------------------------  
   if (!Indicator_On) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
      
   //If one, or both, Range Line settings are invalid, exit------------------------------------            
   if(Line_Selection_Number <1 || Line_Selection_Number >4)           
      {
      Alert("SonicR Chart Panel:  Line_Selection_Number invalid"); return(false);
      }   
             
   //If indicator is "ON", deinitialize and run------------------------------------------------ 
   deinit(); FLAG_deinit = false; 

   //Define today's bar/data for use in labels and range lines                    
   TodayBar   = iBarShift(NULL,PERIOD_D1,Time[0]);
   HiToday    = iHigh (NULL,PERIOD_D1,TodayBar);
   LoToday    = iLow  (NULL,PERIOD_D1,TodayBar); 
   TodayRange = ((HiToday - LoToday)/Point)/Factor;  
   
   if (Main_Panel_On) { 
   //Create Background Boxe to obscure chart OHLC data 
   if (__Obscure_Chart_OHLC_Info) {     
   ObjectCreate(item0, OBJ_LABEL, 0, 0, 0, 0, 0);
   ObjectSetText(item0, "gggggggggggggggggggggggggg", 7, "Webdings");
   ObjectSet(item0, OBJPROP_CORNER, 0);
   ObjectSet(item0, OBJPROP_XDISTANCE, 0);
   ObjectSet(item0, OBJPROP_YDISTANCE, 0);   
   ObjectSet(item0, OBJPROP_COLOR, Chart_Background_Color);  
   ObjectSet(item0, OBJPROP_BACK, false); }       
   //Create Chart Main Panel Background boxes      
   ObjectCreate(item1, OBJ_LABEL, 0, 0, 0, 0, 0);
   ObjectSetText(item1, G, p21, "Webdings");
   ObjectSet(item1, OBJPROP_CORNER, 0);
   ObjectSet(item1, OBJPROP_XDISTANCE, 0);
   ObjectSet(item1, OBJPROP_YDISTANCE, 0);   
   ObjectSet(item1, OBJPROP_COLOR, Panel_Background_Color);
   ObjectSet(item1, OBJPROP_BACK, false);     
   //-----         
   ObjectCreate(item2, OBJ_LABEL, 0, 0, 0, 0, 0);
   ObjectSetText(item2, "g", p21, "Webdings");
   ObjectSet(item2, OBJPROP_CORNER, 0);
   ObjectSet(item2, OBJPROP_XDISTANCE, p22);//52
   ObjectSet(item2, OBJPROP_YDISTANCE, 0);   
   ObjectSet(item2, OBJPROP_COLOR, Panel_Background_Color);
   ObjectSet(item2, OBJPROP_BACK, false);  
         
   //Symbol and Time Frame   
   C= " "; if(Symbol()!="GBPUSD" || Symbol() !="EURUSD") {C= C+" ";}	         
            if (Period()== 1)  C =C +  "     "+Symbol()+"   M1";
      else {if (Period()== 5)  C =C +  "     "+Symbol()+"   M5";
      else {if (Period()== 15) C =C +  "    "+Symbol()+"   M15";
      else {if (Period()== 30)  C =C +  "    "+Symbol()+"   M30";
      else {if (Period()== 60)  C =C +  "     "+Symbol()+"   H1";
      else {if (Period()== 240)  C =C +  "     "+Symbol()+"   H4";
      else {if (Period()== 1440)  C =C +  "   "+Symbol()+"   Daily";
      else {if (Period()== 10080) C =C +  " "+Symbol()+"   Weekly";
      else {if (Period()== 43200) C =C +  ""+Symbol()+"   Monthly"; }}}}}}}}                          
   ObjectCreate(item6, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item6,OBJPROP_CORNER, 0);
   ObjectSet(item6,OBJPROP_XDISTANCE, 0);
   ObjectSet(item6,OBJPROP_YDISTANCE, p23);
   ObjectSet(item6, OBJPROP_COLOR, Symbol_And_TF_Color);         
   ObjectSetText(item6, C, 11, "Arial Bold");
                         
   //Spread
   Spread = MarketInfo(Symbol(), MODE_SPREAD);
   Spread = Spread/Factor;                                        
   ObjectCreate(item7, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item7,OBJPROP_CORNER, 0);
   ObjectSet(item7,OBJPROP_XDISTANCE, p24);
   ObjectSet(item7,OBJPROP_YDISTANCE, p25);
   ObjectSet(item7, OBJPROP_COLOR, Spread_Color);         
   ObjectSetText(item7, "Spread   " + DoubleToStr(Spread,Digits_To_Show_In_Spread), 9, "Arial");

   if (__Apply_Panel_Sm_Med_Lg_123 == 3)
      {
      //Average Range is total period for range, Sundays replaced with additional non-Sunday days.   
      Ranges(Days_For_Range_Average);
      Range = (ARg/(Days_For_Range_Average)/Point)/Factor; 
      ObjectCreate(item8, OBJ_LABEL, 0, 0, 0);
      ObjectSet(item8,OBJPROP_CORNER, 0);
      ObjectSet(item8,OBJPROP_XDISTANCE, p26);
      ObjectSet(item8,OBJPROP_YDISTANCE, p27);
      ObjectSet(item8, OBJPROP_COLOR, Range_Color);              
      ObjectSetText(item8, "Range    " +DoubleToStr(Range,0)+", "+DoubleToStr(TodayRange,0), 9, "Arial");       
      } 
      
   if ((__Apply_Panel_Sm_Med_Lg_123 == 2) || (__Apply_Panel_Sm_Med_Lg_123 == 3))
      {           
      //Time To New Candle	   
      t=Time[0]+(Period()*60)-CurTime();
      s=t%60; string seconds = (s); if (s<10) {seconds = ("0"+seconds);} 
      m=(t-t%60)/60; 
      h=0;  
      for(i=0; i<24; i++)
         {
         if(m>=60) {m=m-60;h=h+1;}
         string minutes = (m); if (m<10) {minutes = ("0"+minutes);}   
         string hours = (h); if (h<10) {hours = ("0"+hours);} 
         string timeleft = (minutes+":"+seconds);    
         if (h>=1) {timeleft= hours+":"+minutes+":"+seconds;}                        
         if (Period()>1440) {timeleft = " (OFF)";}
         }             
      ObjectCreate(item9, OBJ_LABEL, 0, 0, 0);
      ObjectSet(item9,OBJPROP_CORNER, 0);
      ObjectSet(item9,OBJPROP_XDISTANCE, p28);
      ObjectSet(item9,OBJPROP_YDISTANCE, p29);         
      ObjectSet(item9, OBJPROP_COLOR, Candle_Time_Color);        
      ObjectSetText(item9, "Candle   "+timeleft, 9, "Arial");   
      }       
      
   //Market Price 
   if (Bid > Old_Price) 
      {
      Color = PriceLabel_UP_Color; Bid_Dot_Color = Bid_Dot_Up_Color; 
      Static_Price_Color = Color; Static_Bid_Dot_Color = Bid_Dot_Up_Color; 
      }
   if (Bid < Old_Price) 
      {
      Color = PriceLabel_DOWN_Color; Bid_Dot_Color = Bid_Dot_Dn_Color; 
      Static_Price_Color = Color; Static_Bid_Dot_Color = Bid_Dot_Dn_Color; 
      }   
   if (Bid == Old_Price)  {Color = Static_Price_Color; Bid_Dot_Color = Static_Bid_Dot_Color;}  
   Old_Price=Bid;   
   //----Add leading spaces to center price labels
   tab="  "; tab2=" "; if (Display_5_Digits_Forex) {tab=" "; tab2="";}                  
	      if(Symbol()== "USDMXN"){tab= tab2+"";}	   
   else {if(Symbol()== "XAUUSD"){tab= tab2+"";} 	      	 
   else {if(Symbol()== "USDJPY"){tab= tab2+"  ";}
	else {if(Symbol()== "CHFJPY"){tab= tab2+"  ";}
	else {if(Symbol()== "AUDJPY"){tab= tab2+"  ";}
	else {if(Symbol()== "CADJPY"){tab= tab2+"  ";}
	else {if(Symbol()== "XAGUSD"){tab= tab2+"  ";}  }}}}}} 
   //----Set digits in Price & convert Price to string  
   if (Use_For_Forex)
      {
      if(Symbol()== "XAGUSD" || Symbol()== "XAUUSD") 
         {
         fxDigits = 2;
         if (Display_5_Digits_Forex) {fxDigits = 3;}         
         }      
      Price=DoubleToStr(Bid, fxDigits);      
      }         
   else {Price=DoubleToStr(Bid, Digits);}  
   //----Create/move label
   ObjectCreate(item10, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item10,OBJPROP_CORNER, 0);
   ObjectSet(item10,OBJPROP_XDISTANCE, p30); 
   ObjectSet(item10,OBJPROP_YDISTANCE, p31); 
   ObjectSet(item10, OBJPROP_COLOR, Color);     
   ObjectSetText(item10, tab+Price, 18, "Verdana Bold"); 
   }//end Main Panel routine
     
   //----Create SonicR extension panel
   if((Main_Panel_On)  && (__Show_SonicR_Extension)) {
   //Create Sonic Panel Background boxes
   ObjectCreate(item3, OBJ_LABEL, 0, 0, 0, 0, 0);        
   ObjectSetText(item3,  "g", p1, "Webdings");      
   ObjectSet(item3, OBJPROP_CORNER, 0);
   ObjectSet(item3, OBJPROP_XDISTANCE, p2);
   ObjectSet(item3, OBJPROP_YDISTANCE, 0 );    
   ObjectSet(item3, OBJPROP_COLOR, Panel_Background_Color);         
   ObjectSet(item3, OBJPROP_BACK, false);
   //-----  
   ObjectCreate(item4, OBJ_LABEL, 0, 0, 0, 0, 0);        
   ObjectSetText(item4,  "g", p1, "Webdings");      
   ObjectSet(item4, OBJPROP_CORNER, 0);
   ObjectSet(item4, OBJPROP_XDISTANCE, p3);
   ObjectSet(item4, OBJPROP_YDISTANCE, 0 );    
   ObjectSet(item4, OBJPROP_COLOR, Panel_Background_Color);         
   ObjectSet(item4, OBJPROP_BACK, false); 
   
   //Create logo         
   ObjectDelete(item11);
   ObjectCreate(item11, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item11,OBJPROP_CORNER, 0);
   ObjectSet(item11,OBJPROP_XDISTANCE, p4); 
   ObjectSet(item11,OBJPROP_YDISTANCE, p5);
   ObjectSet(item11, OBJPROP_COLOR, sonicS);  
   ObjectSetText(item11, "S", 16, "Verdana Bold");       
   //-----
   ObjectDelete(item12);
   ObjectCreate(item12, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item12,OBJPROP_CORNER, 0);
   ObjectSet(item12,OBJPROP_XDISTANCE, p6); 
   ObjectSet(item12,OBJPROP_YDISTANCE, p7);
   ObjectSet(item12, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item12, "o", 13, "Verdana Bold");    
   //-----
   ObjectDelete(item13);
   ObjectCreate(item13, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item13,OBJPROP_CORNER, 0);
   ObjectSet(item13,OBJPROP_XDISTANCE, p8); 
   ObjectSet(item13,OBJPROP_YDISTANCE, p9);
   ObjectSet(item13, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item13, "n", 13, "Verdana Bold");  
   //-----
   ObjectDelete(item14);
   ObjectCreate(item14, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item14,OBJPROP_CORNER, 0);
   ObjectSet(item14,OBJPROP_XDISTANCE, p10); 
   ObjectSet(item14,OBJPROP_YDISTANCE, p11);
   ObjectSet(item14, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item14, "i", 13, "Verdana Bold");       
   //-----
   ObjectDelete(item15);   
   ObjectCreate(item15, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item15,OBJPROP_CORNER, 0);
   ObjectSet(item15,OBJPROP_XDISTANCE, p12); 
   ObjectSet(item15,OBJPROP_YDISTANCE, p13);
   ObjectSet(item15, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item15, "c", 13, "Verdana Bold");      
   //-----
   ObjectDelete(item16);   
   ObjectCreate(item16, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item16,OBJPROP_CORNER, 0);
   ObjectSet(item16,OBJPROP_XDISTANCE, 210); //p14=210
   ObjectSet(item16,OBJPROP_YDISTANCE, 0);
   ObjectSet(item16, OBJPROP_COLOR, sonicR);     
   ObjectSetText(item16, "R", 16, "Verdana Bold");
   }//end SonicR Panel routine 
     
   //Bid Line---------------------------------------------------------------------------------- 
   if (Bid_On)
      {
      if (!Bid_Dot_Only){
      T1=Time[0]+(Period()*60*Bid_Ray_TFs_Past_Last_Candle); 
      T2=iTime(NULL, NULL, (Period()*60));
      Thickness = Bid_Ray_Thickness;
      F = true;                     
      if (ObjectFind(item17) != 0){
         ObjectCreate(item17, OBJ_TREND, 0, T1, Bid, T2, Bid);     
         ObjectSet(item17, OBJPROP_STYLE, 0);
         ObjectSet(item17, OBJPROP_COLOR, Bid_Ray_Color);
         ObjectSet(item17, OBJPROP_WIDTH, Thickness); 
         ObjectSet(item17, OBJPROP_BACK, false); 
         ObjectSet(item17, OBJPROP_RAY, F);}
      else{ 
         ObjectMove(item17, 0, T1, level);
         ObjectMove(item17, 1, T2, level);}}
      if (Bid_Dot_Included) {   
      T1=Time[0]+(Period()*60*(Bid_Ray_TFs_Past_Last_Candle)); 
      T2=Time[0]+(Period()*60*(Bid_Ray_TFs_Past_Last_Candle)); 
      Thickness = Bid_Dot_Size;
      F = false;  
      if (ObjectFind(item18) != 0){
         ObjectCreate(item18, OBJ_TREND, 0, T1, Bid, T2, Bid);     
         ObjectSet(item18, OBJPROP_STYLE, 0);
         ObjectSet(item18, OBJPROP_COLOR, Bid_Dot_Color);
         ObjectSet(item18, OBJPROP_WIDTH, Thickness); 
         ObjectSet(item18, OBJPROP_BACK, false); 
         ObjectSet(item18, OBJPROP_RAY, F);}
      else{ 
         ObjectMove(item18, 0, T1, level);
         ObjectMove(item18, 1, T2, level);} }                 
      }
       
   //Range High/Low----------------------------------------------------------------------------
   if (Range_H\L_Lines_On && Period()<= Range_H\L_Lines_Max_Display_TF)
      {         
      Ranges(Days_For_Range_Average);
      RangeAvg = NormalizeDouble(ARg/Days_For_Range_Average,4);                
      RangeHigh =  RangeAvg + iLow(NULL,PERIOD_D1,TodayBar);  //Comment(RangeHigh);
      RangeLow  = -RangeAvg + iHigh(NULL,PERIOD_D1,TodayBar); 

      if (HiToday - LoToday > RangeAvg)
         {            
         if (Bid >= HiToday- (HiToday-LoToday)/2) {RangeHigh = LoToday + RangeAvg; RangeLow  = LoToday;}
         else {RangeHigh  = HiToday; RangeLow = HiToday - RangeAvg;}
         }
             
      //Range Lines data to subroutine                                                          
      if(RG_LineStyle_01234>0){RG_SolidLineThickness=0;}    
      DrawRange("RgH", RangeHigh, RangeHigh_Color, RG_LineStyle_01234, RG_SolidLineThickness);   
      DrawRange("RgL",  RangeLow,  RangeLow_Color, RG_LineStyle_01234, RG_SolidLineThickness);        
      }//end range lines routine
  
   //vLines------------------------------------------------------------------------------------
   if (vLines_On && Period()<= vLines_Max_Display_TF)
      {            
      // Start - Added by CaveMan/modified by TAH to cater for DST
      // Code will only check for DST on every new hour bar
      if(TimeHour(iTime(NULL,0,0)) == 0) LastRefresh = -1; // to cater for crossover into a new day
      if(First_Time || TimeHour(iTime(NULL,0,0)) > LastRefresh)
         {
         if (Adjust_for_DST()) 
            {
            Print(" TzPivot - Daylight Savings Time (DST) is in effect.");           
            dst = 0;
            }
         //During DST, most vLine events (but not all) occur 1 hr. earlier, relative to GMT,  
         //than they do during the non-DST time of the year.  So when it is non-DST (an hour shift
         //back/later in time), 1 hr. must be added to the DST based start times to make them occur 
         //an hour later than during DST.  For example, during DST London is GMT+1, but is at GMT+0
         //during non-DST.  Things are happening an hour later.  So, for a GMT based server the
         //London DST start time occurs at 7am GMT time, but occurs at 8am during non-DST, an Hour
         //later.  Therefore, at the shift from DST to non-DST, an hour must be added to the start
         //times of these certain events to make the event vLine occur an hr. later.  This has 
         //nothing to do with the broker/server GMT offset change at DST.
         //"dst" is a component of the GMT based start time for those certain vLine events.  
         else {dst = 1;}           
         First_Time = false; LastRefresh = TimeHour(iTime(NULL,0,0));
         }
      // End - Added by CaveMan/modified by TAH to cater for DST
 
      //Calculate position for vline labels----------------------------------------------------  
      if (vLabels_Dist_To_Border < 1) {vLabels_Dist_To_Border = 1;}            
      top = WindowPriceMax();
      bottom = WindowPriceMin();
      scale = top - bottom;            
      YadjustTop = scale/(9000/vLabels_FontSize);      
      YadjustTop = YadjustTop + (vLabels_Dist_To_Border * YadjustTop);      
      YadjustBot = scale/(500/vLabels_FontSize); 
      YadjustBot = YadjustBot + ((vLabels_Dist_To_Border * YadjustBot)/20);                  	      	
      level = top - YadjustTop; level = level;                 
      if (!vLabels_Chart_Top) {level = bottom + YadjustBot;}       
            
      //Do vlines for current session----------------------------------------------------------
      if (vLines_Current_Session_On) 
      {                        
      if (Day_Separators_On)
         {
         //Do start of session line and label
         if(vLines_Prior_Sessions_To_Show == 0) 
            {
            OpenToday(Label_Day_Separator, Day_Start_Time, vLines_Color,          
            vLines_Style_01234, vLines_Solid_Thickness, level);
            } 
         //Do end of session line and label            
         OpenToday(Label_Day_Separator+" ", Day_End_Time, vLines_Color,                      
         vLines_Style_01234, vLines_Solid_Thickness, level);         
         }                      
      if (Trading_Start_On) 
         {
         OpenToday(Label_Trading_Start, Trading_Start_Time + dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }   
      if (Trading_Stop_On) 
         {
         OpenToday(Label_Trading_Stop, Trading_Stop_Time + dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }
      if (Sydney_Open_On) 
         {
         OpenToday(Label_Sydney_Open, Sydney_Open_Time, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }           
      if (Tokyo_Open_On) 
         {
         OpenToday(Label_Tokyo_Open, Tokyo_Open_Time, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }
      if (Beijing_Open_On) 
         {
         OpenToday(Label_Beijing_Open, Beijing_Open_Time, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }                      
      if (Moscow_Open_On) 
         {
         OpenToday(Label_Moscow_Open, Moscow_Open_Time + dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }         
      if (FrankFurt_Open_On) 
         {
         OpenToday(Label_Frankfurt_Open, Frankfurt_Open_Time + dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }          
      if(London_Open_On) 
         {
         OpenToday(Label_London_Open, London_Open_Time+dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         } 
      if(NewYork_Open_On) 
         {
         OpenToday(Label_NewYork_Open, NewYork_Open_Time + dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }
      if(Stock_Exchange_Open_On) 
         {
         OpenToday(Label_Stock_Exchange_Open, Stock_Exchange_Open_Time + dst, vLines_Color, 
         vLines_Style_01234, vLines_Solid_Thickness, level);
         }
      }
                    
      //Do vlines for previous sessions--------------------------------------------------------   
      if (vLines_Prior_Sessions_To_Show >0) 
         {   
         //Calculate bars per session----------------------------------------------------------        
              if (Period()==1) {bps = 1440;}   
         else if (Period()==5) {bps = 288;}  
         else if (Period()==15){bps = 96;}    
         else if (Period()==30){bps = 48;}
         else if (Period()==60){bps = 24;}   
      
         //Define bar starting prior sessions display------------------------------------------      
         shift = iBarShift(NULL,NULL,iTime(NULL,PERIOD_D1,0));               

         //Execute loop for bars per session X number of prior sessions to show----------------    
         for (i= shift; i<=(shift+(bps*(vLines_Prior_Sessions_To_Show))); i++)       
            {  
            h=TimeHour(Time[i]); 
            m=TimeMinute(Time[i]);   
            if (Day_Separators_On && h==0 && m==0)         
               {
               MF = MathFloor(Day_Start_Time);
               if((h == MF) && (m == 0))
                  {                     
                  OpenPrior(i, Label_Day_Separator, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level); 
                  }           
               }                    
            if(Trading_Start_On)
               {
               if(Trading_Start_Time+Server_GMT_Offset<0) {Trading_Start_Time=Trading_Start_Time+24;}                
               MF = MathFloor(Trading_Start_Time);
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_Trading_Start, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                    
               else if((Trading_Start_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Trading_Start, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Trading_Start_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))               
                  {
                  OpenPrior(i, Label_Trading_Start, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  } 
               }
            if(Trading_Stop_On)
               {
               if(Trading_Stop_Time+Server_GMT_Offset<0) {Trading_Stop_Time=Trading_Stop_Time+24;}                                                      
               MF = MathFloor(Trading_Stop_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_Trading_Stop, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Trading_Stop_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Trading_Stop, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Trading_Stop_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))               
                  {
                  OpenPrior(i, Label_Trading_Stop, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }
            if(Sydney_Open_On) 
               {               
               if(Sydney_Open_Time+Server_GMT_Offset<0) {Sydney_Open_Time=Sydney_Open_Time+24;}                 
               MF = MathFloor(Sydney_Open_Time);
               if((Period() > 30) && (h == MF + Server_GMT_Offset) && (m == 0))                
                  {
                  OpenPrior(i, Label_Sydney_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }               
               else if((Sydney_Open_Time != MF) && (h == MF + Server_GMT_Offset) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Sydney_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Sydney_Open_Time == MF) && (h == MF + Server_GMT_Offset) && (m == 0))
                  {
                  OpenPrior(i, Label_Sydney_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }                    
            if(Tokyo_Open_On) 
               {
               if(Tokyo_Open_Time+Server_GMT_Offset<0) {Tokyo_Open_Time=Tokyo_Open_Time+24;}                
               MF = MathFloor(Tokyo_Open_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset) && (m == 0))                
                  {
                  OpenPrior(i, Label_Tokyo_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }               
               else if((Tokyo_Open_Time != MF) && (h == MF + Server_GMT_Offset) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Tokyo_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Tokyo_Open_Time == MF) && (h == MF + Server_GMT_Offset) && (m == 0))
                  {
                  OpenPrior(i, Label_Tokyo_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }       
            if(Beijing_Open_On) 
               {
               if(Beijing_Open_Time+Server_GMT_Offset<0) {Beijing_Open_Time=Beijing_Open_Time+24;}                   
               MF = MathFloor(Beijing_Open_Time);
               if((Period() > 30) && (h == MF + Server_GMT_Offset) && (m == 0))                
                  {
                  OpenPrior(i, Label_Beijing_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }               
               else if((Beijing_Open_Time != MF) && (h == MF + Server_GMT_Offset) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Beijing_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Beijing_Open_Time == MF) && (h == MF + Server_GMT_Offset) && (m == 0))
                  {
                  OpenPrior(i, Label_Beijing_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }                               
            if(Moscow_Open_On) 
               {
               if(Moscow_Open_Time+Server_GMT_Offset<0) {Moscow_Open_Time=Moscow_Open_Time+24;}                  
               MF = MathFloor(Moscow_Open_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_Moscow_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }               
               else if((Moscow_Open_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Moscow_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Moscow_Open_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))
                  {
                  OpenPrior(i, Label_Moscow_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }                    
            if(FrankFurt_Open_On) 
               {
               if(Frankfurt_Open_Time+Server_GMT_Offset<0) {Frankfurt_Open_Time=Frankfurt_Open_Time+24;}                
               MF = MathFloor(Frankfurt_Open_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_Frankfurt_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }               
               else if((Frankfurt_Open_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Frankfurt_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Frankfurt_Open_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))
                  {
                  OpenPrior(i, Label_Frankfurt_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }          
               }                         
            if(London_Open_On)
               {
               if(London_Open_Time+Server_GMT_Offset<0) {London_Open_Time=London_Open_Time+24;}                
               MF = MathFloor(London_Open_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_London_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }               
               else if((London_Open_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_London_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((London_Open_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))
                  {
                  OpenPrior(i, Label_London_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }          
               }   
            if(NewYork_Open_On) 
               {
               if(NewYork_Open_Time+Server_GMT_Offset<0) {NewYork_Open_Time=NewYork_Open_Time+24;}                
               MF = MathFloor(NewYork_Open_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_NewYork_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                  
               else if((NewYork_Open_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_NewYork_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((NewYork_Open_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))               
                  {
                  OpenPrior(i, Label_NewYork_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }
            if(Stock_Exchange_Open_On) 
               {
               if(Stock_Exchange_Open_Time+Server_GMT_Offset<0) {Stock_Exchange_Open_Time=Stock_Exchange_Open_Time+24;}                
               MF = MathFloor(Stock_Exchange_Open_Time); 
               if((Period() > 30) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))                
                  {
                  OpenPrior(i, Label_Stock_Exchange_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                  
               else if((Stock_Exchange_Open_Time != MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 30)) 
                  {                    
                  OpenPrior(i, Label_Stock_Exchange_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }                   
               else if((Stock_Exchange_Open_Time == MF) && (h == MF + Server_GMT_Offset+ dst) && (m == 0))               
                  {
                  OpenPrior(i, Label_Stock_Exchange_Open, vLines_Color, 
                  vLines_Style_01234, vLines_Solid_Thickness, level);
                  }
               }               
            }//end prior sessions shift loop                            
         }//end prior sessions routine                          
      }//end vLines routine

   //Levels------------------------------------------------------------------------------------
   if (Levels_On && Period()<= Levels_Max_TF){
   
   //initializing line colors
   Line1_Color = Whole_Number_Color; 
   Line2_Color = Half_Number_Color;
   Line3_Color = Quarter_Number_Color;
   Line4_Color = Eighth_Number_Color;
  
   //define needed variables
   ssp1= Bid / Poin;  //bid w/o the decimal
   c1=ssp1%100; //last two significant digits   
   BL=c1; //last two significant digits of starting/reference bid line  
       
   //calculate line levels 
   for (i= -(100-(100-BL))-(100*NumberOfRanges); i<= (100-BL) + (100*NumberOfRanges); i++)        
      {
      ssp=ssp1+i; 
      c1=ssp%100;        
      //if((Period() <= Display_Max_TF) 
      //&& (c1==u1))
      if (c1==u1)
         {           
         linestyle = Line1_Style_01234;
         linecolor = Line1_Color;
         linewidth = Line1_SolidThickness;
         linelevel = ssp*Poin;
         }
      if ((Half_Number_Lines_On) 
      && (c1==u2))
         {              
         linestyle = Line2_Style_01234;
         linecolor = Line2_Color;
         linewidth = Line2_SolidThickness;
         linelevel = ssp*Poin;
         } 
      if ((Quarter_Number_Lines_On) 
      && ((c1==u3) || (c1==u4)))
         {           
         linestyle = Line3_Style_01234;
         linecolor = Line3_Color;
         linewidth = Line3_SolidThickness;
         linelevel = ssp*Poin;
         } 
      if ((Eighth_Number_Lines_On)  
      && ((c1==u5) || (c1==u6) || (c1==u7) || (c1==u8)))
         {            
         linestyle = Line4_Style_01234;
         linecolor = Line4_Color;
         linewidth = Line4_SolidThickness;
         linelevel = ssp*Poin;
         }
            
      //branch to subroutine to draw lines
      SetLevels(linelevel, linecolor, linestyle, linewidth);
      }}//End "for" loop, end "Levels" section
          
   //End of program computations---------------------------------------------------------------        
   return(0);
   }
    
//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine To Create Range Lines And Labels                                     |                                                                                  |
//+-------------------------------------------------------------------------------------------+
void DrawRange(string text, double level, color Color, int linestyle, int thickness)
   {
   //Lines=====================================================================================  
   //Name lines 
   text = StringTrimLeft(text);       
   rangeline  = "[Chart Panel]  " + text + " Line"; 

   //Define variables  
   a = linestyle; b = thickness; c =1; if (a==0)c=b;  
   
   //Where to Start lines
   if(Line_Selection_Number == 1) {startline = WindowFirstVisibleBar();}
   else if(Line_Selection_Number == 2 || Line_Selection_Number == 3) {startline = iTime(NULL, PERIOD_D1, 0);} //start at day separator
   else {startline  = Time[1];} //start at last closed candle                    
                                                                 
   //Where to Stop Lines
   if(Line_Selection_Number == 1 || Line_Selection_Number == 3 || Line_Selection_Number == 4) {stopline = Time[0]; F = true;}
   if(Line_Selection_Number == 2)     
      {
      T=24;
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + T + ":" + "60";                                        
      T = StrToTime(dt)+60; 
      stopline = T; F = false;          
      }    
                  
   //Draw lines 
   if (ObjectFind(rangeline) != 0){
      ObjectCreate(rangeline, OBJ_TREND, 0, startline, level, stopline, level);    
      ObjectSet(rangeline, OBJPROP_STYLE, linestyle);
      ObjectSet(rangeline, OBJPROP_COLOR, Color);
      ObjectSet(rangeline, OBJPROP_WIDTH, c);
      if (Subodinate_Range_Lines) {ObjectSet(rangeline, OBJPROP_BACK, true);}
      else {ObjectSet(rangeline, OBJPROP_BACK, false);} 
      ObjectSet(rangeline, OBJPROP_RAY, F);}
   else{
      ObjectMove(rangeline, 0, startline, level);
      ObjectMove(rangeline, 1, stopline, level);}
   
   //Labels===================================================================================
   //Exit if label not to be shown           
   if (StringSubstr(text,0,2) == "Rg" && !RangeLabels_Shown)return (-1);    
   else 
      {
      //Name label       
      rangelabel = "[Chart Panel]  " + text + " Label";  
         
      //Where to put labels                          
      if (RangeLabels_MaxRight) //start Max Right
         {
         spc="                                        "; //40
         startlabel = Time[0];    
         }
      else if (Line_Selection_Number == 1) //start mid chart
         {
         spc = ""; //00 
         startlabel = Time[WindowFirstVisibleBar()/2];
         }             
      else if (Line_Selection_Number == 2 || Line_Selection_Number == 3)            
         if (Time[WindowFirstVisibleBar()] < iTime(NULL, PERIOD_D1, 0)) //start at day separator
            {
            spc="       "; //07                           
            startlabel= iTime(NULL, PERIOD_D1, 0);  
            }          
         else //start start mid chart     
            {
         spc = ""; //00  
         startlabel = Time[WindowFirstVisibleBar()/2];                         
            }                        
      else if (Line_Selection_Number == 4) //start at current candle   
         {
         spc="         "; //09                   
         startlabel = Time[0];          
         }
                                     
      //Draw labels                            
      if (ObjectFind(rangelabel) != 0){
         ObjectCreate(rangelabel, OBJ_TEXT, 0, startlabel, level);     
         ObjectSetText(rangelabel, spc + text, RangeLabels_FontSize, RangeLabels_FontStyle, RangeLabels_Color);
         if (Subodinate_Range_Labels) {ObjectSet(rangelabel, OBJPROP_BACK, true);}
         else {ObjectSet(rangelabel, OBJPROP_BACK, false);}}        
      else {ObjectMove(rangelabel, 0, startlabel, level);}       
      }
   }
      
//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine To Compute Average Ranges                                            |                                                 
//+-------------------------------------------------------------------------------------------+ 
void Ranges (int period)
   {
   int ii, iii, x, xx;
   //Add ranges over period.  Count number of Sundays and exclude Sunday ranges.         
   ARg = 0; for(i=1; i<=Days_For_Range_Average; i++)
       {
       if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,i))!=0) {
       ARg = ARg + iHigh(NULL,PERIOD_D1,i)- iLow(NULL,PERIOD_D1,i);}
       else {x=x+1;}
       }
   //For number of Sundays, add additional days of range
   for(ii=i+1; ii<i+x+1; ii++) 
       {
       if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,ii))!=0) {       
       ARg = ARg + iHigh(NULL,PERIOD_D1,ii)- iLow(NULL,PERIOD_D1,ii);}
       else {xx=xx+1;}       
       }      
   //If a Sunday reduced added days above, add additional day of range
   for(iii=ii+1; iii<ii+xx+1; iii++) 
       {
       ARg = ARg + iHigh(NULL,PERIOD_D1,iii)- iLow(NULL,PERIOD_D1,iii);
       }                     
   //Return total of ranges for period (Sundays excluded/additional days subsituted)
   return (ARg);   
   }
   
//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine to Create Market Open Lines And Labels For Current Session           |          
//+-------------------------------------------------------------------------------------------+   
void OpenToday(string text, double T1, color Color, int linestyle, int thickness, double level) 
   {
   vline  = "[Chart Panel]  " + StringTrimLeft(text) + " Current Session Line";    
   vlabel = "[Chart Panel]  " + StringTrimLeft(text) + " Current Session Label";  

   a = linestyle;
   b = thickness;
   c =1; if (a==0)c=b;
   
   T1 = T1 + Server_GMT_Offset;
   double T2 = T1;
   if(T2<0) {T2 = T2 + 24;}
   int T4 = MathFloor(T2);
   double V;
   
   //Do datetime for start of current session vline 
   if(text == "day")  // && T1 == 0)        
      {
      Color = Day_Separators_Color; text="";
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,1440,0))+ "." +T1+":"+"00";             
      V = StrToTime(dt);  
      } 
      
   //Do datetime for end of current session vline
   else if(text == "day ") // && T1 == 24)               
      {
      Color = Day_Separators_Color; text="";
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,1440,0))+ "." +T1+":"+"00";        
      V = StrToTime(dt) + 86400;         
      }
    
   //Do all other vLines
   else //if(text != "day")
      {         
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,1440,0))+ "." +T4+":"+"00";           
      V = StrToTime(dt);      
      if(T2 != T4) {V = V + 1800;}  //add half hour 
      }  
                             
   if (ObjectFind(vline) != 0){
      if(!vLines_Extend_To_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, V, 0, V, 100);}  
      else {ObjectCreate(vline, OBJ_VLINE, 0, V, 0);}
      ObjectSet(vline, OBJPROP_STYLE, a);    
      ObjectSet(vline, OBJPROP_WIDTH, c);
      ObjectSet(vline, OBJPROP_COLOR, Color);
      ObjectSet(vline, OBJPROP_BACK, true); }
   else{
      ObjectMove(vline, 0, V, 0);} 

   if ((!text == "day") && (!text == "day ")) {                                
   if (ObjectFind(vlabel) != 0) {
      ObjectCreate (vlabel, OBJ_TEXT, 0, V, level);
      ObjectSetText(vlabel, text, vLabels_FontSize, vLabels_FontStyle, vLabels_Color);
      ObjectSet(vlabel, OBJPROP_BACK, false);}       
   else{ObjectMove(vlabel, 0, V, level);}  }
   }
   
//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine to Create Market Open Lines And Labels For Prior Sessions            |          
//+-------------------------------------------------------------------------------------------+   
void OpenPrior(int i, string text, color Color, int linestyle, int thickness, double level) 
   {   
   vline  = "[Chart Panel]  " + StringTrimLeft(text) + " Prior Session  " + (i-1) + " Line";    
   vlabel = "[Chart Panel]  " + StringTrimLeft(text) + " Prior Session  " + (i-1) + " Label";   

   if ((text == "day") || (text == "day ")) {Color = Day_Separators_Color; text = "";}
  
   a = linestyle;
   b = thickness;
   c =1; if (a==0)c=b;
                                 
   if (ObjectFind(vline) != 0){ 
      if(!vLines_Extend_To_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, Time[i], 0, Time[i], 100);} 
      else {ObjectCreate(vline, OBJ_VLINE, 0, Time[i], 0);}     
      ObjectSet(vline, OBJPROP_STYLE, a);    
      ObjectSet(vline, OBJPROP_WIDTH, c);
      ObjectSet(vline, OBJPROP_COLOR, Color);
      ObjectSet(vline, OBJPROP_BACK, true); }
   else{
      ObjectMove(vline, 0, Time[i], 0);}
      
   if ((!text == "day") && (!text == "day ")) {                                 
   if (ObjectFind(vlabel) != 0) {
      ObjectCreate (vlabel, OBJ_TEXT, 0, Time[i], level);
      ObjectSetText(vlabel, text, vLabels_FontSize, vLabels_FontStyle, vLabels_Color);
      ObjectSet(vlabel, OBJPROP_BACK, false);}       
   else{ObjectMove(vlabel, 0, Time[i], level);} }   
   }
   
//+-------------------------------------------------------------------------------------------+
//| Sub-routine to name and draw chart backgournd lines                                       |                                                                                  |
//+---------------------------------------------------------------------------------------- --+
void SetLevels(double level, color linecolor, int linestyle, int linewidth)
   {
   //create line label
   if (linewidth > 1) linestyle = 0;  
   linename = "[Chart Panel] " + StringTrimLeft(DoubleToStr(level,Digits)) + " Line";   
   // create or move line 
   if (ObjectFind(linename) != 0)
      {
      ObjectCreate(linename,  OBJ_TREND, 0, WindowFirstVisibleBar(), level, Time[0], level); 
      ObjectSet(linename, OBJPROP_STYLE, linestyle);     
      ObjectSet(linename, OBJPROP_COLOR, linecolor);
      ObjectSet(linename, OBJPROP_WIDTH, linewidth);
      ObjectSet(linename, OBJPROP_BACK, true); 
      }
   else 
      {
      ObjectMove(linename, 0, WindowFirstVisibleBar(), level);
      ObjectMove(linename, 1, Time[0], level);
      }
   }
   
// Start - Added by CaveMan to cater for DST
//+-------------------------------------------------------------------------------------------+
//| Calculate and return actual date given which Sunday of the month information              |
//|  --> Only US and EU DST is supported for the time being.                                  |
//+-------------------------------------------------------------------------------------------+
bool Adjust_for_DST()
   {
   // This definition section can be made into extern for user configuration but not neccesary.
   string Instructions1       = "Enter which Sunday of the month";
   string Instructions2       = "DST should start and end.";
   string Instructions3       = "1,2,3,4,5,6 = 1st, 2nd, 3rd, 4th, 5th and 6th";
   string Instructions4       = "7 = Last Sundy of the month";
  
   int us_dst_month_start     = 3;        // March
   int us_dst_start_sunday    = 2;        // 2nd Sunday of March start DST     
   string us_dst_time_start   = "02:00";  // DST starting time
   int us_dst_month_end       = 11;       // November   
   int us_dst_end_sunday      = 1;        // 1st Sunday of November end DST   
   string us_dst_time_end     = "02:00";  // DST ending time
 
   int eu_dst_month_start     = 3;        // March
   int eu_dst_start_sunday    = 7;        // Last Sunday of March start DST  
   string eu_dst_time_start   = "02:00";  // DST starting time
   int eu_dst_month_end       = 10;       // October
   int eu_dst_end_sunday      = 7;        // Last Sunday of October end DST   
   string eu_dst_time_end     = "02:00";  // DST ending time

   int aus_dst_month_start    = 4;        // April
   int aus_dst_start_sunday   = 1;        // 1st Sunday of April start DST   
   string aus_dst_time_start  = "02:00";  // DST starting time
   int aus_dst_month_end      = 10;       // October
   int aus_dst_end_sunday     = 1;        // 1st Sunday of October end DST   
   string aus_dst_time_end    = "02:00";  // DST ending time

   int nz_dst_month_start     = 4;        // April
   int nz_dst_start_sunday    = 1;        // 1st Sunday of April start DST  
   string nz_dst_time_start   = "02:00";  // DST starting time
   int nz_dst_month_end       = 9;        // September
   int nz_dst_end_sunday      = 7;        // Last Sunday of September end DST   
   string nz_dst_time_end     = "02:00";  // DST ending time

   datetime us_dst_start,   us_dst_end;   // dst start and end datetime
   datetime eu_dst_start,   eu_dst_end; 
   datetime aus_dst_start, aus_dst_end; 
   datetime nz_dst_start,   nz_dst_end; 
   datetime cus_dst_start, cus_dst_end; 
      
   int cus_dst_month_start   = Custom_DST_Start_Month;
   int cus_dst_start_sunday  = Custom_DST_Start_Sunday;
   string cus_dst_time_start = Custom_DST_Start_Time;
   int cus_dst_month_end     =  Custom_DST_End_Month;   
   int cus_dst_end_sunday    = Custom_DST_End_Sunday;
   string cus_dst_time_end   =  Custom_DST_End_Time;  
   
   //Print("US DST Start/End Date = ", TimeToStr(us_dst_start), " / ", TimeToStr(us_dst_end));
   //Print("EU DST Start/End Date = ", TimeToStr(eu_dst_start), " / ", TimeToStr(eu_dst_end));
      
   us_dst_start  = check_date(us_dst_start_sunday,  us_dst_month_start,  us_dst_time_start);
   us_dst_end    = check_date(us_dst_end_sunday,    us_dst_month_end,    us_dst_time_end);
   eu_dst_start  = check_date(eu_dst_start_sunday,  eu_dst_month_start,  eu_dst_time_start);
   eu_dst_end    = check_date(eu_dst_end_sunday,    eu_dst_month_end,    eu_dst_time_end);
   aus_dst_start = check_date(aus_dst_start_sunday, aus_dst_month_start, aus_dst_time_start);
   aus_dst_end   = check_date(aus_dst_end_sunday,   aus_dst_month_end,   aus_dst_time_end);
   nz_dst_start  = check_date(nz_dst_start_sunday,  nz_dst_month_start,  nz_dst_time_start);
   nz_dst_end    = check_date(nz_dst_end_sunday,    nz_dst_month_end,    nz_dst_time_end);
   cus_dst_start = check_date(cus_dst_start_sunday, cus_dst_month_start, cus_dst_time_start);
   cus_dst_end   = check_date(cus_dst_end_sunday,   cus_dst_month_end,   cus_dst_time_end);
      
   if (DST_Zone == 1 && iTime(NULL,0,0) > us_dst_start && iTime(NULL,0,0) < us_dst_end)
      {
      Server_GMT_Offset = DST_Server_GMT_Offset; 
      return(true);
      }
   else if (DST_Zone == 2 && iTime(NULL,0,0) > eu_dst_start && iTime(NULL,0,0) < eu_dst_end)
      { 
      Server_GMT_Offset = DST_Server_GMT_Offset;      
      return(true);
      }
   else if (DST_Zone == 3 && iTime(NULL,0,0) > aus_dst_start && iTime(NULL,0,0) < aus_dst_end)
      {
      Server_GMT_Offset = DST_Server_GMT_Offset; 
      return(true);
      }
   else if (DST_Zone == 4 && iTime(NULL,0,0) > nz_dst_start && iTime(NULL,0,0) < nz_dst_end)
      {
      Server_GMT_Offset = DST_Server_GMT_Offset; 
      return(true);
      }
   else if (DST_Zone == 5 && iTime(NULL,0,0) > cus_dst_start && iTime(NULL,0,0) < cus_dst_end)
      {
      Server_GMT_Offset = DST_Server_GMT_Offset; 
      return(true);
      }
   else
      {
      Server_GMT_Offset = Non_DST_Server_GMT_Offset; 
      return(false);
      }  
   }              

//+-------------------------------------------------------------------------------------------+
//| Calculate and return actual date given which Sunday of the month information              |
//+-------------------------------------------------------------------------------------------+
datetime check_date(int which_sunday, int which_month, string what_time)
{
   string start_date;
   int day_of_week, start_day;

   if (which_sunday == 7)  // Last Sunday of the month
      {
         for (int i = 31; i > 20; i--)
            {
               start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + i;
               day_of_week = TimeDayOfWeek(StrToTime(start_date));
               if (day_of_week == 0) // equals to Sunday
                  {
                     start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + i + " " + what_time;
                     break;
                   }
             } 
      }
   else // other Sundays of the month besides last Sunday of the month
      {
         start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + "1";
         day_of_week = TimeDayOfWeek(StrToTime(start_date));
   
         if (day_of_week == 0) start_day = 1 + 7*(which_sunday - 1);
         else start_day = 1 + (7 - day_of_week) + 7 * (which_sunday - 1);
   
         start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + start_day + " " + what_time;
      }
      
   return(StrToTime(start_date));
}
// End - Added by CaveMan to cater for DST

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+      

