//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                SonicR Clock Panel.mq4                                     |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+
#property copyright "Copyright @ 2010 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview:

This is a version of the P4L Clock acknowledged below, trimmed down and modified for SonicR.
It is designed to be used with the SonicR Chart Panel indicator that places a data panel in 
the upper left corner of the chart.  This indicator creates an extension panel to display 
the clock.  

This extension panel comes in three sizes (sm, med, lg).  Five time zones are displayed on 
the small and medium panels.  Seven are displayed on the large panel.  If you select more 
than this, the clock display will overrun the clock panel background.  

You can select default colors for white and black charts, which override the External Inputs
colors.  In this manner, you can use the external input colors to store your own custom 
colors, and still have immediate access to colors for white and black charts. 

You can show "Broker" or "Local" designations in the labels of other locations selected for
display.  By doing so, you will not need the Broker and Local display lines; important since 
lines are limited.  To add designations (B, (L), or (BL) to the locations you choose, you 
must find out which locations match clock times with Broker/Local.  To do this, set this 
indicator to show all 21 choices and view the results.  It is ok to overrun the background 
for this test.  Note locations matching with Broker and Local times.  Next, from locations 
you intend to display, type into Part_3 the names of any matching locations.  If your 
"Broker" and "Local" times are the same, then type in twice the single name of any matching
location.  Once this is done, reset your selections for display.  If your selections include 
locations that match times with Broker/Local, you will not need to include those lines also.  

You can display only 7 locations on the large panel, and only 5 on the small/medium panels.
To make your selections, imput "0" for any location that is not to be displayed.  Enter "1" 
for those two additional locations that are to be displayed only on the one large panel.  
Enter "2" for those five locations that are to be displayed on the two small/medium panels.

                                                                 - Traderathome, 12-01-2010 
----------------------------------------------------------------------------------------------
Acknowledgements:

Kent - "pips4life" //P4L Clock.mq4 (v2.7), a rewrite of versions of Clock.mq4
       "pips4life, a user at forexfactory.com"
----------------------------------------------------------------------------------------------                                                                  
Suggested Settings:               White Chart           Black Chart   

Location_MktClosed                Black                 C'38,166,38'
Clock_MktClosed                   Black                 C'0,122,244'
Location_MktOpen                  Crimson               Red 
Clock_MktOpen                     MediumBlue            DarkOrange 
Background                        C'233,233,252'        C'25,25,25'                                                                  
---------------------------------------------------------------------------------------------*/  


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+                                                               
#property indicator_chart_window     

#include <stdlib.mqh>
//See function documentation at http://msdn.microsoft.com/en-us/library/ms725473(VS.85).aspx
#import "kernel32.dll"
void GetLocalTime(int& LocalTimeArray[]);
void GetSystemTime(int& systemTimeArray[]);
int  GetTimeZoneInformation(int& LocalTZInfoArray[]);
bool SystemTimeToTzSpecificLocalTime(int& targetTZinfoArray[], 
     int& systemTimeArray[], int& targetTimeArray[]);
#import

//Global External Inputs-----------------------------------------------------------------------
extern string   Part_1                          = "Indicator Display Controls:";
extern bool     Indicator_On                    = false;
extern int      Apply_Panel_Sm_Med_Lg_123       = 2;
extern bool     Apply_Default_Color_Selections  = true;
extern bool     Apply_On_White_vs_Black_Chart   = true;         
extern color    Location_MktClosed              = Black;        
extern color    Clock_MktClosed                 = Black;  
extern color    Location_MktOpen                = Crimson;     
extern color    Clock_MktOpen                   = MediumBlue;  
extern color    Background                      = C'233,233,252';
extern bool     Display_Times_With_AMPM         = true;  
extern int      Indicator_Max_Display_TF        = 43200;  
extern string   TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string   TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string   __                              = "";
extern string   Part_2                          = "Choosing Locations to Display:";
extern string   Note_1                          = "Select 7 maximum from list,";
extern string   Note_2                          = "5 for both size panels, and";
extern string   Note_3                          = "2 more for the full size panel.";
extern string   Note_4                          = "0 = display off.";
extern string   Note_5                          = "1 = Display only on full size panel.";
extern string   Note_6                          = "2 = Display on both size panels.";
extern int      Broker                          = 0;
extern int      Local                           = 0; //Your coomputer's time.
extern int      Auckland                        = 0;
extern int      Sydney                          = 1;
extern int      Tokyo                           = 2;
extern int      China                           = 1;
extern int      Jakarta                         = 0;
extern int      India                           = 0;
extern int      Dubai                           = 0;
extern int      Moscow                          = 0;
extern int      Israel                          = 0;
extern int      Helsinki                        = 2;
extern int      Berlin                          = 0;
extern int 	    London                          = 2;
extern int  	 GMT                             = 2; //No GMT change w/DST. Winter London=GMT+1, Summer London=GMT.
extern int      Brazil                          = 0;
extern int  	 NewYork                         = 2;
extern int      Central                         = 0;
extern int      Mexico                          = 0;
extern int      Mountain                        = 0;
extern int  	 Pacific                         = 0;

extern string   ___                             = "";
extern string   Part_3                          = "Adding  (B), (L)  and  (B,L) Labels:";
extern string   Note_7                          = "Enter location names exactly as";
extern string   Note_8                          = "above, otherwise leave blank.";
extern string   Location_Matching_Broker        = "Helsinki";
extern string   Location_Matching_Local         = "NewYork";
     
//Global Buffer, Variables, Etc.---------------------------------------------------------------
bool     FLAG_deinit;

int      obj_total,k,z,top, time, place, offset;
string   item1 = "[MPClock] _Box1";  
string   item2 = "[MPClock] _Box2"; 

string   BrokerTZ, LocalTZ, Aucklandp, Sydneyp, Tokyop, Chinap, Jakartap, Indiap, Dubaip, Moscowp, Israelp, Helsinkip,
         Berlinp, Londonp, UTCp, Brazilp, NYp, Centralp, Mexicop, Mountainp, Pacificp, name;
       
int      AucklandTZInfoArray[43],SydneyTZInfoArray[43],TokyoTZInfoArray[43],ChinaTZInfoArray[43],
         JakartaTZInfoArray[43],IndiaTZInfoArray[43],DubaiTZInfoArray[43],MoscowTZInfoArray[43],
         IsraelTZInfoArray[43],HelsinkiTZInfoArray[43],BerlinTZInfoArray[43],LondonTZInfoArray[43],
         BrazilTZInfoArray[43],NewYorkTZInfoArray[43],CentralTZInfoArray[43],MexicoTZInfoArray[43],
         MountainTZInfoArray[43],PacificTZInfoArray[43],LocalTZInfoArray[43]; 
          
int      LocalTimeArray[4],systemTimeArray[4],AucklandTimeArray[4],SydneyTimeArray[4],TokyoTimeArray[4],
         ChinaTimeArray[4],JakartaTimeArray[4],IndiaTimeArray[4],DubaiTimeArray[4],MoscowTimeArray[4],
         IsraelTimeArray[4],HelsinkiTimeArray[4],BerlinTimeArray[4],LondonTimeArray[4],BrazilTimeArray[4],
         NewYorkTimeArray[4],CentralTimeArray[4],MexicoTimeArray[4],MountainTimeArray[4],PacificTimeArray[4];
         
datetime Local_Time,BrokerTime,AucklandDT,SydneyDT,TokyoDT,ChinaDT,JakartaDT,IndiaDT,DubaiDT,MoscowDT,
         IsraelDT,HelsinkiDT,BerlinDT,LondonDT,UTC,BrazilDT,NewYorkDT,CentralDT,MexicoDT,MountainDT,PacificDT;

string   Locals,Brokers,Aucklands,Sydneys,Tokyos,Chinas,Jakartas,Indias,Dubais,Moscows,Israels,Helsinkis,
         Berlins,Londons,UTCs,Brazils,NewYorks,Centrals,Mexicos,Mountains,Pacifics;
   
int      LocalOpenHour        = 8;       
int      LocalCloseHour       = 17;
int      SydneyLocalOpenHour  = 7;          
int      SydneyLocalCloseHour = 16;  
int      ClockFontSize,TimezoneFontSize,LineSpacing,p1,p2,p3,p4,p5,p6;
string   G;
string   FontName = "Arial";

//+-------------------------------------------------------------------------------------------+
//| Custom Indicator Initialization                                                           |
//+-------------------------------------------------------------------------------------------+
int init()
  {   
   //With the first DLL call below, the program will exit (and stop) automatically after one alert. 
   if ( !IsDllsAllowed() ) 
      {
      Alert( WindowExpertName(),": Error! Check 'Allow DLL Imports' in indicator Common Tab.");   
      }
   
   FLAG_deinit  = false;
             
   GetAllTimeZoneInfo(AucklandTZInfoArray, SydneyTZInfoArray, TokyoTZInfoArray, ChinaTZInfoArray, 
      JakartaTZInfoArray, IndiaTZInfoArray, DubaiTZInfoArray, MoscowTZInfoArray, IsraelTZInfoArray, 
      HelsinkiTZInfoArray, BerlinTZInfoArray, LondonTZInfoArray, BrazilTZInfoArray, NewYorkTZInfoArray, 
      CentralTZInfoArray, MexicoTZInfoArray, MountainTZInfoArray, PacificTZInfoArray); //, LocalTZInfoArray);  
      
      if (Apply_Panel_Sm_Med_Lg_123 == 1)
         {
         ClockFontSize = 8; 
         TimezoneFontSize = 8;         
         LineSpacing = 11;
         G= "gg";
         p1= 48; p2= 145; p3= 150; p4= 2; p5= 153; p6= 194;
         if (Display_Times_With_AMPM) {p2= 145; p3= 170; p6= 215;}
         }              
      else if (Apply_Panel_Sm_Med_Lg_123 == 2)
         {
         ClockFontSize = 8; 
         TimezoneFontSize = 8;          
         LineSpacing = 12;
         G= "gg";
         p1= 56; p2= 135; p3= 140; p4= 6; p5= 156; p6= 196;  //p3=210
         if (Display_Times_With_AMPM) {p2= 135; p3= 160; p6= 216;}  //p3=230
         }                         
      else if (Apply_Panel_Sm_Med_Lg_123 == 3)
         {
         ClockFontSize = 8; 
         TimezoneFontSize = 8;        
         LineSpacing = 12;
         G= "g"; 
         p1= 69; p2= 150; p3= 200; p4= 3; p5= 166; p6= 210; 
         if (Display_Times_With_AMPM) {p2= 150; p3= 225; p6= 229;}
         }
             
   if (Apply_Default_Color_Selections)
      {         
      if (Apply_On_White_vs_Black_Chart)
         {
         Location_MktClosed            = Black;        
         Clock_MktClosed               = Black;  
         Location_MktOpen              = Crimson;     
         Clock_MktOpen                 = MediumBlue;  
         Background                    = C'233,233,252';              
         } 
      else
         {
         Location_MktClosed            = C'36,151,36';        
         Clock_MktClosed               = C'0,122,244';  
         Location_MktOpen              = Red;     
         Clock_MktOpen                 = DarkOrange;  
         Background                    = C'25,25,25';               
         }
      }          
           
   //Background 
	if (ObjectFind(item1) == -1){
      ObjectCreate(item1, OBJ_LABEL, 0, 0, 0, 0, 0);}         
   ObjectSetText(item1, "g", p1, "Webdings");      
   ObjectSet(item1, OBJPROP_CORNER, 0);
   ObjectSet(item1, OBJPROP_BACK, false);
   ObjectSet(item1, OBJPROP_XDISTANCE, p2);
   ObjectSet(item1, OBJPROP_YDISTANCE, 0 );    
   ObjectSet(item1, OBJPROP_COLOR, Background); 
         
	if (ObjectFind(item2) == -1){
      ObjectCreate(item2, OBJ_LABEL, 0, 0, 0, 0, 0);}         
   ObjectSetText(item2, G, p1, "Webdings");      
   ObjectSet(item2, OBJPROP_CORNER, 0);
   ObjectSet(item2, OBJPROP_BACK, false);
   ObjectSet(item2, OBJPROP_XDISTANCE, p3);
   ObjectSet(item2, OBJPROP_YDISTANCE, 0 );    
   ObjectSet(item2, OBJPROP_COLOR, Background); 
          
   top    = p4;     //Starting Vertical Offset Pixels for all text
   offset = 0;     //Start first line of text at "top" without added linespacing  
   time   = p5;   //Horizontal Offset Pixels for clock/time      
   place  = p6;   //Horizontal Offset Pixels for location/place 
   
   BrokerTZ = Location_Matching_Broker;
   LocalTZ  = Location_Matching_Local;   
   
  	// Begin world timezones, listed IN ORDER!
   if((Broker == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Broker == 1)))    	  	
      {
      ObjectMakeLabel( "[MPClock] Brokert", time, top+offset );
      ObjectMakeLabel( "[MPClock] Brokerl", place, top+offset );
     	offset+=LineSpacing;
      } 
   if((Auckland == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Auckland == 1)))         	
      { 
      if((BrokerTZ == "Auckland") && (LocalTZ == "Auckland")) {Aucklandp= "Auckland  (B,L)";}
      else if(BrokerTZ == "Auckland") {Aucklandp= "Auckland  (B)";}
      else if(LocalTZ == "Auckland") {Aucklandp= "Auckland  (L)";}               
      else {Aucklandp= "Auckland";}             	     	       	        
   	ObjectMakeLabel( "[MPClock] Aucklandt", time, top+offset );         
   	ObjectMakeLabel( "[MPClock] Aucklandl", place, top+offset );
   	offset+=LineSpacing;    
      }
   if((Sydney == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Sydney == 1)))      
      {
      if((BrokerTZ == "Sydney") && (LocalTZ == "Sydney")) {Sydneyp= "Sydney  (B,L)";}    
      else if(BrokerTZ == "Sydney") {Sydneyp= "Sydney  (B)";}
      else if(LocalTZ == "Sydney") {Sydneyp= "Sydney  (L)";}               
      else {Sydneyp= "Sydney";}           
   	ObjectMakeLabel( "[MPClock] Sydneyt", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Sydneyl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Tokyo == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Tokyo == 1)))
      {
      if((BrokerTZ == "Tokyo") && (LocalTZ == "Tokyo")) {Tokyop= "Tokyo  (B,L)";}
      else if(BrokerTZ == "Tokyo") {Tokyop= "Tokyo  (B)";}
      else if(LocalTZ == "Tokyo") {Tokyop= "Tokyo  (L)";}               
      else {Tokyop= "Tokyo";} 
   	ObjectMakeLabel( "[MPClock] Tokyot", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Tokyol", place, top+offset );
   	offset+=LineSpacing;
      }
   if((China == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (China == 1)))        
      {
      if((BrokerTZ == "Beijing") && (LocalTZ == "Beijing")) {Chinap= "Beijing  (B,L)";}   
      else if(BrokerTZ == "Beijing") {Chinap= "Beijing  (B)";}
      else if(LocalTZ == "Beijing") {Chinap= "Beijing  (L)";}                  
      else {Chinap= "Beijing";} 
   	ObjectMakeLabel( "[MPClock] Chinat", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Chinal", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Jakarta == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Jakarta == 1)))        
      {
      if((BrokerTZ == "Jakarta") && (LocalTZ == "Jakarta")) {Jakartap= "Jakarta  (B,L)";}         
      else if(BrokerTZ == "Jakarta") {Jakartap= "Jakarta  (B)";}
      else if(LocalTZ == "Jakarta") {Jakartap= "Jakarta  (L)";}        
      else {Jakartap= "Jakarta";} 
   	ObjectMakeLabel( "[MPClock] Jakartat", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Jakartal", place, top+offset );
   	offset+=LineSpacing;
      }
   if((India == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (India == 1)))       
      {
      if((BrokerTZ == "India") && (LocalTZ == "India")) {Indiap= "India  (B,L)";}         
      else if(BrokerTZ == "India") {Indiap= "India  (B)";}
      else if(LocalTZ == "India") {Indiap= "India  (L)";}          
   	else {Indiap= "India";} 
   	ObjectMakeLabel( "[MPClock] Indiat", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Indial", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Dubai == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Dubai == 1)))        
      {
      if((BrokerTZ == "Dubai") && (LocalTZ == "Dubai")) {Dubaip= "Dubai  (B,L)";}           
      else if(BrokerTZ == "Dubai") {Dubaip= "Dubai  (B)";}
      else if(LocalTZ == "Dubai") {Dubaip= "Dubai  (L)";}         
   	else {Dubaip= "Dubai";}
   	ObjectMakeLabel( "[MPClock] Dubait", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Dubail", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Moscow == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Moscow == 1)))       
      {
      if((BrokerTZ == "Moscow") && (LocalTZ == "Moscow")) {Moscowp= "Moscow  (B,L)";}       
      else if(BrokerTZ == "Moscow") {Moscowp= "Moscow  (B)";}
      else if(LocalTZ == "Moscow") {Moscowp= "Moscow  (L)";}      
      else {Moscowp= "Moscow";} 
   	ObjectMakeLabel( "[MPClock] Moscowt", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Moscowl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Israel == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Israel == 1)))        
      {
      if((BrokerTZ == "Israel") && (LocalTZ == "Israel")) {Israelp= "Israel (B,L)";}         
      else if(BrokerTZ == "Israel") {Israelp= "Israel  (B)";}
      else if(LocalTZ == "Israel") {Israelp= "Israel  (L)";}        
      else {Israelp= "Israel";}
   	ObjectMakeLabel( "[MPClock] Israelt", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Israell", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Helsinki == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Helsinki == 1)))        
      {
      if((BrokerTZ == "Helsinki") && (LocalTZ == "Helsinki")) {Helsinkip= "Helsinki  (B,L)";}
      else if(BrokerTZ == "Helsinki") {Helsinkip= "Helsinki  (B)";}
      else if(LocalTZ == "Helsinki") {Helsinkip= "Helsinki  (L)";}             
      else {Helsinkip= "Helsinki";} 	                             	       
   	ObjectMakeLabel( "[MPClock] Helsinkit", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Helsinkil", place, top+offset );
   	offset+=LineSpacing; 	
      }
   if((Berlin == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Berlin == 1)))        
      {
      if((BrokerTZ == "Berlin") && (LocalTZ == "Berlin")) {Berlinp= "Berlin  (B,L)";}	           
      else if(BrokerTZ == "Berlin") {Berlinp= "Berlin  (B)";}
      else if(LocalTZ == "Berlin") {Berlinp= "Berlin  (L)";}          
      else {Berlinp= "Berlin";} 
   	ObjectMakeLabel( "[MPClock] Berlint", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Berlinl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((London == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (London == 1)))
      {
      if((BrokerTZ == "London") && (LocalTZ == "London")) {Londonp= "London  (B,L)";}         
      else if(BrokerTZ == "London") {Londonp= "London  (B)";}
      else if(LocalTZ == "London") {Londonp= "London  (L)";}         
      else {Londonp= "London";} 
	   ObjectMakeLabel( "[MPClock] Londont", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Londonl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((GMT == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (GMT == 1)))        
      {
      if((BrokerTZ == "GMT") && (LocalTZ == "GMT")) {UTCp= "GMT  (B,L)";}        
      else if(BrokerTZ == "GMT") {UTCp= "GMT  (B)";}
      else if(LocalTZ == "GMT") {UTCp= "GMT  (L)";}        
      else {UTCp= "GMT";} 
   	ObjectMakeLabel( "[MPClock] utct", time, top+offset );
   	ObjectMakeLabel( "[MPClock] utcl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Brazil == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Brazil == 1)))       
      {
      if((BrokerTZ == "Brazil") && (LocalTZ == "Brazil")) {Brazilp= "Brazil  (B,L)";}       
      else if(BrokerTZ == "Brazil") {Brazilp= "Brazil  (B)";}
      else if(LocalTZ == "Brazil") {Brazilp= "Brazil  (L)";}        
      else {Brazilp= "Brazil";} 
   	ObjectMakeLabel( "[MPClock] Brazilt", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Brazill", place, top+offset );
   	offset+=LineSpacing;
      }
   if((NewYork == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (NewYork == 1)))        
      {
      if((BrokerTZ == "NewYork") && (LocalTZ == "NewYork")) {NYp= "NewYork  (B,L)";}
      else if(BrokerTZ == "NewYork") {NYp= "NewYork  (B)";}
      else if(LocalTZ == "NewYork") {NYp= "NewYork  (L)";}                	   
      else {NYp= "NewYork";}  
	   ObjectMakeLabel( "[MPClock] NewYorkt", time, top+offset );
   	ObjectMakeLabel( "[MPClock] NewYorkl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Central == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Central == 1)))        
      {
      if((BrokerTZ == "Central") && (LocalTZ == "Central")) {Centralp= "Central  (B,L)";}	         
      else if(BrokerTZ == "Central") {Centralp= "Central  (B)";}
      else if(LocalTZ == "Central") {Centralp= "Central  (L)";}        
      else {Centralp= "Central";}
   	ObjectMakeLabel( "[MPClock] Centralt", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Centrall", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Mexico == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Mexico == 1)))        
      {
      if((BrokerTZ == "Mexico") && (LocalTZ == "Mexico")) {Mexicop= "Mexico  (B,L)";}       
      else if(BrokerTZ == "Mexico") {Mexicop= "Mexico  (B)";}
      else if(LocalTZ == "Mexico") {Mexicop= "Mexico  (L)";}         
      else {Mexicop= "Mexico";} 
   	ObjectMakeLabel( "[MPClock] Mexicot", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Mexicol", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Mountain == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Mountain == 1)))        
      {
      if((BrokerTZ == "Mountain") && (LocalTZ == "Mountain")) {Mountainp= "Mountain  (B,L)";}       
      else if(BrokerTZ == "Mountain") {Mountainp= "Mountain  (B)";}
      else if(LocalTZ == "Mountain") {Mountainp= "Mountain  (L)";}         
      else {Mountainp= "Mountain";} 
   	ObjectMakeLabel( "[MPClock] Mountaint", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Mountainl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Pacific == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Pacific == 1)))        
      {
      if((BrokerTZ == "Pacific") && (LocalTZ == "Pacific")) {Pacificp= "Pacific  (B,L)";}        
      else if(BrokerTZ == "Pacific") {Pacificp= "Pacific  (B)";}
      else if(LocalTZ == "Pacific") {Pacificp= "Pacific  (L)";}        
      else {Pacificp= "Pacific";} 
	   ObjectMakeLabel( "[MPClock] Pacifict", time, top+offset );
   	ObjectMakeLabel( "[MPClock] Pacificl", place, top+offset );
   	offset+=LineSpacing;
      }
   if((Local == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Local == 1)))        
      {     
      ObjectMakeLabel( "[MPClock] Localt", time, top+offset );
      ObjectMakeLabel( "[MPClock] Locall", place, top+offset );
      offset+=LineSpacing;
      }
                             
   return(0);
   } 

//+-------------------------------------------------------------------------------------------+
//| Custom Indicator Deinitialization                                                         |
//+-------------------------------------------------------------------------------------------+
int deinit()
   {
   obj_total= ObjectsTotal();  
   for (k= obj_total; k>=0; k--)
      {
      name= ObjectName(k); 
      if (StringSubstr(name,0,9)=="[MPClock]"){ObjectDelete(name);}
      }
   return(0);
   }
   
//+-------------------------------------------------------------------------------------------+
//| Custom Indicator Start -- Runs with each new tick                                         |
//+-------------------------------------------------------------------------------------------+
int start()
  {
   //If Indicator is "Off" or chart is out of range deinitialize only once, not every tick-----
   if((!Indicator_On) || (Period() > Indicator_Max_Display_TF)) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }  
      
   //Otherwise, just proceed-------------------------------------------------------------------        
   FLAG_deinit = false; 
   
   GetSystemTime(systemTimeArray);
   UTC = TimeArrayToTime(systemTimeArray); 
        
   BrokerTime = TimeCurrent();  
      
   SystemTimeToTzSpecificLocalTime(AucklandTZInfoArray, systemTimeArray, AucklandTimeArray);
   AucklandDT = TimeArrayToTime(AucklandTimeArray);
      
   SystemTimeToTzSpecificLocalTime(SydneyTZInfoArray, systemTimeArray, SydneyTimeArray);
   SydneyDT   = TimeArrayToTime(SydneyTimeArray);
   
   SystemTimeToTzSpecificLocalTime(TokyoTZInfoArray, systemTimeArray, TokyoTimeArray);
   TokyoDT    = TimeArrayToTime(TokyoTimeArray);
      
   SystemTimeToTzSpecificLocalTime(ChinaTZInfoArray, systemTimeArray, ChinaTimeArray);
   ChinaDT    = TimeArrayToTime(ChinaTimeArray);
      
   SystemTimeToTzSpecificLocalTime(JakartaTZInfoArray, systemTimeArray, JakartaTimeArray);
   JakartaDT  = TimeArrayToTime(JakartaTimeArray);
      
   SystemTimeToTzSpecificLocalTime(IndiaTZInfoArray, systemTimeArray, IndiaTimeArray);
   IndiaDT    = TimeArrayToTime(IndiaTimeArray);
      
   SystemTimeToTzSpecificLocalTime(DubaiTZInfoArray, systemTimeArray, DubaiTimeArray);
   DubaiDT    = TimeArrayToTime(DubaiTimeArray);
      
   SystemTimeToTzSpecificLocalTime(MoscowTZInfoArray, systemTimeArray, MoscowTimeArray);
   MoscowDT   = TimeArrayToTime(MoscowTimeArray);
      
   SystemTimeToTzSpecificLocalTime(IsraelTZInfoArray, systemTimeArray, IsraelTimeArray);
   IsraelDT   = TimeArrayToTime(IsraelTimeArray);
      
   SystemTimeToTzSpecificLocalTime(HelsinkiTZInfoArray, systemTimeArray, HelsinkiTimeArray);
   HelsinkiDT = TimeArrayToTime(HelsinkiTimeArray);  
      
   SystemTimeToTzSpecificLocalTime(BerlinTZInfoArray, systemTimeArray, BerlinTimeArray);
   BerlinDT   = TimeArrayToTime(BerlinTimeArray);
      
   SystemTimeToTzSpecificLocalTime(LondonTZInfoArray, systemTimeArray, LondonTimeArray);
   LondonDT   = TimeArrayToTime(LondonTimeArray);
     
   SystemTimeToTzSpecificLocalTime(BrazilTZInfoArray, systemTimeArray, BrazilTimeArray);
   BrazilDT   = TimeArrayToTime(BrazilTimeArray);
   
   SystemTimeToTzSpecificLocalTime(NewYorkTZInfoArray, systemTimeArray, NewYorkTimeArray);
   NewYorkDT  = TimeArrayToTime(NewYorkTimeArray);
   
   SystemTimeToTzSpecificLocalTime(CentralTZInfoArray, systemTimeArray, CentralTimeArray);
   CentralDT  = TimeArrayToTime(CentralTimeArray);
      
   SystemTimeToTzSpecificLocalTime(MexicoTZInfoArray, systemTimeArray, MexicoTimeArray);
   MexicoDT   = TimeArrayToTime(MexicoTimeArray);
      
   SystemTimeToTzSpecificLocalTime(MountainTZInfoArray, systemTimeArray, MountainTimeArray);
   MountainDT = TimeArrayToTime(MountainTimeArray);
      
   SystemTimeToTzSpecificLocalTime(PacificTZInfoArray, systemTimeArray, PacificTimeArray);
   PacificDT  = TimeArrayToTime(PacificTimeArray);   
   
   GetLocalTime(LocalTimeArray);
   Local_Time = TimeArrayToTime(LocalTimeArray);   
   
   Brokers   = TimeToString( TimeCurrent() );   
   Aucklands = TimeToString( AucklandDT  );
   Sydneys   = TimeToString( SydneyDT  );
   Tokyos    = TimeToString( TokyoDT  );
   Chinas    = TimeToString( ChinaDT  );
   Jakartas  = TimeToString( JakartaDT  );
   Indias    = TimeToString( IndiaDT  );
   Dubais    = TimeToString( DubaiDT  );
   Moscows   = TimeToString( MoscowDT  );
   Israels   = TimeToString( IsraelDT  );
   Helsinkis = TimeToString( HelsinkiDT  );
   Berlins   = TimeToString( BerlinDT  );
   Londons   = TimeToString( LondonDT  );
   UTCs      = TimeToString( UTC );
   Brazils   = TimeToString( BrazilDT  );
   NewYorks  = TimeToString( NewYorkDT  );
   Centrals  = TimeToString( CentralDT  );
   Mexicos   = TimeToString( MexicoDT  );
   Mountains = TimeToString( MountainDT  );
   Pacifics  = TimeToString( PacificDT  );
   Locals    = TimeToString( Local_Time  ); 
        
   if((Broker == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Broker == 1)))  
      {
	   ObjectSetText( "[MPClock] Brokert", Brokers, ClockFontSize, FontName, Clock_MktClosed );
      ObjectSetText( "[MPClock] Brokerl", "Broker", TimezoneFontSize, FontName, Location_MktClosed );
      }
   if((Auckland == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Auckland == 1)))                   
      {
   	if (TimeDayOfWeek(AucklandDT) != 0 && TimeDayOfWeek(AucklandDT) != 6 
   	&& TimeHour(AucklandDT) >= LocalOpenHour && TimeHour(AucklandDT) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktOpen );      	
   	   ObjectSetText( "[MPClock] Aucklandl", Aucklandp, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Aucklandl", Aucklandp, TimezoneFontSize, FontName, Location_MktClosed );
      	}
      }
   if((Sydney == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Sydney == 1)))        
      {      	     	      	     
   	if (TimeDayOfWeek(SydneyDT) != 0 && TimeDayOfWeek(SydneyDT) != 6 
   	&& TimeHour(SydneyDT) >= SydneyLocalOpenHour && TimeHour(SydneyDT) < SydneyLocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktOpen );   	   
   	   ObjectSetText( "[MPClock] Sydneyl", Sydneyp, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Sydneyl", Sydneyp, TimezoneFontSize, FontName, Location_MktClosed );
      	}
      }
   if((Tokyo == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Tokyo == 1)))           
      {         	      	         	         
   	if (TimeDayOfWeek(TokyoDT) != 0 && TimeDayOfWeek(TokyoDT) != 6 
   	&& TimeHour(TokyoDT) >= LocalOpenHour && TimeHour(TokyoDT) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktOpen );   	   
   	   ObjectSetText( "[MPClock] Tokyol", Tokyop, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Tokyol", Tokyop, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((China == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (China == 1)))        
      {        	     	           
   	if (TimeDayOfWeek(ChinaDT) != 0 && TimeDayOfWeek(ChinaDT) != 6 
   	&& TimeHour(ChinaDT) >= LocalOpenHour && TimeHour(ChinaDT) < LocalCloseHour)
   	   {
    	   ObjectSetText( "[MPClock] Chinat", Chinas, ClockFontSize, FontName, Clock_MktOpen );  	   
   	   ObjectSetText( "[MPClock] Chinal", Chinap, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Chinat", Chinas, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Chinal", Chinap, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((Jakarta == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Jakarta == 1)))   
      {         	     	         	     
   	if (TimeDayOfWeek(JakartaDT) != 0 && TimeDayOfWeek(JakartaDT) != 6
   	&& TimeHour(JakartaDT) >= LocalOpenHour && TimeHour(JakartaDT) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktOpen );   	   
   	   ObjectSetText( "[MPClock] Jakartal", Jakartap, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Jakartal", Jakartap, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((India == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (India == 1)))      
      {              	       	     
   	if (TimeDayOfWeek(IndiaDT) != 0 && TimeDayOfWeek(IndiaDT) != 6 
   	&& TimeHour(IndiaDT) >= LocalOpenHour && TimeHour(IndiaDT) < LocalCloseHour)
   	   {
    	   ObjectSetText( "[MPClock] Indiat", Indias, ClockFontSize, FontName, Clock_MktOpen );  	   
   	   ObjectSetText( "[MPClock] Indial", Indiap, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
      	{
    	   ObjectSetText( "[MPClock] Indiat", Indias, ClockFontSize, FontName, Clock_MktClosed );     	
   	   ObjectSetText( "[MPClock] Indial", Indiap, TimezoneFontSize, FontName, Location_MktClosed );
         }
      }
   if((Dubai == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Dubai == 1)))   
      {       	      	       	     
   	if (TimeDayOfWeek(DubaiDT) != 0 && TimeDayOfWeek(DubaiDT) != 6 
   	&& TimeHour(DubaiDT) >= LocalOpenHour && TimeHour(DubaiDT) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Dubait", Dubais, ClockFontSize, FontName, Clock_MktOpen );     	
   	   ObjectSetText( "[MPClock] Dubail", Dubaip, TimezoneFontSize, FontName, Location_MktOpen ); 
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Dubait", Dubais, ClockFontSize, FontName, Clock_MktClosed );  	   
   	   ObjectSetText( "[MPClock] Dubail", Dubaip, TimezoneFontSize, FontName, Location_MktClosed );
      	}
      }
   if((Moscow == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Moscow == 1)))              
      { 	           	      	        	       
   	if (TimeDayOfWeek(MoscowDT) != 0 && TimeDayOfWeek(MoscowDT) != 6 
   	&& TimeHour(MoscowDT) >= LocalOpenHour && TimeHour(MoscowDT) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktOpen );  	   
   	   ObjectSetText( "[MPClock] Moscowl", Moscowp, TimezoneFontSize, FontName, Location_MktOpen ); 
      	}
   	else
   	   {
    	   ObjectSetText( "[MPClock] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktClosed );  	   
   	   ObjectSetText( "[MPClock] Moscowl", Moscowp, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((Israel == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Israel == 1)))                          
      {          	     	         	    
   	if (TimeDayOfWeek(IsraelDT) != 0 && TimeDayOfWeek(IsraelDT) != 6
   	&& TimeHour(IsraelDT) >= LocalOpenHour && TimeHour(IsraelDT) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Israelt", Israels, ClockFontSize, FontName, Clock_MktOpen );      	
   	   ObjectSetText( "[MPClock] Israell", Israelp, TimezoneFontSize, FontName, Location_MktOpen );
      	}
   	else
      	{
  	      ObjectSetText( "[MPClock] Israelt", Israels, ClockFontSize, FontName, Clock_MktClosed );       	
   	   ObjectSetText( "[MPClock] Israell", Israelp, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      } 
   if((Helsinki == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Helsinki == 1)))                    
      {        	                             	                 
   	if (TimeDayOfWeek(HelsinkiDT) != 0 && TimeDayOfWeek(HelsinkiDT) != 6 
   	&& TimeHour(HelsinkiDT) >= LocalOpenHour && TimeHour(HelsinkiDT) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktOpen );      	
   	   ObjectSetText( "[MPClock] Helsinkil", Helsinkip, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
    	   ObjectSetText( "[MPClock] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktClosed );  	   
   	   ObjectSetText( "[MPClock] Helsinkil", Helsinkip, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((Berlin == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Berlin == 1)))                         
      {                       
   	if (TimeDayOfWeek(BerlinDT) != 0 && TimeDayOfWeek(BerlinDT) != 6 
   	&& TimeHour(BerlinDT) >= LocalOpenHour && TimeHour(BerlinDT) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Berlint", Berlins, ClockFontSize, FontName, Clock_MktOpen );      	
   	   ObjectSetText( "[MPClock] Berlinl", Berlinp, TimezoneFontSize, FontName, Location_MktOpen );
      	}
   	else
      	{
   	   ObjectSetText( "[MPClock] Berlint", Berlins, ClockFontSize, FontName, Clock_MktClosed );     	
   	   ObjectSetText( "[MPClock] Berlinl", Berlinp, TimezoneFontSize, FontName, Location_MktClosed ); 
      	}
      } 
   if((London == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (London == 1)))            
      {                                
   	if (TimeDayOfWeek(LondonDT) != 0 && TimeDayOfWeek(LondonDT) != 6 
   	&& TimeHour(LondonDT) >= LocalOpenHour && TimeHour(LondonDT) < LocalCloseHour)
   	   {
      	ObjectSetText( "[MPClock] Londont", Londons, ClockFontSize, FontName, Clock_MktOpen );   	   
      	ObjectSetText( "[MPClock] Londonl", Londonp, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Londont", Londons, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Londonl", Londonp, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((GMT == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (GMT == 1)))                
      {	              
   	if (TimeDayOfWeek(UTC) != 0 && TimeDayOfWeek(UTC) != 6 
   	&& TimeHour(UTC) >= LocalOpenHour && TimeHour(UTC) < LocalCloseHour)
   	   {
      	ObjectSetText( "[MPClock] utct", UTCs, ClockFontSize, FontName, Clock_MktOpen );   	   
      	ObjectSetText( "[MPClock] utcl", UTCp, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {         
   	ObjectSetText( "[MPClock] utct", UTCs, ClockFontSize, FontName, Clock_MktClosed );                    	         
   	ObjectSetText( "[MPClock] utcl", UTCp, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((Brazil == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Brazil == 1)))           
      {               	     
   	if (TimeDayOfWeek(BrazilDT) != 0 && TimeDayOfWeek(BrazilDT) != 6 
   	&& TimeHour(BrazilDT) >= LocalOpenHour && TimeHour(BrazilDT) < LocalCloseHour)
   	   {
    	   ObjectSetText( "[MPClock] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktOpen );  	   
   	   ObjectSetText( "[MPClock] Brazill", Brazilp, TimezoneFontSize, FontName, Location_MktOpen );
      	}
   	else
      	{
   	   ObjectSetText( "[MPClock] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktClosed );      	
   	   ObjectSetText( "[MPClock] Brazill", Brazilp, TimezoneFontSize, FontName, Location_MktClosed );
      	}
      }
   if((NewYork == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (NewYork == 1)))     
      {                   
   	if (TimeDayOfWeek(NewYorkDT) != 0 && TimeDayOfWeek(NewYorkDT) != 6 
   	&& TimeHour(NewYorkDT) >= LocalOpenHour && TimeHour(NewYorkDT) < LocalCloseHour)
   	   {
      	ObjectSetText( "[MPClock] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktOpen );   	   
   	   ObjectSetText( "[MPClock] NewYorkl", NYp, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
      	ObjectSetText( "[MPClock] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktClosed );  	   
      	ObjectSetText( "[MPClock] NewYorkl", NYp, TimezoneFontSize, FontName, Location_MktClosed ); 
         }
      }
   if((Central == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Central == 1)))             
      {                       	      
   	if (TimeDayOfWeek(CentralDT) != 0 && TimeDayOfWeek(CentralDT) != 6 
   	&& TimeHour(CentralDT) >= LocalOpenHour && TimeHour(CentralDT) < LocalCloseHour)
   	   {
  	      ObjectSetText( "[MPClock] Centralt", Centrals, ClockFontSize, FontName, Clock_MktOpen );   	   
   	   ObjectSetText( "[MPClock] Centrall", Centralp, TimezoneFontSize, FontName, Location_MktOpen ); 
   	   }
   	else
   	   {
  	      ObjectSetText( "[MPClock] Centralt", Centrals, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Centrall", Centralp, TimezoneFontSize, FontName, Location_MktClosed ); 
   	   }
      }
   if((Mexico == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Mexico == 1)))             
      {                    	     
   	if (TimeDayOfWeek(MexicoDT) != 0 && TimeDayOfWeek(MexicoDT) != 6 
   	&& TimeHour(MexicoDT) >= LocalOpenHour && TimeHour(MexicoDT) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktOpen );   	   
   	   ObjectSetText( "[MPClock] Mexicol", Mexicop, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Mexicol", Mexicop, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((Mountain == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Mountain == 1)))         
      {	                 	      
   	if (TimeDayOfWeek(MountainDT) != 0 && TimeDayOfWeek(MountainDT) != 6 
   	&& TimeHour(MountainDT) >= LocalOpenHour && TimeHour(MountainDT) < LocalCloseHour)
      	{
  	      ObjectSetText( "[MPClock] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktOpen );      	
   	   ObjectSetText( "[MPClock] Mountainl", Mountainp, TimezoneFontSize, FontName, Location_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktClosed );   	   
   	   ObjectSetText( "[MPClock] Mountainl", Mountainp, TimezoneFontSize, FontName, Location_MktClosed );
   	   }
      }
   if((Pacific == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Pacific == 1)))                
      {                	     
   	if (TimeDayOfWeek(PacificDT) != 0 && TimeDayOfWeek(PacificDT) != 6 
   	&& TimeHour(PacificDT) >= LocalOpenHour && TimeHour(PacificDT) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktOpen );  	   
   	   ObjectSetText( "[MPClock] Pacificl", Pacificp, TimezoneFontSize, FontName, Location_MktOpen ); 
   	   }
   	else
      	{
    	   ObjectSetText( "[MPClock] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktClosed );     	
   	   ObjectSetText( "[MPClock] Pacificl", Pacificp, TimezoneFontSize, FontName, Location_MktClosed );
      	}
      }
   if((Local == 2) || ((Apply_Panel_Sm_Med_Lg_123==3) && (Local == 1)))        
      {
      ObjectSetText( "[MPClock] Localt", Locals, ClockFontSize, FontName,Clock_MktClosed );      
      ObjectSetText( "[MPClock] Locall", "Local", TimezoneFontSize, FontName, Location_MktClosed ); 
      }
   return(0);
   } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to make city and time labels                                                   |
//+-------------------------------------------------------------------------------------------+
int ObjectMakeLabel( string n, int xoff, int yoff )
   {
   if (ObjectFind(n) != 0){    
      ObjectCreate( n, OBJ_LABEL, 0, 0, 0 );
      ObjectSet( n, OBJPROP_CORNER, 0 );
      ObjectSet( n, OBJPROP_XDISTANCE, xoff );
      ObjectSet( n, OBJPROP_YDISTANCE, yoff );
      ObjectSet( n, OBJPROP_BACK, false ); }
   else{ObjectMove(n, 0, xoff, yoff);}       
   } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to convert time to a string for time labels                                    |
//+-------------------------------------------------------------------------------------------+
string TimeToString( datetime when )
   {
   string timeStr;
   int hour = TimeHour( when );
   if ( !Display_Times_With_AMPM ) 
      {
      timeStr = (TimeToStr( when, TIME_MINUTES));
      }

   // User wants 12HourTime format with "AM" or "PM".   
   // FYI, if >12:00, subtract 12 hours in seconds which is 12*60*60=43200
   else
      {
      if ( hour >  12 || hour == 0) timeStr = TimeToStr( (when - 43200), TIME_MINUTES);
      else timeStr = TimeToStr( when, TIME_MINUTES);
      if ( hour >= 12) timeStr = StringConcatenate(timeStr, " PM");
      else timeStr = StringConcatenate(timeStr, " AM");
      }
   return (timeStr);
   }

//+-------------------------------------------------------------------------------------------+
//| Subroutine to retrieve date and time from time array                                      |
//+-------------------------------------------------------------------------------------------+
datetime TimeArrayToTime(int& LocalTimeArray[])
   {  
   int    nYear,nMonth,nDOW,nDay,nHour,nMin,nSec,nMilliSec;
   nYear=LocalTimeArray[0]&0x0000FFFF;
   nMonth=LocalTimeArray[0]>>16;
   //nDOW=LocalTimeArray[1]&0x0000FFFF;
   nDay=LocalTimeArray[1]>>16;
   nHour=LocalTimeArray[2]&0x0000FFFF;
   nMin=LocalTimeArray[2]>>16;
   nSec=LocalTimeArray[3]&0x0000FFFF;
   nMilliSec=LocalTimeArray[3]>>16;
   string LocalTimeS = FormatDateTime(nYear,nMonth,nDay,nHour,nMin,nSec);
   datetime Local_Time = StrToTime( LocalTimeS );
   return(Local_Time);
   } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to format time labels                                                          |
//+-------------------------------------------------------------------------------------------+
string FormatDateTime(int nYear,int nMonth,int nDay,int nHour,int nMin,int nSec)
   {
   string sMonth,sDay,sHour,sMin,sSec;
   sMonth=100+nMonth;
   sMonth=StringSubstr(sMonth,1);
   sDay=100+nDay;
   sDay=StringSubstr(sDay,1);
   sHour=100+nHour;
   sHour=StringSubstr(sHour,1);
   sMin=100+nMin;
   sMin=StringSubstr(sMin,1);
   sSec=100+nSec;
   sSec=StringSubstr(sSec,1);
   return(StringConcatenate(nYear,".",sMonth,".",sDay," ",sHour,":",sMin,":",sSec));
   } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to get time zone information for each city                                     |
//+-------------------------------------------------------------------------------------------+
void GetAllTimeZoneInfo(int& AucklandTZInfoArray[], int& SydneyTZInfoArray[], 
   int& TokyoTZInfoArray[], int& ChinaTZInfoArray[], int& JakartaTZInfoArray[], 
   int& IndiaTZInfoArray[], int& DubaiTZInfoArray[], int& MoscowTZInfoArray[], 
   int& IsraelTZInfoArray[], int& HelsinkiTZInfoArray[], int& BerlinTZInfoArray[], 
   int& LondonTZInfoArray[], int& BrazilTZInfoArray[], int& NewYorkTZInfoArray[], 
   int& CentralTZInfoArray[], int& MexicoTZInfoArray[], int& MountainTZInfoArray[], 
   int& PacificTZInfoArray[] ) //, int& LocalTZInfoArray[])
   
   {
   AucklandTZInfoArray[0] = -720;
   AucklandTZInfoArray[17] = 262144; //4<<16 == 262144  April
   AucklandTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   AucklandTZInfoArray[19] = 3;
   AucklandTZInfoArray[20] = 0;
   AucklandTZInfoArray[21] = 0;
   AucklandTZInfoArray[38] = 589824; //9<<16 == 589824  September
   AucklandTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   AucklandTZInfoArray[40] = 2;
   AucklandTZInfoArray[41] = 0;
   AucklandTZInfoArray[42] = -60;
   
   SydneyTZInfoArray[0] = -600;
   SydneyTZInfoArray[17] = 262144; //4<<16 == 262144  April
   SydneyTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   SydneyTZInfoArray[19] = 3;
   SydneyTZInfoArray[20] = 0;
   SydneyTZInfoArray[21] = 0;
   SydneyTZInfoArray[38] = 655360; //10<<16 == 655360  October
   SydneyTZInfoArray[39] = 65536;  //1<<16 == 65536    1st Sunday
   SydneyTZInfoArray[40] = 2;
   SydneyTZInfoArray[41] = 0;
   SydneyTZInfoArray[42] = -60;
   
   // FYI Tokyo = Seoul
   TokyoTZInfoArray[0] = -540;
   TokyoTZInfoArray[17] = 0; 
   TokyoTZInfoArray[18] = 0;
   TokyoTZInfoArray[19] = 0;
   TokyoTZInfoArray[20] = 0;
   TokyoTZInfoArray[21] = 0;
   TokyoTZInfoArray[38] = 0;
   TokyoTZInfoArray[39] = 0;
   TokyoTZInfoArray[40] = 0;
   TokyoTZInfoArray[41] = 0;
   TokyoTZInfoArray[42] = 0;
   
   // FYI, Beijing = Perth, Singapore, Taipei, Hong Kong
   ArrayCopy(ChinaTZInfoArray, TokyoTZInfoArray);
   ChinaTZInfoArray[0] = -480;
   
   // FYI, Jakarta = Bangkok
   ArrayCopy(JakartaTZInfoArray, TokyoTZInfoArray);
   JakartaTZInfoArray[0] = -420;
   
   ArrayCopy(IndiaTZInfoArray, TokyoTZInfoArray);
   IndiaTZInfoArray[0] = -330;  // NOTE! Top of the hour is 30 min off most world timezones.
   
   ArrayCopy(DubaiTZInfoArray, TokyoTZInfoArray);
   DubaiTZInfoArray[0] = -240;
 
   IsraelTZInfoArray[0] = -120;
   IsraelTZInfoArray[17] = 589824;  //9<<16 == 589824  September
   IsraelTZInfoArray[18] = 131072;  //2<<16  = 131072
   IsraelTZInfoArray[19] = 2;
   IsraelTZInfoArray[20] = 0;
   IsraelTZInfoArray[21] = 0;
   IsraelTZInfoArray[38] = 196608; //3<<16 == 196608  March
   IsraelTZInfoArray[39] = 327685; //5<<16  = 327680 AND "5" (*IF* Sunday is "0", is "5" Friday??)
   IsraelTZInfoArray[40] = 2;
   IsraelTZInfoArray[41] = 0;
   IsraelTZInfoArray[42] = -60;
  
   LondonTZInfoArray[0] = 0;
   LondonTZInfoArray[17] = 655360; //10<<16 == 655360  October
   LondonTZInfoArray[18] = 327680; //5<<16 == 327680.  5th/Last Sunday. 
   LondonTZInfoArray[19] = 2;
   LondonTZInfoArray[20] = 0;
   LondonTZInfoArray[21] = 0;
   LondonTZInfoArray[38] = 196608; //3<<16 == 196608  March
   LondonTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   LondonTZInfoArray[40] = 1;
   LondonTZInfoArray[41] = 0;
   LondonTZInfoArray[42] = -60;
   
   ArrayCopy(MoscowTZInfoArray, LondonTZInfoArray);
   MoscowTZInfoArray[0] = -180;
   MoscowTZInfoArray[19] = 3;
   MoscowTZInfoArray[40] = 2;
   
   // FYI, Helsinki = Athens
   ArrayCopy(HelsinkiTZInfoArray, LondonTZInfoArray);
   HelsinkiTZInfoArray[0] = -120;
   HelsinkiTZInfoArray[19] = 4;
   HelsinkiTZInfoArray[40] = 3;
   
   // FYI, Berlin = Belgrade, Brussels, Paris, Sarajevo
   ArrayCopy(BerlinTZInfoArray, MoscowTZInfoArray);
   BerlinTZInfoArray[0] = -60;
   
   // NOTE! Brazil's ST/DST is likely a hardcoded date which will need updating EVERY YEAR!
   BrazilTZInfoArray[0] = 180;
   BrazilTZInfoArray[17] = 131072;   //2<<16  = 131072
   BrazilTZInfoArray[18] = 196614;   //3<<16 == 196608 AND 6, so, Saturday?? or Mar 6?
   BrazilTZInfoArray[19] = 3866647;
   BrazilTZInfoArray[20] = 65470523;
   BrazilTZInfoArray[21] = 0;
   BrazilTZInfoArray[38] = 655360;   //10<<16 == 655360
   BrazilTZInfoArray[39] = 196614;
   BrazilTZInfoArray[40] = 3866647;
   BrazilTZInfoArray[41] = 65470523;
   BrazilTZInfoArray[42] = -60;
   
   //ArrayCopy(BuenasAriesTZInfoArray, BuenasAriesTZInfoArray);
   //BuenasAriesTZInfoArray[0] = 180;
   
   NewYorkTZInfoArray[0] = 300;
   NewYorkTZInfoArray[17] = 720896; //wYear = 0. wMonth = 11, and 11<<16 == 720896
   NewYorkTZInfoArray[18] = 65536;  //wDOW = 0 = Sunday. nDay = 1 and 1<<16 == 65536 
   NewYorkTZInfoArray[19] = 2;
   NewYorkTZInfoArray[20] = 0;
   NewYorkTZInfoArray[21] = 0;
   NewYorkTZInfoArray[38] = 196608; //3<<16 == 196608  March
   NewYorkTZInfoArray[39] = 131072; //2<<16 == 131072  2nd Sunday
   NewYorkTZInfoArray[40] = 2;
   NewYorkTZInfoArray[41] = 0;
   NewYorkTZInfoArray[42] = -60;
   
   ArrayCopy(CentralTZInfoArray, NewYorkTZInfoArray);
   CentralTZInfoArray[0] = 360;
 
   MexicoTZInfoArray[0] = 360;
   MexicoTZInfoArray[17] = 655360; //10<<16 == 655360  October
   MexicoTZInfoArray[18] = 327680; //5<<16 == 327680  5th/Last Sunday
   MexicoTZInfoArray[19] = 2;
   MexicoTZInfoArray[20] = 0;
   MexicoTZInfoArray[21] = 0;
   MexicoTZInfoArray[38] = 262144; //4<<16 == 262144  April
   MexicoTZInfoArray[39] = 65536;  //1<<16 == 65536   1st Sunday
   MexicoTZInfoArray[40] = 2;
   MexicoTZInfoArray[41] = 0;
   MexicoTZInfoArray[42] = -60;
   
   ArrayCopy(MountainTZInfoArray, NewYorkTZInfoArray);
   MountainTZInfoArray[0] = 420;
   
   ArrayCopy(PacificTZInfoArray, NewYorkTZInfoArray);
   PacificTZInfoArray[0] = 480;   
   } 

//+-------------------------------------------------------------------------------------------+
//| Custom Indicator End                                                                      |
//+-------------------------------------------------------------------------------------------+

