//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                    SonicR MACD.mq4                                        |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+
#property copyright "Traderathome - public domain code"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview:

This indicator displays the "standard" MACD study.  Howeever, this version puts both the MACD 
and Signal line on top of the histogram for better visibility.  And it adds a Zero line which
also enhances the appearance of the study.

Default color settings are included for both white and black charts.

You can turn the indicator off, preserving your chart settings.  And you can specify a range 
of time frames beyond which the indicator will not display.  This allows you to include a
different indicator in the same subwindow to be used on different TFs.  For example, you might
want a Volume indy for TFs M30 and lower, and a MACD indy for TFs H1 and higher.

                                                                    - Traderathome, 12-01-2010
----------------------------------------------------------------------------------------------                                                       
Suggested Settings:          White Chart                     Black Chart   

#property indicator_color1   SlateBlue      //Histogram
#property indicator_color2   SlateBlue      //Zero Line 
#property indicator_color3   Red            //Signal Line 
#property indicator_color4   C'0,159,0'     //Macd Line   
#property indicator_width1   2
#property indicator_width2   1
#property indicator_width3   1 
#property indicator_width4   2 
#property indicator_style1   STYLE_SOLID  
#property indicator_style2   STYLE_SOLID   
#property indicator_style3   STYLE_SOLID    
#property indicator_style4   STYLE_SOLID

#property indicator_color5                                   SlateBlue      //Histogram
#property indicator_color6                                   SlateBlue      //Zero Line 
#property indicator_color7                                   Red            //Signal Line 
#property indicator_color8                                   LimeGreen      //Macd Line   
#property indicator_width5                                   2
#property indicator_width6                                   1
#property indicator_width7                                   1 
#property indicator_width8                                   1
#property indicator_style5                                   STYLE_SOLID    
#property indicator_style6                                   STYLE_SOLID    
#property indicator_style7                                   STYLE_DOT     
#property indicator_style8                                   STYLE_SOLID                                                                                                                                                                                    
---------------------------------------------------------------------------------------------*/                                                         


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |
//+-------------------------------------------------------------------------------------------+
//Global Indicator Settings
#property indicator_separate_window

#property  indicator_buffers 8
#property indicator_color1   SlateBlue      //Histogram
#property indicator_color2   SlateBlue      //Zero Line 
#property indicator_color3   Red            //Signal Line 
#property indicator_color4   C'0,159,0'     //Macd Line   
#property indicator_width1   2
#property indicator_width2   1
#property indicator_width3   1 
#property indicator_width4   2
#property indicator_style1   STYLE_SOLID    //STYLE_SOLID
#property indicator_style2   STYLE_SOLID    //STYLE_SOLID
#property indicator_style3   STYLE_SOLID    //STYLE_DOT
#property indicator_style4   STYLE_SOLID

#property indicator_color5   SlateBlue      //Histogram
#property indicator_color6   SlateBlue      //Zero Line 
#property indicator_color7   Red            //Signal Line 
#property indicator_color8   LimeGreen      //Macd Line   
#property indicator_width5   2
#property indicator_width6   1
#property indicator_width7   1 
#property indicator_width8   1
#property indicator_style5   STYLE_SOLID    //STYLE_SOLID
#property indicator_style6   STYLE_SOLID    //STYLE_SOLID
#property indicator_style7   STYLE_DOT      //STYLE_DOT
#property indicator_style8   STYLE_SOLID

//Global External Inputs
extern bool   Indicator_On                  = true;
extern int    FastMAPeriod                  = 12;
extern int    SlowMAPeriod                  = 26;
extern int    SignalMAPeriod                = 9;
extern bool   Apply_On_White_vs_Black_Chart = true;
extern int    Display_Min_TF                = 60;
extern int    Display_Max_TF                = 43200;
extern string TF_Choices_1                  = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                  = "D  W  M =  1440,  10080,  43200";

//Global Buffers, Constants, Variables
bool          FLAG_deinit;
double        HistogramBufferW[], HistogramBufferB[];
double        ZerolineW[], ZerolineB[];
double        MACDLineBufferW[], MACDLineBufferB[];
double        SignalLineBufferW[], SignalLineBufferB[];
int           i, limit, counted_bars;


//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |
//+-------------------------------------------------------------------------------------------+
int init()
   {
   FLAG_deinit  = false;  
    
   if(Apply_On_White_vs_Black_Chart)
      {        
      SetIndexBuffer(0,HistogramBufferW);
      SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,ZerolineW); 
      SetIndexStyle(1,DRAW_LINE);            
      SetIndexBuffer(2,SignalLineBufferW);
      SetIndexStyle(2,DRAW_LINE);            
      SetIndexBuffer(3,MACDLineBufferW);
      SetIndexStyle(3,DRAW_LINE);
      }
   else
      {         
      SetIndexBuffer(4,HistogramBufferB); 
      SetIndexStyle(4,DRAW_HISTOGRAM);
      SetIndexBuffer(5,ZerolineB); 
      SetIndexStyle(5,DRAW_LINE);      
      SetIndexBuffer(6,SignalLineBufferB);
      SetIndexStyle(6,DRAW_LINE);           
      SetIndexBuffer(7,MACDLineBufferB);
      SetIndexStyle(7,DRAW_LINE);
      }      
      
   //Indicator subwindow name and data labels
   IndicatorShortName("SonicR MACD  ("+FastMAPeriod+","+SlowMAPeriod+","+SignalMAPeriod+")   ");
   SetIndexLabel(0, NULL);  
   SetIndexLabel(1, NULL); 
   SetIndexLabel(2, NULL); 
   SetIndexLabel(3, NULL);
   SetIndexLabel(4, NULL);
   SetIndexLabel(5, NULL);
   SetIndexLabel(6, NULL);
   SetIndexLabel(7, NULL);

   return(0);
   }
//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |
//+-------------------------------------------------------------------------------------------+
int deinit()
   {

   return(0);
   }
//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |
//+-------------------------------------------------------------------------------------------+
int start()
   {
   //If Indicator is "Off" or chart is out of range deinitialize only once, not every tick.  
   if ((!Indicator_On) || (Period() < Display_Min_TF) || (Period() > Display_Max_TF))
      {
      IndicatorShortName("SonicR MACD  ("+FastMAPeriod+","+SlowMAPeriod+","+SignalMAPeriod+")  -off   ");
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}      
      return(0);
      }

   //Else indicator is "On"/ TF is in display range.  Clear variables & go.                   
   deinit(); FLAG_deinit = false;   
       
   //Check for possible errors
   counted_bars = IndicatorCounted();
   if (counted_bars<0) return(-1);
   
   //Last counted bar will be recounted
   if (counted_bars>0) counted_bars--;
   limit = Bars - counted_bars;
   
   //Fill buffers
   if(Apply_On_White_vs_Black_Chart)
      {        
      for(i=0; i<limit; i++)
      MACDLineBufferW[i] = iMA(NULL,0,FastMAPeriod,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowMAPeriod,0,MODE_EMA,PRICE_CLOSE,i);
      for(i=0; i<limit; i++) 
      ZerolineW[i] = 0;
      for(i=0; i<limit; i++)
      SignalLineBufferW[i] = iMAOnArray(MACDLineBufferW,Bars,SignalMAPeriod,0,MODE_SMA,i);   
      for(i=0; i<limit; i++)
      HistogramBufferW[i] = (MACDLineBufferW[i] - SignalLineBufferW[i]);   
      }
   else
      {      
      for(i=0; i<limit; i++)        
      MACDLineBufferB[i] = iMA(NULL,0,FastMAPeriod,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowMAPeriod,0,MODE_EMA,PRICE_CLOSE,i);
      for(i=0; i<limit; i++) 
      ZerolineB[i] = 0;
      for(i=0; i<limit; i++)
      SignalLineBufferB[i] = iMAOnArray(MACDLineBufferB,Bars,SignalMAPeriod,0,MODE_SMA,i);   
      for(i=0; i<limit; i++)
      HistogramBufferB[i] = (MACDLineBufferB[i] - SignalLineBufferB[i]);           
      } 
       
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+