//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                  SonicR Trend Line.mq4                                    |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2010 traderathome"
#property link      "email: traderathome@msn.com"

/*---------------------------------------------------------------------------------------------
Overview:

This indicator displays the SonicR Trend.  It can be turned On/Off, preserving your chart 
settings.  And it can be set to automatically not display above a certain TF that you input. 

Default color settings are included for both white and black charts.

The SonicR Trend is based on the inputs "Trend_Control_MA= 89" and "Trend_Control_TF= 15". 
These inputs are used to mirror the M30 and H1 charts to the M15 chart, making the Trend line 
display on these two higher TFs approximately the same as it does on the M15 TF.  This feature 
for the M30 and H1 charts is easily turned On/Off individually. 
                              
                                                                     - Traderathome, 12-01-2010 
----------------------------------------------------------------------------------------------- 
Suggested Settings:             White Chart      Black Chart

#property indicator_color1   C'255,255,128' 
#property indicator_color2   Black
#property indicator_width1   3
#property indicator_width2   1
#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_DOT

#property indicator_color3                       C'255,255,151'
#property indicator_color4                       CLR_NONE
#property indicator_width3                       1
#property indicator_width4                       1
#property indicator_style3                       STYLE_SOLID
#property indicator_style4                       STYLE_DOT                                                                                                                                                                 
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

#property indicator_buffers  4
#property indicator_color1   C'255,255,128' 
#property indicator_color2   Black
#property indicator_width1   3
#property indicator_width2   1
#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_DOT

#property indicator_color3   C'255,255,151' 
#property indicator_color4   CLR_NONE
#property indicator_width3   1
#property indicator_width4   1
#property indicator_style3   STYLE_SOLID
#property indicator_style4   STYLE_DOT

//global external inputs
extern string  Part_1                        = "Indicator Display Controls:";
extern bool    Indicator_On                  = true;
extern bool    Trend_M30_Mirror_M15          = true;
extern bool    Trend_H1_Mirror_M15           = true;
extern bool    Apply_On_White_vs_Black_Chart = true;
extern int     Indicator_Max_Display_TF      = 43200;
extern string  TF_Choices_1                  = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string  TF_Choices_2                  = "D  W  M =  1440,  10080,  43200";

extern string  __                            = "";
extern string  Part_2                        = "Trend Line Settings:";
extern int     Trend_Control_MA              = 89;
extern int     Trend_Control_TF              = 15;
extern bool    Trend_MA_Exp_vs_Simple        = true;

//global buffers and variables
bool    FLAG_deinit;
double  Trend1BufferW[], Trend2BufferW[], Trend1BufferB[], Trend2BufferB[];
int     Trend_MA_Period,i,counted_bars,limit,BarShift,Factor;
int     MA_Type                      = 1;
string  MA_Type_Choices              = "0=SMA, 1=EMA";
int     MA_Price                     = 0;
string  MA_Price_Choices             = "0=C  1=O  2=H  3=L  4=M  5=P  6=W"; 

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
   {
   FLAG_deinit = false;
   MA_Type = 1; if (!Trend_MA_Exp_vs_Simple) {MA_Type = 0;}
       
   if(Apply_On_White_vs_Black_Chart)
      {        
      SetIndexBuffer(0, Trend1BufferW);
      SetIndexBuffer(1, Trend2BufferW);              
      SetIndexStyle(0, DRAW_LINE);   
      SetIndexStyle(1, DRAW_LINE);              
      SetIndexEmptyValue(0,0);
      SetIndexEmptyValue(1,0);      
      }
   else
      {          
      SetIndexBuffer(2, Trend1BufferB);
      SetIndexBuffer(3, Trend2BufferB);              
      SetIndexStyle(2, DRAW_LINE);   
      SetIndexStyle(3, DRAW_LINE);             
      SetIndexEmptyValue(2,0);
      SetIndexEmptyValue(3,0); 
      } 
      
   //Application of Trend Period Adjustment
   Trend_MA_Period = Trend_Control_MA;              
   if (((Trend_M30_Mirror_M15) && (Period() == 30)) || ((Trend_H1_Mirror_M15) && (Period() == 60))) 
      {          
      Trend_MA_Period= MathRound(Trend_Control_MA * Trend_Control_TF / Period());          
      }                                 
     
   //Define Indicator Short Name 
   IndicatorShortName("SonicR Trend ( "+DoubleToStr(Trend_MA_Period,0) + " ) ");                     
               
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+      
int deinit()
   {      
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+     
int start()
   {
   //If indicator is "Off" or chart TF is out of range
   //deinitialize only once, not every ticc, and exit.  
   if( (!Indicator_On) || (Period() > Indicator_Max_Display_TF)) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }

   //If indicator is "ON", deinitialize and run   
   deinit(); FLAG_deinit = false;
                       
   //Calculate MAs   
   //check for possible errors
   counted_bars = IndicatorCounted();
   if(counted_bars < 0)  return(-1); 
     
   //last counted bar will be recounted
   if(counted_bars > 0) counted_bars--;    
   limit = Bars - counted_bars;
   
   //do for white or black chart     
   if(Apply_On_White_vs_Black_Chart)
      {
      for(i = limit - 1; i >= 0; i--)  
         {
         BarShift = iBarShift(NULL,NULL,Time[i],true);                                  
         Trend1BufferW[i]   = iMA(NULL,NULL,Trend_MA_Period,0,MA_Type,MA_Price,BarShift); 
         Trend2BufferW[i]   = iMA(NULL,NULL,Trend_MA_Period,0,MA_Type,MA_Price,BarShift);
         }          
      }         
   else
      {
      for(i = limit - 1; i >= 0; i--)  
         {
         BarShift = iBarShift(NULL,NULL,Time[i],true);                                  
         Trend1BufferB[i]   = iMA(NULL,NULL,Trend_MA_Period,0,MA_Type,MA_Price,BarShift); 
         Trend2BufferB[i]   = iMA(NULL,NULL,Trend_MA_Period,0,MA_Type,MA_Price,BarShift); 
         }               
      }
                                  
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+       

