//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                 SonicR Tunnel Dragon.mq4                                  |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2010 traderathome"
#property link      "email: traderathome@msn.com"

/*---------------------------------------------------------------------------------------------
Overview:

This indicator displays the SonicR Dragon.  It can be turned On/Off, preserving your chart 
settings.  And it can be set to automatically not display above a certain TF that you input.

Default color settings are included for both white and black charts.

The SonicR Dragon is based on the inputs "Dragon_Control_MA= 34" and "Dragon_Control_TF= 15". 
These inputs are used to mirror the M30 and H1 charts to the M15 chart, making the Dragon 
display on these two higher TFs approximately the same as it does on the M15 TF.  This feature 
for the M30 and H1 charts is easily turned On/Off individually.
                               
                                                                     - Traderathome, 12-01-2010 
----------------------------------------------------------------------------------------------- 
Suggested Settings:             White Chart      Black Chart

#property indicator_color1   DodgerBlue 
#property indicator_color2   Red
#property indicator_color3   DodgerBlue
#property indicator_width1   1
#property indicator_width2   1
#property indicator_width3   1
#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_SOLID
#property indicator_style3   STYLE_SOLID

#property indicator_color4                       DarkGray
#property indicator_color5                       DarkGray
#property indicator_color6                       DarkGray
#property indicator_width4                       1
#property indicator_width5                       1
#property indicator_width6                       
#property indicator_style4                       STYLE_SOLID
#property indicator_style5                       STYLE_DOT 
#property indicator_style6                       STYLE_SOLID                                                                                                    
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

#property indicator_buffers  6
#property indicator_color1   DodgerBlue 
#property indicator_color2   Red
#property indicator_color3   DodgerBlue
#property indicator_width1   1
#property indicator_width2   1
#property indicator_width3   1
#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_SOLID
#property indicator_style3   STYLE_SOLID

#property indicator_color4   DarkGray
#property indicator_color5   DarkGray
#property indicator_color6   DarkGray
#property indicator_width4   1
#property indicator_width5   1
#property indicator_width6   1
#property indicator_style4   STYLE_SOLID
#property indicator_style5   STYLE_DOT 
#property indicator_style6   STYLE_SOLID 

//global external inputs
extern string  Part_1                        = "Indicator Display Controls:";
extern bool    Indicator_On                  = true;
extern bool    Dragon_M30_Mirror_M15         = true;
extern bool    Dragon_H1_Mirror_M15          = true;
extern bool    Apply_On_White_vs_Black_Chart = true;
extern int     Display_Max_TF                = 43200;
extern string  TF_Choices_1                  = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string  TF_Choices_2                  = "D  W  M =  1440,  10080,  43200";

extern string  __                            = "";
extern string  Part_2                        = "Dragon Settings:";
extern int     Dragon_Control_MA             = 34;
extern int     Dragon_Control_TF             = 15;
extern bool    Dragon_MA_Exp_vs_Simple       = true;
extern bool    Center_Line_Close_vs_Median   = false;

//global buffers and variables
bool    FLAG_deinit;

double  MAhighBufferW[], MAcenterBufferW[], MAlowBufferW[],
        MAhighBufferB[], MAcenterBufferB[], MAlowBufferB[], Y;
int     Dragon_MA_Period,i,counted_bars,limit,BarShift,Factor;
int     Dragon_Type                          = 1;
string  Dragon_Type_Choices                  = "0=SMA, 1=EMA";
int     Centerline                           = 0;
string  Centerline_Choices                   = "0=CLOSE, 4=MEDIAN";

double  Dragon_M30_ShiftAdjust_Pips  = 05;  
double  Dragon_M60_ShiftAdjust_Pips  = 12;   

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
   {
   FLAG_deinit = false;
   Dragon_Type = 1; if (!Dragon_MA_Exp_vs_Simple) {Dragon_Type = 0;}   
   Centerline = 0; if (!Center_Line_Close_vs_Median) {Centerline = 4;}
   
   //Select proper Factor for Dragon expansion number
        if (Digits == 2) Factor = 100;
   else if (Digits == 3) Factor = 1000;         
   else if (Digits == 4) Factor = 10000;    
   else if (Digits == 5) Factor = 100000;
      
   if(Apply_On_White_vs_Black_Chart)
      { 
      SetIndexBuffer(0, MAhighBufferW);
      SetIndexBuffer(1, MAcenterBufferW);
      SetIndexBuffer(2, MAlowBufferW);
      SetIndexStyle(0, DRAW_LINE);
      SetIndexStyle(1, DRAW_LINE);   
      SetIndexStyle(2, DRAW_LINE);
      SetIndexEmptyValue(0,0);
      SetIndexEmptyValue(1,0);
      SetIndexEmptyValue(2,0); 
      }      
   else
      {
      SetIndexBuffer(3, MAhighBufferB);
      SetIndexBuffer(4, MAcenterBufferB);
      SetIndexBuffer(5, MAlowBufferB);
      SetIndexStyle(3, DRAW_LINE);
      SetIndexStyle(4, DRAW_LINE);   
      SetIndexStyle(5, DRAW_LINE);
      SetIndexEmptyValue(3,0);
      SetIndexEmptyValue(4,0);
      SetIndexEmptyValue(5,0);
      } 

   //Application of Dragon Period Adjustment
   Y = 0;
   Dragon_MA_Period= Dragon_Control_MA;              
   if ((Dragon_M30_Mirror_M15) && (Period() == 30)) 
      {          
      Dragon_MA_Period= MathRound(Dragon_Control_MA * Dragon_Control_TF / Period());
      Y = Dragon_M30_ShiftAdjust_Pips/Factor;        
      }   
   if ((Dragon_H1_Mirror_M15) && (Period() == 60)) 
      {          
      Dragon_MA_Period= MathRound(Dragon_Control_MA * Dragon_Control_TF / Period());            
      Y = Dragon_M60_ShiftAdjust_Pips/Factor; 
      }                                
     
   //Indicator Short Name 
   IndicatorShortName("SonicR Dragon ( "+DoubleToStr(Dragon_MA_Period,0) + " ) ");                     
               
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+      
int deinit()
   {      
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+     
int start()
   {
   //If indicator is "Off" or chart TF is out of range
   //deinitialize only once, not every ticc, and exit.  
   if( (!Indicator_On) || (Period() > Display_Max_TF)) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }

   //If indicator is "ON", deinitialize and run   
   deinit(); FLAG_deinit = false;
                       
   //Calculate MAs   
   //check for possible errors
   counted_bars = IndicatorCounted();
   if(counted_bars < 0)  return(-1); 
     
   //last counted bar will be recounted
   if(counted_bars > 0) counted_bars--;    
   limit = Bars - counted_bars;
   
   //do for white or black chart
   if(Apply_On_White_vs_Black_Chart)
      {   
      for(i = limit - 1; i >= 0; i--)  
         {
         BarShift = iBarShift(NULL,NULL,Time[i],true); 
         MAhighBufferW[i]   = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_Type,PRICE_HIGH,BarShift)+Y;       
         MAcenterBufferW[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_Type,Centerline,BarShift); 
         MAlowBufferW[i]    = iMA(NULL,NULL,Dragon_MA_Period ,0,Dragon_Type,PRICE_LOW,BarShift)+Y;        
         if ((Dragon_M30_Mirror_M15) && (Period() == 30))
            {      
            MAhighBufferW[i]   =  MAhighBufferW[i]- ((MAhighBufferW[i]-MAlowBufferW[i])/6.5);
            MAlowBufferW[i]    =  MAlowBufferW[i] + ((MAhighBufferW[i]-MAlowBufferW[i])/6.5);
            }
         if ((Dragon_H1_Mirror_M15) && (Period() == 60))
            {      
            MAhighBufferW[i]   =  MAhighBufferW[i]- ((MAhighBufferW[i]-MAlowBufferW[i])/3.4);
            MAlowBufferW[i]    =  MAlowBufferW[i] + ((MAhighBufferW[i]-MAlowBufferW[i])/3.4);
            } 
         }                        
      }         
   else
      {
      for(i = limit - 1; i >= 0; i--)  
         {
         BarShift = iBarShift(NULL,NULL,Time[i],true);         
         MAhighBufferB[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_Type,PRICE_HIGH,BarShift)+Y;       
         MAcenterBufferB[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_Type,Centerline,BarShift); 
         MAlowBufferB[i] = iMA(NULL,NULL,Dragon_MA_Period ,0,Dragon_Type,PRICE_LOW,BarShift)+Y;
      
         if ((Dragon_M30_Mirror_M15) && (Period() == 30))
            {      
            MAhighBufferB[i]   =  MAhighBufferB[i]- ((MAhighBufferB[i]-MAlowBufferB[i])/6.5);
            MAlowBufferB[i]    =  MAlowBufferB[i] + ((MAhighBufferB[i]-MAlowBufferB[i])/6.5);
            }
         if ((Dragon_H1_Mirror_M15) && (Period() == 60))
            {      
            MAhighBufferB[i]   =  MAhighBufferB[i]- ((MAhighBufferB[i]-MAlowBufferB[i])/3.4);
            MAlowBufferB[i]    =  MAlowBufferB[i] + ((MAhighBufferB[i]-MAlowBufferB[i])/3.4);
            }
         }             
      }               
                                  
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+       

