//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                 SonicR TzPivots.mq4                                       |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+
#property copyright "CaveMan, Domas4 and Traderathome, Copyright @ 2010"
#property link      "email: traderathome@msn.com"

/*---------------------------------------------------------------------------------------------
Overview:

This indicator is based on the TzPivotsD venue indicators, but it is streamlined for SonicR. 
It is a significant upgrade from the original release of SonicR TzPivots (02-01-2010), and 
contains substantial improvements over TzPivotsD_v3.5 (the last release).

It uses Caveman's benchmark time zone shift and DST codes, but with new changes that make it 
easier to switch between time shifted pivots and broker/server pivots.  Once you have made the
proper inputs for "DST_Server_GMT_Offset" and "DST_Choice_GMT_Offset" and "DST_Zone", you can 
then simply designate "true/false" for "Time_Shifted_Pivots_On" to change between the time 
shifted pivots and your broker server pivots.  No other changes are necessary whether it is the 
DST time of the year, or not!  An asterisk (*) will appear in the pivot labels signifying  the
shifted pivots are being displayed, and does not appear when displaying broker/server pivots.

The indicator can be used for both forex and non-forex items. In the Forex mode, you can show
either 4 digit or 5 digits.  If your platform is 4 digit and you select the 5 digit display, 
a "0" is added to displayed numbers. If it is a 5 digit platform and you select to display 4 
digits, numbers are rounded off to 4 digits.  

You can display, or not display, each catagory of pivot lines.  Start/stop placement of the
lines has been kept very simple.  Pivot label placement is fully automatic with the only 
exception being the "PivotLabels_Max_Right_Override", which you can select as an option.

Separators now include yesterday, today and tomorrow.  They are fixed to your broker/server
session times, not TZ shifted sessions.  There is a fouth separator at the today TZ shifted 
session you can display when needing to confirm your TZ shift settings.

Colors, line styles, solid line thicknesses, font styles and sizes....anything that might be
changed between using black or white charts, has been hard coded for black and white charts.
In addition, you have the normal ability to make your own choices in External Inputs.  You
can easily switch between defaults for black charts, white charts, and your own selections.

This indicator is flexible in its display choices, is competant in displaying broker server
pivots or pivots bassed on any other time zone, and is now easier to use!
      
                                                                    - Traderathome, 12-01-2010
-----------------------------------------------------------------------------------------------
Acknowlegements:

CaveMan - who generously devoted his time and programming skills to extensively recode the 
          time zone shift section, and add DST code which automates applying an hour time
          shift when appropriate at DST/non-DST changes.  With CaveMan's time zone shift code 
          it is anticipated the indicator can handle any situation where there are gaps between 
          trading days, where there are partial trading days, or both.  And with his DST code 
          there is no need to adjust your time zone shift settings twice a year at DST/non-DST
          changes.  Also, it provides for customizing your own DST Zone/settings in case your
          broker/server implements these changes at times not covered by the preset DST Zones. 

Domas4  - "THANKS!" are owed to Domas4 for providing price labeling code compatible with non-
          forex items.  
                                                                                                                                                                                                
----------------------------------------------------------------------------------------------- 
Comments regarding the indicator's External Inputs:

Indicator_On:
   Use "true" to turn indicator "On", and "false" to turn "Off".  This way you do not have to
   delete the indicator from the chart's list of indicators when you do not want it displayed.
   And the indicator will preserve your settings for the chart.
   
Indicator_Display_Max_TF:
   On charts of a higher TF than you input, the indicator will not display.  The choices you
   select from in order to make entry here are shown below in "TF_Choices".
       
Use_Day_Formula:
   Enter "false" if you want pivots based on the fibonacci formula. 
      
Time_Shifted_Pivots_On; 
   To get pivots based on a time zone different from your broker server's time zone, set this 
   "true".  An asterisk (*) will appear after the central pivot label confirming that time 
   zone shifting, per the two TZ shift settings below, is in effect.  For your standard broker
   server pivots, set this "false". The asterisk will not display, confirming the pivots are
   not time zone shifted pivots.  A "false" setting assures no time shift will occur by 
   internally changing the "DST_Choice_GMT_Offset" to be equal to the "DST_Server_GMT_Offset".
                                      
DST_Server_GMT_Offset:
   If your broker/server is GMT based all year round, input "0".  Otherwise do the following.
   Enter the number of hours difference between GMT and the time zone of your platform server, 
   for the DST time of the year!  It is very important you use the DST broker/server offset 
   from GMT, and not the non-DST time of year offset!  Your server time is evident in the time 
   scale below the charts.  Using this information, to determine your broker/server hours 
   difference from GMT go to World Clock.com (http://www.timeanddate.com/worldclock/) and see 
   the current GMT time.  Your chart time will be either earlier than GMT, at GMT, or after
   GMT time, depending on whether your broker server is east, at, or west of GMT.  In any case,
   the positive, zero, or negative hours difference respectively is the number you input here
   if the current time is DST time.  If you are setting this up during the non-DST time of 
   year, then you need to add (+) one hour to your number.  This is to compensate for the fact
   that when the world shifts from DST to non-DST there is a -1 hour shift in time.  Once set,
   you will not have to change it because hard code automatically does the adjusting during 
   changeovers between DST and non-DST times of the year.
     
DST_Choice_GMT_Offset:
   Enter the number of hours difference between GMT and the timezone you choose as the basis 
   for pivots calculations.  For example, if you wish to have the pivot day start at NY time, 
   then enter "-4" since during the DST time of the year NY is 4 hours after GMT.  You base 
   your setting on the DST time of the year. Once set, you do not have to change it during 
   changeovers between DST and non-DST times of the year.  The code will automatically do the 
   adjusting.  Be sure to enter the proper "DST_Zone" number in part 6 of the external inputs.
  
Use_For_Forex:
   For forex application, set to "true" to enable the proper digits display for the prices of
   the various forex pairs. 
   
Display_5_Digits:
   Regardless of whether your platform is 4-digit or 5-digit, you can control the digits 
   displayed with this input.  If your platform is 4 digit and you select the 5 digit display, 
   a "0" is added to displayed numbers.  If your platform is 5 digit and you select to not 
   display 5 digits, numbers are rounded off to 4 digits.  
   
TF_Choices:
   This simply shows the choices to be selected from to enter into "Max_Display_TF".

Apply_Default_Color_Selections:
   In addition to the colors selected in the External Inputs, there are hard coded color
   selections for black and for white charts.  You access the hard coded colors by making
   this input "true".  An input of "false" will use the External Inputs.  Colors, line styles,
   solid line thicknesses, font styles and sizes are all included in the hard codes.
   
Apply_On_White_vs_Black_Chart:
   an input of "true" will use the hard coded colors, etc., for a white chart.  An input of 
   "false" will use the hard coded colors, etc., for a black chart.

TF_Choices_1:
   The numbers provided are to select time frames M1 thru H4.

TF_Choices_2:
   The numbers provided are to select time frames Daily, Weekly and Monthly.
               
Line_Selection_Number:
   Using "1" will produce pivot lines across the full screen.  Using "2" will start the pivot 
   lines at today's separator and end them at tomorrow's separator.  Using "3" will start the 
   lines at the current candle and end them at the chart right border.  When using "2", the 
   starting point of the lines will eventually disappear off the left of the chart on the 
   lowest time frames (M1, M5), making the lines full screen wide.  When this happens they  
   are automatically handled as full screen wide lines and their labels are mid-screen.  

A Note on Pivot lines Selection:
   Each catagory of pivot lines (central pivot, main S&R pivots, and mid-pivots) can be 
   turned On/Off with "true/false".
             
A Note on Pivot lines Color, Line Style and Thickness:
   Enter a color for the pivot line.  Enter the number for the desired line style (0=solid, 
   1=dash, 2=dot, 3=dashdot, 4=dashdashdot).  Enter the number (1, 2, 3, etc.) for the
   deired thickness if it is to be a solid line.  

A Note on Pivot Labels Color, Selection, Font Style and Size:
   Enter a color for the pivot lines labels.  As a group, the labels can be turned On/Off 
   with "true/false".  Enter the name of the font style you want.  Arial, Arial Bold, 
   Verdana and Verdana Bold are good choices.  Enter the font size # desired (usually 8).  
   
PivotLabels_Prices: 
   To include prices in the labels, enter "true".
   
Subordinate_Pivot_Labels:
   Enter "true" if the labels undesirably overwrite other chart items.   
   
PivotLabels_Max_Right_Override:
   Overrides all other label placement settings and places the labels as far right as MT4
   coding allows. 

A Note on Separator Selection:
   The "Yesterday", "Today", "Tomorrow" and "TzShift" separators can individually be turned 
   On/Off with "true/false".

A Note on Separator Color, Line Style And Thickness:
   Enter a color for the separator line.  Enter the number for the desired line style 
   (0=solid, 1=dash, 2=dot, 3=dashdot, 4=dashdashdot).  Enter the number (1, 2, 3 etc.) for 
   the deired thickness if it is to be a solid line. 

A Note on Separator Labels Selection:
   As a group, the separator labels can be turned On/Off with "true/false".
 
A Note on Separator Labels Color, Selection, Font Style, Font Size And Subordination:
   Enter a color for the separator label.  As a group, the separator labels can be turned 
   On/Off with "true/false".  Enter the name of the font style you want.  Arial, Arial Bold, 
   Verdana and Verdana Bold are good choices.  Enter the font size # desired (usually 8). 
 
SeparatorLabels_Chart_Bottom:
   "true" will place the Period Separator labels at the bottom of the screen.  "false" will
   place them at the top.  If the screen is enlarged, downsized, or scrolled, these labels
   will be displaced.  Their postion should be restored with the next data tick. 

SeparatorLabels_Dist_To_Border:
   Enter a number to move the label farther from the chart top/bottom border.  The larger
   the number, the greater the shift.  This is useful if a change in font style, or a 
   change to a larger font size leaves the label too close to the chart border.

A Note On The TzTest_Panel:
   This Displays key data in management of the TzShift code and is used for testing only.

Comments On The "DST Zone Setting" Section:
   Four DST zones are preset for you to choose from (US, EU, AUS and NZ), hard coded to
   make an hour shift for DST changeovers at the appropriate time twice yearly.  If your
   broker does not make DST changeovers on their server per one of these 4 zones, then
   you must customize a 5th DST zone based on when your broker does make DST changeovers.  
   
   Be warned that you cannot reliably assume that your broker/server is in the "eu" zone
   just because the chart time scale is based on what time it is somewhere in Europe.  Nor 
   can you reliably assume that your broker/server is in the "us" zone just because that 
   is where the broker is located.  You are therefore advised to ignore your chart time 
   scale and broker location and find out from your broker exactly when do they implement
   DST changes on their server. 

DST_Zone:
   Enter the number (1, 2, 3, 4) for the DST zone that represents when your broker/server
   makes the DST changeovers.  If your broker/server makes DST changeovers not covered by
   these DST zones, then enter "5" and proceed to customize a 5th DST zone accordingly.
   To customize your DST Zone settings just follow the instructions provided by Caveman in 
   the External Inputs Window.  You are to input the correct Month, which Sunday of the 
   Month, and at what hour (24 hour format) your broker/server starts DST.  Repeat the same 
   information for when your broker/server ends DST.  If you do not know when these times 
   are, obtain this information from your broker.
                                                                                                                                                                                 
----------------------------------------------------------------------------------------------
Color Suggestions:              White Chart         Black Chart

CentralPivot_Color              Magenta             Magenta
MainResistance_Pivots_Color     Tomato              FireBrick
MainSupport_Pivots_Color        C'64,64,255'        C'29,66,177'
MidPivots_Color                 Olive               C'85,85,0'    
PivotLabels_Color               MidnightBlue        C'148,148,148'
Separator_Color                 Silver              C'73,73,73'
SeparatorLabels_Color           Gray                C'128,128,128'
TzTest_Panel_Box_Color          C'233,233,252'      C'50,0,70'
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

//---- External Inputs
extern string Part_1                          = "Indicator Display Controls:";
extern bool   Indicator_On                    = true;
extern int    Indicator_Display_Max_TF        = 60;
extern bool   Use_Day_Formula                 = true;
extern bool   Time_Shifted_Pivots_On          = false;
extern int    DST_Server_GMT_Offset           = 0;   
extern int    DST_Choice_GMT_Offset           = 0;
extern bool   Use_For_Forex                   = true;
extern bool   Display_5_Digits                = false;
extern bool   Apply_Default_Color_Selections  = true;
extern bool   Apply_On_White_vs_Black_Chart   = true;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string __                              = "";
extern string Part_2                          = "Pivot Lines Start/Stop Settings:";
extern string note1                           = "1 = Full screen";
extern string note2                           = "2 = Current Session";
extern string note3                           = "3 = Session start";
extern string note4                           = "4 = Last candle start";
extern int    Line_Selection_Number           =  2;
 
extern string ___                             = "";
extern string Part_3                          = "Pivot Lines Appearance Settings:";
extern bool   CentralPivot_On                 = true;
extern bool   MainPivots_On                   = true; 
extern bool   MidPivots_On                    = false;
extern color  CentralPivot_Color              = Magenta; 
extern int    CP_LineStyle_01234              = 2;
extern int    CP_SolidLineThickness           = 1;
extern color  Resistance_Pivots_Color         = Tomato; 
extern int    R_LineStyle_01234               = 2;
extern int    R_SolidLineThickness            = 1; 
extern color  Support_Pivots_Color            = C'64,64,255';
extern int    S_LineStyle_01234               = 2;
extern int    S_SolidLineThickness            = 1; 
extern color  MidPivots_Color                 = Olive;
extern int    mP_LineStyle_01234              = 2;
extern int    mP_SolidLineThickness           = 1; 

extern string ____                            = "";     
extern string Part_4                          = "Pivot Label Settings:"; 
extern color  PivotLabels_Color               = MidnightBlue;
extern bool   PivotLabels_On                  = true;  
extern string PivotLabels_FontStyle           = "Verdana";
extern int    PivotLabels_FontSize            = 8;
extern bool   PivotLabels_Prices              = true;
extern bool   Subodinate_Pivot_Labels         = false;
extern bool   PivotLabels_Max_Right_Override  = false;

extern string _____                           = "";
extern string Part_5                          = "Day Separators Settings:";
extern bool   Separator_Yesterday_On          = true;
extern bool   Separator_Today_On              = true;
extern bool   Separator_Tomorrow_On           = true;
extern bool   Separator_TzShift_On            = false;
extern color  Separator_Color                 = Silver;
extern int    Separator_LineStyle_01234       = 2;
extern int    Separator_SolidLineThickness    = 1;
extern bool   SeparatorLabels_On              = true;
extern color  SeparatorLabels_Color           = Gray; 
extern string SeparatorLabels_FontStyle       = "Verdana Bold";
extern int    SeparatorLabels_FontSize        = 8;
extern bool   SeparatorLabels_Chart_Bottom    = false;
extern int    SeparatorLabels_Dist_To_Border  = 2;
  
// Start - Added by CaveMan to cater for DST
extern string ______                          = "";
extern string Part_6                          = "DST Zone Settings:";
extern string note5                           = "Enter preset DST zone number 1 - 4";
extern string note6                           = "or enter ' 5 ' for a custom DST zone.";
extern string note7                           = "1 US   2 EU   3 AUS   4 NZ   5 Cust.";
extern int    DST_Zone                        =  2;
extern string note8                           = "To customized A DST setting -";  
extern string note9                           = "Enter # month DST starts,";
extern int    Custom_DST_Start_Month          = 3; //March
extern string note10                          = "Enter # Sunday DST starts,";
extern string note11                          = "1-1st Sunday, 2-2nd ....7-Last Sunday";
extern int    Custom_DST_Start_Sunday         = 2; //2nd Sunday of March start DST
extern string note12                          = "Enter 24 hour time DST starts,";
extern string Custom_DST_Start_Time           = "02:00"; //DST starting time
extern string note13                          = "Repeat for DST end,";
extern int    Custom_DST_End_Month            = 11; //November
extern int    Custom_DST_End_Sunday           = 1;  //1st Sunday of November end DST
extern string Custom_DST_End_Time             = "02:00"; //DST ending time
// End - Added by CaveMan to cater for DST

extern string _______                         = "";
extern string Part_7                          = "Tz Test Section:";
extern color  TzTest_Panel_Box_Color          = C'233,233,252';
extern bool   Show_TzTest_Panel               = false;

//---- Buffers, Constants, Variables-----------------------------------------------------------
bool     FLAG_deinit;
int      a, b, c, A, B, DZ, E, F,T, Z, i, j, k, digits, fxDigits; 
double   p,d,q,Ra,q2; 
double   r1,r2,r3,r4,r5, s1,s2,s3,s4,s5; 
double   YadjustTop,YadjustBot,level;
string   Yesterday,Today,Tomorrow,Price,spc,tabR,tabL,GMT, dt;
 
// Start - Added by CaveMan/modified by TAH to cater for DST
int      Non_DST_Server_GMT_Offset, LastRefresh, First_Time; //original_server_time,
// End - Added by CaveMan/modified by TAH to cater for DST

datetime starttoday, startyesterday, starttomorrow, startline, stopline, startlabel, 
         startpivottoday, noshiftdaystart;
double   today_high, today_low, today_open, yesterday_high, yesterday_open, yesterday_low, 
         yesterday_close, true_yesterday_high, true_yesterday_low;
int      idxfirstbartoday, idxlastbartoday, idxfirstbaryesterday, idxlastbaryesterday,  
         idxstartyesterday, idxendyesterday;  
                                      
//+-------------------------------------------------------------------------------------------+
//| Program Initialization                                                                    |
//+-------------------------------------------------------------------------------------------+
int init()
   { 
   FLAG_deinit  = false; 

// Start - Added by CaveMan/modified by TAH to cater for DST
   //original_server_time           = DST_Server_GMT_Offset;
   if(DST_Server_GMT_Offset != 0) {
      Non_DST_Server_GMT_Offset = DST_Server_GMT_Offset - 1;}  
   LastRefresh                    = TimeHour(iTime(NULL,0,0));
   First_Time                     = true;
// End - Added by CaveMan/modified by TAH to cater for DST

   if (Use_For_Forex)
      {
      string sub=StringSubstr(Symbol(), 3, 3);
      if (Display_5_Digits) {if (sub == "JPY") {fxDigits = 3;} else {fxDigits = 5;}}
      else {if (sub == "JPY") {fxDigits = 2;} else {fxDigits = 4;}}
      }
      
   if (Apply_Default_Color_Selections)
      {         
      if (Apply_On_White_vs_Black_Chart)
         {
         CentralPivot_Color              = Magenta; 
         CP_LineStyle_01234              = 0;
         CP_SolidLineThickness           = 1;
         Resistance_Pivots_Color         = Tomato; 
         R_LineStyle_01234               = 0;
         R_SolidLineThickness            = 1; 
         Support_Pivots_Color            = C'64,64,255'; 
         S_LineStyle_01234               = 0;
         S_SolidLineThickness            = 1; 
         MidPivots_Color                 = Olive;
         mP_LineStyle_01234              = 0;
         mP_SolidLineThickness           = 1;
         PivotLabels_Color               = MidnightBlue;          
         PivotLabels_FontStyle           = "Verdana";
         PivotLabels_FontSize            = 8;
         Separator_Color                 = Silver;
         Separator_LineStyle_01234       = 2;
         Separator_SolidLineThickness    = 1; 
         SeparatorLabels_Color           = Gray; 
         SeparatorLabels_FontStyle       = "Verdana Bold";
         SeparatorLabels_FontSize        = 8; 
         TzTest_Panel_Box_Color          =  C'233,233,252';              
         } 
      else
         {
         CentralPivot_Color              = Magenta;  //BurlyWood; 
         CP_LineStyle_01234              = 0;
         CP_SolidLineThickness           = 1;
         Resistance_Pivots_Color         = FireBrick; 
         R_LineStyle_01234               = 0;
         R_SolidLineThickness            = 1; 
         Support_Pivots_Color            = C'29,66,177';
         S_LineStyle_01234               = 0;
         S_SolidLineThickness            = 1; 
         MidPivots_Color                 = C'85,85,0';
         mP_LineStyle_01234              = 0;
         mP_SolidLineThickness           = 1;
         PivotLabels_Color               = C'148,148,148';  
         PivotLabels_FontStyle           = "Verdana";
         PivotLabels_FontSize            = 8;
         Separator_Color                 = C'73,73,73';  
         Separator_LineStyle_01234       = 2;
         Separator_SolidLineThickness    = 1; 
         SeparatorLabels_Color           = C'128,128,128';    
         SeparatorLabels_FontStyle       = "Verdana Bold";
         SeparatorLabels_FontSize        = 8; 
         TzTest_Panel_Box_Color          = C'50,0,70';                       
         }
      }
                                     
   return(0);
   }
   
//+-------------------------------------------------------------------------------------------+
//| Program De-initialization                                                                 |
//+-------------------------------------------------------------------------------------------+
int deinit()
   {
   int obj_total= ObjectsTotal();  
   for (k= obj_total; k>=0; k--){
      string name= ObjectName(k);           
      if (StringSubstr(name,0,11)=="[TzPivotsD]") {ObjectDelete(name);}}
   Comment(""); 
   return(0);
   }
   
//+-------------------------------------------------------------------------------------------+
//| Program Start                                                                             |
//+-------------------------------------------------------------------------------------------+
int start()
{
   //If Indicator is "Off" or chart is out of range deinitialize only once, not every tick-----  
   if((!Indicator_On) || (Period() > Indicator_Display_Max_TF)) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
      
   //If the Line_Selection_Number is out of range, exit----------------------------------------          
   if(Line_Selection_Number <1 || Line_Selection_Number >4)
      {
      Alert("SonicR TzPivots:  Line_Selection_Number invalid"); return(false);
      }  
      
   //Else indicator is "On", TF/line settings are in display range.  Clear variables & go-------                   
   deinit(); FLAG_deinit = false;   
   
// Start - Added by CaveMan to cater for DST
// Code will only check for DST on every new hour bar
   if(TimeHour(iTime(NULL,0,0)) == 0) LastRefresh = -1; // to cater for crossover into a new day
   if(First_Time || TimeHour(iTime(NULL,0,0)) > LastRefresh){
      if (Adjust_for_DST()) {Print(" TzPivot - Daylight Savings Time (DST) is in effect.");}
      First_Time = false; LastRefresh = TimeHour(iTime(NULL,0,0));}
// End - Added by CaveMan to cater for DST

   //Tz shift code,let's find out which hour bars make today and yesterday----------------------
   if (!Time_Shifted_Pivots_On) {DST_Choice_GMT_Offset = DST_Server_GMT_Offset;}                        
   TzShift(DST_Server_GMT_Offset,DST_Choice_GMT_Offset,idxfirstbartoday, idxfirstbaryesterday, 
           idxlastbaryesterday, idxstartyesterday, idxendyesterday); 

   //For pivot lines, until first bar of today ends, use time of previous bar
   startpivottoday = Time[1];
   if (Time[0] > iTime(NULL, PERIOD_H1, idxfirstbartoday)) {startpivottoday = iTime(NULL, PERIOD_H1, idxfirstbartoday);} 

   //Walk forward through yestday's start and collect high/lows within the same day
   true_yesterday_high= -99999;  // not high enough to remain alltime high
   true_yesterday_low=  +99999;  // not low enough to remain alltime low 
   for (int idxbar= idxstartyesterday; idxbar>=idxendyesterday; idxbar--)
      {
      true_yesterday_high= MathMax(iHigh(NULL, PERIOD_H1, idxbar), true_yesterday_high);
      true_yesterday_low= MathMin(iLow(NULL, PERIOD_H1, idxbar), true_yesterday_low);      
      }   
   
   //Walk forward through yestday's start and collect high/lows within the same day
   yesterday_high= -99999;  // not high enough to remain alltime high
   yesterday_low=  +99999;  // not low enough to remain alltime low 
   for (idxbar= idxfirstbaryesterday; idxbar>=idxlastbaryesterday; idxbar--)
      {
      // grab first value for open
      if (yesterday_open==0) yesterday_open= iOpen(NULL, PERIOD_H1, idxbar);      
      yesterday_high= MathMax(iHigh(NULL, PERIOD_H1, idxbar), yesterday_high);
      yesterday_low= MathMin(iLow(NULL, PERIOD_H1, idxbar), yesterday_low);      
      yesterday_close= iClose(NULL, PERIOD_H1, idxbar);
      }

   //Walk forward through today and collect high/lows within the same day
   today_open= iOpen(NULL, PERIOD_H1, idxfirstbartoday); 
   today_high= -99999; 
   today_low=  +99999; 
   for (j= idxfirstbartoday; j>=0; j--)
      {
      today_high= MathMax(today_high, iHigh(NULL, PERIOD_H1, j));
      today_low= MathMin(today_low, iLow(NULL, PERIOD_H1, j));
      }
     
   //Define yesterday/today data used to calculate pivot levels--------------------------------
   // double p,d,q;   
   d = (today_high - today_low);
   q = (yesterday_high - yesterday_low);
   p = (yesterday_high + yesterday_low + yesterday_close) / 3;       

   //Calculate Pivot Levels--------------------------------------------------------------------     
   if(Use_Day_Formula) 
   {
   r5 = (4*p)+(yesterday_high-(4*yesterday_low));
   r4 = (3*p)+(yesterday_high-(3*yesterday_low));
   r3 = (2*p)+(yesterday_high-(2*yesterday_low));
   r2 = p+(yesterday_high - yesterday_low);  //r2 = p-s1+r1;
   r1 = (2*p)-yesterday_low;  
   s1 = (2*p)-yesterday_high;
   s2 = p-(yesterday_high - yesterday_low);  //s2 = p-r1+s1;
   s3 = (2*p)-((2* yesterday_high)-yesterday_low);
   s4 = (3*p)-((3* yesterday_high)-yesterday_low);
   s5 = (4*p)-((4* yesterday_high)-yesterday_low);
   }    
   if(!Use_Day_Formula)
   {
   r5 = p+ (q * 2.618);
   r4 = p+ (q * 1.618);
  	r3 = p+q;  
   r2 = p+ (q * 0.618); 
   r1 = p+ (q * 0.382); 
   s1 = p- (q * 0.382);     
   s2 = p- (q * 0.618);    
  	s3 = p-q; 
   s4 = p- (q * 1.618);
   s5 = p- (q * 2.618);  
   }

   //Central Pivot-----------------------------------------------------------------------------
   if(CentralPivot_On) {   
   if(Use_Day_Formula) {     
   Pivots("DPV", p, CentralPivot_Color, CP_LineStyle_01234, CP_SolidLineThickness); }  
   if(Use_Day_Formula == false) {  
   Pivots("FPV", p, CentralPivot_Color, CP_LineStyle_01234, CP_SolidLineThickness); }}
                 
   //Main Pivots-------------------------------------------------------------------------------
   if(MainPivots_On) {   
   if(R_LineStyle_01234>0){R_SolidLineThickness=0;} 
   if(S_LineStyle_01234>0){S_SolidLineThickness=0;}       
   if(Use_Day_Formula) {
   Pivots("DR5", r5, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);  
   Pivots("DR4", r4, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);   
   Pivots("DR3", r3, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);   
   Pivots("DR2", r2, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness); 
   Pivots("DR1", r1, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);   
   Pivots("DS1", s1, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);
   Pivots("DS2", s2, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);
   Pivots("DS3", s3, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);  
   Pivots("DS4", s4, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);
   Pivots("DS5", s5, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);}   
      
   else if(!Use_Day_Formula) {
   Pivots("FR5", r5, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);  
   Pivots("FR4", r4, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);   
   Pivots("FR3", r3, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);   
   Pivots("FR2", r2, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness); 
   Pivots("FR1", r1, Resistance_Pivots_Color, R_LineStyle_01234, R_SolidLineThickness);   
   Pivots("FS1", s1, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);
   Pivots("FS2", s2, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);
   Pivots("FS3", s3, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);   
   Pivots("FS4", s4, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);
   Pivots("FS5", s5, Support_Pivots_Color, S_LineStyle_01234, S_SolidLineThickness);}}
   
   //MidPivots---------------==========-------------------------------------------------------
   if (MidPivots_On) {   
   if(mP_LineStyle_01234>0){mP_SolidLineThickness=0;}
   Pivots("MR5", (r4+r5)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MR4", (r3+r4)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness); 
   Pivots("MR3", (r2+r3)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MR2", (r1+r2)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MR1", (p+r1)/2,  MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MS1", (p+s1)/2,  MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MS2", (s1+s2)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MS3", (s2+s3)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);          
   Pivots("MS4", (s3+s4)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness);
   Pivots("MS5", (s4+s5)/2, MidPivots_Color, mP_LineStyle_01234, mP_SolidLineThickness); }    
  
   //Draw the Period Separators and labels-----------------------------------------------------
   if(Period() < 1440) 
      {
      if((Separator_Today_On) || (Separator_Yesterday_On) || 
         (Separator_Tomorrow_On) || (Separator_TzShift_On)) 
         {     
         //TzShift Separator label content adjustments
         DZ = DST_Server_GMT_Offset;
         if(DZ>=0) {GMT = "GMT+"+ DZ;}
         if(DZ< 0) {GMT = "GMT" + DZ;}
            
         //Separator label position adjustments        
         if (SeparatorLabels_Dist_To_Border < 1) {SeparatorLabels_Dist_To_Border = 1;}   
      	YadjustTop = (WindowPriceMax()-WindowPriceMin())/(9000/SeparatorLabels_FontSize); //;//5000
      	YadjustTop = YadjustTop + (SeparatorLabels_Dist_To_Border * YadjustTop); 
      	YadjustBot = (WindowPriceMax()-WindowPriceMin())/(500/SeparatorLabels_FontSize); //375	
      	YadjustBot = YadjustBot + ((SeparatorLabels_Dist_To_Border * YadjustBot)/20); 
         level = WindowPriceMax() - YadjustTop; if (SeparatorLabels_Chart_Bottom){level = WindowPriceMin() + YadjustBot;}
         
         //Prepare inputs for subroutine that draws Separator lines and labels
         if(Separator_LineStyle_01234>0) {Separator_SolidLineThickness=1;}	                         
         if(Separator_Yesterday_On)
            {
            startyesterday = iTime(NULL, PERIOD_D1, 1);                
            Separator("YesterdayStart", Yesterday, startyesterday, SeparatorLabels_Color,level);              
            }
         if(Separator_Today_On) 
            {
            starttoday = iTime(NULL, PERIOD_D1, 0);                               
            Separator("TodayStart", Today, starttoday, SeparatorLabels_Color,level);               
            }                 
        if(Separator_Tomorrow_On)
            {
            starttomorrow = iTime(NULL, PERIOD_D1, 0) +86400;                                                
            Separator("TomorrowStart", Tomorrow, starttomorrow, SeparatorLabels_Color,level);               
            } 
        if(Separator_TzShift_On)
            {
            starttoday = iTime(NULL,PERIOD_H1, idxfirstbartoday) + (DZ*3600);                                      
            Separator("TzShiftStart", GMT, starttoday, SeparatorLabels_Color,level);                  
            }     
         }                          
      } 
     
   return(0); //Finished 
}
   
//+-------------------------------------------------------------------------------------------+
//| Pivots sub-routine to name and draw pivot lines and labels                                |                                                                                  |
//+---------------------------------------------------------------------------------------- --+
void Pivots(string text, double level, color col1, int linestyle, int thickness) 
   {
   //Lines=====================================================================================    
   //Name lines 
   text = StringTrimLeft(text);
   string pivotline  = "[TzPivotsD] " + text + " Line"; 

   //Define variables   
   a = linestyle; b = thickness; c =1; if (a==0)c=b;  
   
   //Where to Start lines
   if(Line_Selection_Number == 1) {startline = WindowFirstVisibleBar();}
   else if(Line_Selection_Number == 2 || Line_Selection_Number == 3) {startline = iTime(NULL, PERIOD_D1, 0);} //start at day separator 
   else {startline  = Time[1];} //start at last closed candle                    
                                                                      
   //Where to Stop Lines
   if(Line_Selection_Number == 1 || Line_Selection_Number == 3 || Line_Selection_Number == 4) {stopline = Time[0]; F = true;}
   else //if(Line_Selection_Number == 2)     
      {
      T=24;
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + T + ":" + "60";                                        
      T = StrToTime(dt)+60; 
      stopline = T; F = false;          
      }    
       
   //Draw lines 
   if (ObjectFind(pivotline) != 0){
      ObjectCreate(pivotline, OBJ_TREND, 0, startline, level, stopline, level);
      ObjectSet(pivotline, OBJPROP_STYLE, linestyle);
      ObjectSet(pivotline, OBJPROP_COLOR, col1);
      ObjectSet(pivotline, OBJPROP_WIDTH, c);
      ObjectSet(pivotline, OBJPROP_BACK, true); 
      ObjectSet(pivotline, OBJPROP_RAY, F);}
   else{
      ObjectMove(pivotline, 0, startline, level);
      ObjectMove(pivotline, 1, stopline, level);}
           
   //Labels===================================================================================
   if (!PivotLabels_On) return (-1); 
   
   else
      {
      //Name label
      string pivotlabel = "[TzPivotsD] " + text + " Label";
           
      //Append asterisk when shifted pivots are displayed     
      //if(DST_Server_Offset_From_GMT != DST_Choice_Offset_From_GMT) {text = text + "*";} 
      if(Time_Shifted_Pivots_On) {text = text + "*";} 
      //Append space when normal pivots are displayed (to keep spacing same)      
      else {text = text + " ";}
                       
      //Define the standard digits used for Price
      if (PivotLabels_Prices) {
      if (Use_For_Forex) {Price=DoubleToStr(level, fxDigits);}
      else {Price=DoubleToStr(level, Digits);} 
      if (StrToInteger(text)==0) {text = text + " " + Price;} }    
 
      //Set labels starting point and alignment spacing (independent of manual tabs)
      if (PivotLabels_Max_Right_Override)  //start Max Right
         {
         spc="                                         "; //41
         if(PivotLabels_Prices) {spc="                                                    ";} //52
         startlabel = Time[0];       
         } 
         
      else if (Line_Selection_Number == 1) //start mid chart
         {
         spc = " "; //01
         if(PivotLabels_Prices ){spc="            ";} //12 
         startlabel = Time[WindowFirstVisibleBar()/2];
         }         
                        
      else if (Line_Selection_Number == 2 || Line_Selection_Number == 3)            
         if (Time[WindowFirstVisibleBar()] < iTime(NULL, PERIOD_D1, 0)) //start at day separator
            {
            spc="        "; //08
            if(PivotLabels_Prices ){spc="                   ";}  //19
            startlabel = iTime(NULL, PERIOD_D1, 0);     
            }          
         else //start mid chart        
            {
            spc = " "; //01
            if(PivotLabels_Prices ){spc="            ";} //12 
            startlabel = Time[WindowFirstVisibleBar()/2];                    
            }  
                                              
      else //if(Line_Selection_Number == 4) //start at current candle
         {
         spc="          ";  //10   
         if(PivotLabels_Prices ){spc="                     ";} //21      
         startlabel = Time[0];   
         }             
 
      //Draw labels                             
      if (ObjectFind(pivotlabel) != 0){
         ObjectCreate(pivotlabel, OBJ_TEXT, 0, startlabel, level);     
         ObjectSetText(pivotlabel, spc + text, PivotLabels_FontSize, PivotLabels_FontStyle, PivotLabels_Color);
         if (Subodinate_Pivot_Labels) {ObjectSet(pivotlabel, OBJPROP_BACK, true);} 
         else {ObjectSet(pivotlabel, OBJPROP_BACK, false);}}       
      else{ObjectMove(pivotlabel, 0, startlabel, level);} 
      }       
   }
   
//+-------------------------------------------------------------------------------------------+
//| Separator sub-routine to name and draw separator lines and labels                         |                                                        
//+-------------------------------------------------------------------------------------------+
void Separator(string objname, string text, int x, color col1, double vleveltext) 
   {
   string vline= "[TzPivotsD] " + objname, vlabel = vline + " Label";
   
   if (ObjectFind(vline) != 0){ 
      ObjectCreate(vline, OBJ_TREND, 0, x, 0, x, 100);
      ObjectSet(vline, OBJPROP_STYLE, Separator_LineStyle_01234);
      ObjectSet(vline, OBJPROP_COLOR, Separator_Color);
      ObjectSet(vline, OBJPROP_WIDTH, Separator_SolidLineThickness);
      ObjectSet(vline, OBJPROP_BACK, true);}
   else{
      ObjectMove(vline, 0, x, 0); 
      ObjectMove(vline, 1, x, 100);}
        
   if(SeparatorLabels_On){                           
      if (ObjectFind(vlabel) != 0){
         ObjectCreate (vlabel, OBJ_TEXT, 0, x, vleveltext);
         ObjectSetText(vlabel, text, SeparatorLabels_FontSize, SeparatorLabels_FontStyle, SeparatorLabels_Color); 
         ObjectSet(vlabel, OBJPROP_BACK, true);}               
      else{ObjectMove(vlabel, 0, x, vleveltext);}}
   }

//+-------------------------------------------------------------------------------------------+
//| TzShift sub-routine computes index of first/last bars of yesterday & first bar  of today  |
//+-------------------------------------------------------------------------------------------+
void TzShift(int tzServer, int tzChoice, int &idxfirstbartoday, int &idxfirstbaryesterday,int &idxlastbaryesterday,
             int &idxstartyesterday, int &idxendyesterday)//CaveMan adds 2 variables to decouple yHL & Yest/Today separators                
   {      
   int tzdiff               = tzServer - tzChoice,
       tzdiffsec            = tzdiff*3600, 
       barsperday           = (1440/PERIOD_H1),      
       dayofweektoday       = TimeDayOfWeek(iTime(NULL, PERIOD_H1, 0) - tzdiffsec),
       dayofweektofind      = -1;     
                
   //Due to gaps in the data, and shift of time around weekends (due to time zone)
   //it is not as easy as to just look back for a bar with 00:00 time  
   idxfirstbartoday= 0; idxfirstbaryesterday= 0;  idxlastbaryesterday= 0;     
   switch (dayofweektoday) { 
      case 0: dayofweektofind = 5; break; // Sun (Sunday considered as short trading day)
      default: dayofweektofind = dayofweektoday -1; break;}                    

   //Search backwards for the last occurence of a "today's first bar"
   for (int i=1; i<=barsperday + 1; i++){
      datetime timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
      if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
   for (int j= 0; j<=2*barsperday + 1; j++) {
      datetime timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
      if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
   for (j= 1; j<=barsperday; j++) {
      datetime timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
      if(TimeDayOfWeek(timey2) != dayofweektofind){idxfirstbaryesterday = idxlastbaryesterday+j-1; break;}}
      
        
// Start - Added by CaveMan to cater for missing/partial previous day's data  
   //This section added to assign yesterday's start & end index for use by yHL & Yest/Today separators
   // Search for Friday if yesterday is a Sunday
   if (dayofweektofind != 0)
         {
          idxstartyesterday = idxfirstbaryesterday; // Assign yesterday's first bar location to index
          idxendyesterday = idxlastbaryesterday;    // Assign yesterday's last bar location to index
         }
   else // Search for Friday's data range
         {
            //Search backwards for the last occurence of a "today's first bar"
            for (i=1; i<=barsperday + 1; i++){
               timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
               if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
            for (j= 0; j<=2*barsperday + 1; j++) {
               timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
               if(TimeDayOfWeek(timey) == 5) {idxlastbaryesterday = i+j; break;}}
            for (j= 1; j<=barsperday; j++) {
               timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
               if(TimeDayOfWeek(timey2) != 5){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}

           idxstartyesterday = idxfirstbaryesterday; // Assign yesterday's first bar location to index
           idxendyesterday = idxfirstbartoday + 1;    // Assign yesterday's last bar location to index
          }
   //This section takes care of the Christmas and New Year Day problem where there is a day of partial data on the eve
   //of these holidays followed by an entire day without any data at all. The workaround to handle this on market
   //open after the holiday is to use data from the day before the eve of the holiday right up to the end of the eve of
   //the holiday for pivot computation. E.g. On the 26 December market opens after Christmas. Use full day data of 23rd 
   //plus partial data of 24th Dec to compute the pivots. Same applies to New Year Day.
   int YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
   if (YesterdayBarsTotal == 0)   
      {
         if (dayofweektofind == 0) { dayofweektofind = 4;} 
         else if (dayofweektofind == 1) { dayofweektofind = 5;} 
         else {dayofweektofind = dayofweektofind - 1;}

         //Repeat this search with the new dayofweektofind information
         for (i=1; i<=barsperday + 1; i++){
            timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
            if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
         for (j= 0; j<=2*barsperday + 1; j++) {
            timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
            if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
         for (j= 1; j<=3*barsperday; j++) {
            timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
            if(TimeDayOfWeek(timey2) != dayofweektofind){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
         idxstartyesterday = idxfirstbaryesterday; // Assign yesterday's first bar location to index
         idxendyesterday = idxlastbaryesterday;    // Assign yesterday's last bar location to index
         YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
       }      
   //Standard codes to handle days of partial data (including Fridays and Sundays)
   //If one hour bars < 24, use additional data from one more day back 
   if (YesterdayBarsTotal < 23)
      { 
         if (dayofweektoday == 0 && dayofweektofind == 4) {dayofweektofind = 3;} // for Sunday, search Thursday and found Thursday partial data, then search Wednesday.
         else if (dayofweektoday == 1 && dayofweektofind == 4) {dayofweektofind = 4;} // for Monday, Friday no data then include Thursday and Sunday data.
         else if (dayofweektofind == 0 || dayofweektofind == 1) { dayofweektofind = 5;} 
         else { dayofweektofind = dayofweektofind - 1; }
            //Search backwards for the last occurence of a "today's first bar"
            for (i=1; i<=barsperday + 1; i++){
               timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
               if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
            for (j= 0; j<=3*barsperday + 1; j++){ // Need to increase another 24 hours to search
               timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
               if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
            for (j= 1; j<=4*barsperday; j++) {
               timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
               if(TimeDayOfWeek(timey2) != dayofweektofind){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}     
   //This small section handles a very unique combination of events that coincide during the short opening trading hours 
   //of the Sunday following Christmas Day and New Year Day if both of these holidays happen to fall on a Thursday. 
   //This occured on the short Sunday trading hours before Monday kicks in. For those few hours, pivot used on Friday are 
   //extended into Sunday. These pivot levels are calculated using full Thursday's data. In this unique situation, 
   //Thursday was New Year Day (major public holiday)with no data at all. This will cause TzPivot to malfunction in similar 
   //situation in the future. As a workaround, for those short few hours of Sunday, if a similar situation should occur,
   //partial data of Wednesday and Friday will be used for pivot computation. 
         YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
         if (YesterdayBarsTotal == 0)   
            {
               if (dayofweektofind == 1) { dayofweektofind = 5;}
               else {dayofweektofind = dayofweektofind - 1;}
               //Repeat this search with the new dayofweektofind information
               for (i=1; i<=barsperday + 1; i++){
                  timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
                  if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
               for (j= 0; j<=2*barsperday + 1; j++) {
                  timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
                  if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
               for (j= 1; j<=3*barsperday; j++) {
                  timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
                  if(TimeDayOfWeek(timey2) != dayofweektofind){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
               idxlastbaryesterday = idxfirstbartoday + 1;
             }
   //Include partial day's data and combine with previous day's data for pivot computation. Do not include for Sundays
   //as Sunday short trading hours uses Friday's pivots that are computed from full Thursday's data.
   //This is more for handling of holidays including Christmas and New Year Day. 
            if (dayofweektoday != 0) idxlastbaryesterday = idxfirstbartoday + 1; 
       }      
   //Comments for troubleshooting purposes.       {
   if (Show_TzTest_Panel) { 
      int y = 92; 
      
      ObjectCreate("[TzPivotsD] TZbox1", OBJ_LABEL, 0, 0, 0, 0, 0);
      ObjectSetText("[TzPivotsD] TZbox1", "g", 110, "Webdings");
      ObjectSet("[TzPivotsD] TZbox1", OBJPROP_CORNER, 0);
      ObjectSet("[TzPivotsD] TZbox1", OBJPROP_XDISTANCE, 0);
      ObjectSet("[TzPivotsD] TZbox1", OBJPROP_YDISTANCE, y);                 
      ObjectSet("[TzPivotsD] TZbox1", OBJPROP_COLOR, TzTest_Panel_Box_Color);
      ObjectSet("[TzPivotsD] TZbox1", OBJPROP_BACK, false); 
           
      ObjectCreate("[TzPivotsD] TZbox2", OBJ_LABEL, 0, 0, 0, 0, 0);
      ObjectSetText("[TzPivotsD] TZbox2", "g", 110, "Webdings");
      ObjectSet("[TzPivotsD] TZbox2", OBJPROP_CORNER, 0);
      ObjectSet("[TzPivotsD] TZbox2", OBJPROP_XDISTANCE, 131);
      ObjectSet("[TzPivotsD] TZbox2", OBJPROP_YDISTANCE, y);                 
      ObjectSet("[TzPivotsD] TZbox2", OBJPROP_COLOR, TzTest_Panel_Box_Color);
      ObjectSet("[TzPivotsD] TZbox2", OBJPROP_BACK, false);       
                        
      string C; Comment(C,"\n","\n","\n","\n","\n","\n",
      "\n    -------------------   TZ  SHIFT  DATA   ------------------\n", 
      "\n    High, Low, Close         =   ", yesterday_high, " / ", yesterday_low, " / ", yesterday_close,
      "\n    No. of yesterday bars   =   ", YesterdayBarsTotal,
      "\n    Day of week today      =   ", dayofweektoday,
      "\n    Day of week to find     =   ", dayofweektofind,
      "\n    Today Start                =   ", TimeToStr(iTime(NULL,PERIOD_H1,idxfirstbartoday), TIME_DATE|TIME_MINUTES), " (",idxfirstbartoday , ") ",
      "\n    Pivot Range Start        =   ", TimeToStr(iTime(NULL,PERIOD_H1,idxfirstbaryesterday), TIME_DATE|TIME_MINUTES)," (",idxfirstbaryesterday ,") ",
      "\n    Pivot Range End         =   ", TimeToStr(iTime(NULL,PERIOD_H1,idxlastbaryesterday), TIME_DATE|TIME_MINUTES), " (",idxlastbaryesterday,") ",
      "\n    Separator Start            =   ", TimeToStr(iTime(NULL,PERIOD_H1,idxstartyesterday), TIME_DATE|TIME_MINUTES)," (",idxstartyesterday ,") ",
      "\n    Separator End             =   ", TimeToStr(iTime(NULL,PERIOD_H1,idxendyesterday), TIME_DATE|TIME_MINUTES), " (",idxendyesterday,") ");}             
// End - Added by CaveMan to cater for missing/partial previous day's data        

      int today_start = TimeDayOfWeek(iTime(NULL,PERIOD_H1, idxfirstbartoday) - tzdiffsec);
      int yesterday_start = TimeDayOfWeek(iTime(NULL,PERIOD_H1, idxstartyesterday) - tzdiffsec);//CaveMan modified from idxfirstbaryesterday
      int tomorrow_start = TimeDayOfWeek(iTime(NULL,PERIOD_H1, idxfirstbartoday) - tzdiffsec + 86400);              

      switch(yesterday_start){
         case 0: Yesterday = " Sunday"; break;
         case 1: Yesterday = " Monday"; break;
         case 2: Yesterday = "  Tuesday"; break;
         case 3: Yesterday = "Wednesday"; break;
         case 4: Yesterday = " Thursday"; break;
         case 5: Yesterday = "  Friday";  break;}  
                
      switch(today_start){
         case 0: Today     = " Sunday"; break;
         case 1: Today     = " Monday"; break;
         case 2: Today     = "  Tuesday"; break;
         case 3: Today     = "Wednesday"; break;
         case 4: Today     = " Thursday"; break;
         case 5: Today     = "  Friday";  break;}               
           
      switch(tomorrow_start){
         case 0: Tomorrow  = " Sunday"; break;
         case 1: Tomorrow  = " Monday"; break;
         case 2: Tomorrow  = "  Tuesday"; break;
         case 3: Tomorrow  = "Wednesday"; break;
         case 4: Tomorrow  = " Thursday"; break;
         case 5: Tomorrow  = "  Friday";  break; 
         default:Tomorrow  = " Sunday"; break;}}     
                                
// Start - Added by CaveMan to cater for DST
//+-------------------------------------------------------------------------------------------+
//| Calculate and return actual date given which Sunday of the month information              |
//|  --> Only US and EU DST is supported for the time being.                                  |
//+-------------------------------------------------------------------------------------------+
bool Adjust_for_DST()
   {
   // This definition section can be made into extern for user configuration but not neccesary.
   string Instructions1       = "Enter which Sunday of the month";
   string Instructions2       = "DST should start and end.";
   string Instructions3       = "1,2,3,4,5,6 = 1st, 2nd, 3rd, 4th, 5th and 6th";
   string Instructions4       = "7 = Last Sundy of the month";

   int us_dst_month_start     = 3;        // March
   int us_dst_start_sunday    = 2;        // 2nd Sunday of March start DST     
   string us_dst_time_start   = "02:00";  // DST starting time
   int us_dst_month_end       = 11;       // November   
   int us_dst_end_sunday      = 1;        // 1st Sunday of November end DST   
   string us_dst_time_end     = "02:00";  // DST ending time
 
   int eu_dst_month_start     = 3;        // March
   int eu_dst_start_sunday    = 7;        // Last Sunday of March start DST  
   string eu_dst_time_start   = "02:00";  // DST starting time
   int eu_dst_month_end       = 10;       // October
   int eu_dst_end_sunday      = 7;        // Last Sunday of October end DST   
   string eu_dst_time_end     = "02:00";  // DST ending time
 
   int aus_dst_month_start    = 4;        // April
   int aus_dst_start_sunday   = 1;        // 1st Sunday of April start DST   
   string aus_dst_time_start  = "02:00";  // DST starting time
   int aus_dst_month_end      = 10;       // October
   int aus_dst_end_sunday     = 1;        // 1st Sunday of October end DST   
   string aus_dst_time_end    = "02:00";  // DST ending time

   int nz_dst_month_start     = 4;        // April
   int nz_dst_start_sunday    = 1;        // 1st Sunday of April start DST  
   string nz_dst_time_start   = "02:00";  // DST starting time
   int nz_dst_month_end       = 9;        // September
   int nz_dst_end_sunday      = 7;        // Last Sunday of September end DST   
   string nz_dst_time_end     = "02:00";  // DST ending time

   datetime us_dst_start,   us_dst_end;   // dst start and end datetime
   datetime eu_dst_start,   eu_dst_end; 
   datetime aus_dst_start, aus_dst_end; 
   datetime nz_dst_start,   nz_dst_end; 
   datetime cus_dst_start, cus_dst_end; 
      
   int cus_dst_month_start   = Custom_DST_Start_Month;
   int cus_dst_start_sunday  = Custom_DST_Start_Sunday;
   string cus_dst_time_start = Custom_DST_Start_Time;
   int cus_dst_month_end     =  Custom_DST_End_Month;   
   int cus_dst_end_sunday    = Custom_DST_End_Sunday;
   string cus_dst_time_end   =  Custom_DST_End_Time;  
   
   //Print("US DST Start/End Date = ", TimeToStr(us_dst_start), " / ", TimeToStr(us_dst_end));
   //Print("EU DST Start/End Date = ", TimeToStr(eu_dst_start), " / ", TimeToStr(eu_dst_end));
      
   us_dst_start  = check_date(us_dst_start_sunday,  us_dst_month_start,  us_dst_time_start);
   us_dst_end    = check_date(us_dst_end_sunday,    us_dst_month_end,    us_dst_time_end);
   eu_dst_start  = check_date(eu_dst_start_sunday,  eu_dst_month_start,  eu_dst_time_start);
   eu_dst_end    = check_date(eu_dst_end_sunday,    eu_dst_month_end,    eu_dst_time_end);
   aus_dst_start = check_date(aus_dst_start_sunday, aus_dst_month_start, aus_dst_time_start);
   aus_dst_end   = check_date(aus_dst_end_sunday,   aus_dst_month_end,   aus_dst_time_end);
   nz_dst_start  = check_date(nz_dst_start_sunday,  nz_dst_month_start,  nz_dst_time_start);
   nz_dst_end    = check_date(nz_dst_end_sunday,    nz_dst_month_end,    nz_dst_time_end);
   cus_dst_start = check_date(cus_dst_start_sunday, cus_dst_month_start, cus_dst_time_start);
   cus_dst_end   = check_date(cus_dst_end_sunday,   cus_dst_month_end,   cus_dst_time_end);
      
   if (DST_Zone == 1 && iTime(NULL,0,0) > us_dst_start && iTime(NULL,0,0) < us_dst_end)
      {
      return(true);
      }
   else if (DST_Zone == 2 && iTime(NULL,0,0) > eu_dst_start && iTime(NULL,0,0) < eu_dst_end)
      { 
      return(true);
      }
   else if (DST_Zone == 3 && iTime(NULL,0,0) > aus_dst_start && iTime(NULL,0,0) < aus_dst_end)
      {
      return(true);
      }
   else if (DST_Zone == 4 && iTime(NULL,0,0) > nz_dst_start && iTime(NULL,0,0) < nz_dst_end)
      {
      return(true);
      }
   else if (DST_Zone == 5 && iTime(NULL,0,0) > cus_dst_start && iTime(NULL,0,0) < cus_dst_end)
      {
      return(true);
      }
   else
      {
      DST_Server_GMT_Offset = Non_DST_Server_GMT_Offset;
      return(false);
      }
   }              

//+-------------------------------------------------------------------------------------------+
//| Calculate and return actual date given which Sunday of the month information              |
//+-------------------------------------------------------------------------------------------+
datetime check_date(int which_sunday, int which_month, string what_time)
{
   string start_date;
   int day_of_week, start_day;

   if (which_sunday == 7)  // Last Sunday of the month
      {
         for (int i = 31; i > 20; i--)
            {
               start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + i;
               day_of_week = TimeDayOfWeek(StrToTime(start_date));
               if (day_of_week == 0) // equals to Sunday
                  {
                     start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + i + " " + what_time;
                     break;
                   }
             } 
      }
   else // other Sundays of the month besides last Sunday of the month
      {
         start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + "1";
         day_of_week = TimeDayOfWeek(StrToTime(start_date));
   
         if (day_of_week == 0) start_day = 1 + 7*(which_sunday - 1);
         else start_day = 1 + (7 - day_of_week) + 7 * (which_sunday - 1);
   
         start_date = TimeYear(iTime(NULL,0,0)) + "." + which_month + "." + start_day + " " + what_time;
      }
      
   return(StrToTime(start_date));
}
// End - Added by CaveMan to cater for DST

//+-------------------------------------------------------------------------------------------+
//| Program End                                                                               |                                                        
//+-------------------------------------------------------------------------------------------+