//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                   SonicR Volume.mq4                                       |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+
#property copyright "Copyright  2010 traderathome"
#property link      "email:   traderathome@msn.com"

/*---------------------------------------------------------------------------------------------
Overview:

This indicator produces a multi-colored histogram with a line is drawn across it representing
the updated average volume at each bar, based upon the averaging period selected.

The histogram bars represent the count of trades executed each TF.  For easy visual recognition
there are only 3 color catagories of histogram bars: (1) below the average volume, 
(2) up to twice the average volume, (3) more than twice the average volume.

The indicator subwindow ShortName label displays the "Period" used for averaging, the computed
"Average" volume, and the "Bar" real time volume.

Default color settings are included for both white and black charts.

You can turn the indicator off, preserving your chart settings.  And you can specify a range 
of time frames beyond which the indicator will not display.  This allows you to include a
different indicator in the same subwindow to be used on different TFs.  For example, you might
want a Volume indy for TFs M30 and lower, and a MACD indy for TFs H1 and higher.

                                                                    - Traderathome, 12-01-2010  
-----------------------------------------------------------------------------------------------
Acknowledgements:

Miss Pips       - for the code enabling the real-time display of the current bar volume 
                  in the indicator ShortName.
Vadim Shumiloff - for the Normalized Volume Oscillator.mq4 which provided core code.  
     
-----------------------------------------------------------------------------------------------
Suggested Settings:         White Chart        Black Chart  

#property indicator_color1  Gainsboro       
#property indicator_color2  Turquoise     
#property indicator_color3  MediumPurple   
#property indicator_color4  Gray        
#property indicator_width1  2  
#property indicator_width2  2 
#property indicator_width3  2  
#property indicator_width4  2  

#property indicator_color5                     DarkSlateBlue  
#property indicator_color6                     LimeGreen    
#property indicator_color7                     White          
#property indicator_color8                     Red            
#property indicator_width5                     2    
#property indicator_width6                     2 
#property indicator_width7                     2  
#property indicator_width8                     2                                                        
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |
//+-------------------------------------------------------------------------------------------+
#property indicator_separate_window

#property indicator_buffers 8
#property indicator_color1  Gainsboro       //graduation 1 bars
#property indicator_color2  Turquoise       //graduation 2 bars
#property indicator_color3  MediumPurple    //graduation 3 bars
#property indicator_color4  Gray            //Average Line
#property indicator_width1  2  
#property indicator_width2  2 
#property indicator_width3  2  
#property indicator_width4  2  

#property indicator_color5  DarkSlateBlue   //graduation 1 bars
#property indicator_color6  LimeGreen       //graduation 2 bars
#property indicator_color7  White           //graduation 3 bars
#property indicator_color8  Red             //Average Line
#property indicator_width5  2    
#property indicator_width6  2 
#property indicator_width7  2  
#property indicator_width8  2 

//Global External Inputs                                                  
extern bool   Indicator_On                  = true;
extern int    Averaging_Period              = 20;
extern bool   Apply_On_White_vs_Black_Chart = true;
extern int    Display_Min_TF                = 1;
extern int    Display_Max_TF                = 30;
extern string TF_Choices_1                  = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                  = "D  W  M =  1440,  10080,  43200";

//Global Buffers & Other Inputs
bool     FLAG_deinit;
double   BufferH1[];
double   BufferH2[];
double   BufferH3[];
double   BufferH4[];
double   BufferH5[];
double   BufferH6[];
double   BufferH7[];
double   BufferH8[];   
double   avg, nvo; 
int      i, j, counted_bars;
string   Avg_Volume;

//+-------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                  |
//+-------------------------------------------------------------------------------------------+ 
int init()
   {
   FLAG_deinit  = false;
       
   if(Apply_On_White_vs_Black_Chart)
      {       
      SetIndexBuffer(0, BufferH1);   
      SetIndexStyle(0, DRAW_HISTOGRAM);  
      SetIndexBuffer(1, BufferH2);   
      SetIndexStyle(1, DRAW_HISTOGRAM);   
      SetIndexBuffer(2, BufferH3);  
      SetIndexStyle(2, DRAW_HISTOGRAM);
      SetIndexBuffer(3, BufferH4);   
      SetIndexStyle(3, DRAW_LINE);       
      }
      else
      {
      SetIndexBuffer(4, BufferH5);   
      SetIndexStyle(4, DRAW_HISTOGRAM);  
      SetIndexBuffer(5, BufferH6);   
      SetIndexStyle(5, DRAW_HISTOGRAM);   
      SetIndexBuffer(6, BufferH7);  
      SetIndexStyle(6, DRAW_HISTOGRAM); 
      SetIndexBuffer(7, BufferH8);   
      SetIndexStyle(7, DRAW_LINE);             
      }        
 
   //Indicator subwindow data labels     
   SetIndexLabel(0,  NULL);
   SetIndexLabel(1,  NULL); 
   SetIndexLabel(2,  NULL);
   SetIndexLabel(3,  NULL);
   SetIndexLabel(4,  NULL);
   SetIndexLabel(5,  NULL);
   SetIndexLabel(6,  NULL);
   SetIndexLabel(7,  NULL);
    
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Custom indicator deinitialization function                                                |
//+-------------------------------------------------------------------------------------------+    
int deinit()
   {
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Custom indicator start function                                                           |
//+-------------------------------------------------------------------------------------------+
int start()
   { 
   //If Indicator is "Off" deinitialize only once, not every tick------------------------------  
   if ((!Indicator_On) || (Period() < Display_Min_TF) || (Period() > Display_Max_TF))
      {
      IndicatorShortName("SonicR Volume  -Off.   ");
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }   

   //At each bar, compute new average volume, and in real time determine bar height and color        
   counted_bars = IndicatorCounted();      
   for(i = Bars-1-IndicatorCounted(); i >= 0; i--) 
      {
      if(Apply_On_White_vs_Black_Chart)
         {     
         //Reset buffers to zero.                          
         BufferH1[i] = 0;               //graduation 1 bars, up average
         BufferH2[i] = 0;               //graduation 2 bars, average to 100% over average
         BufferH3[i] = 0;               //graduation 3 bars, more than 100% over average 
         BufferH4[i] = 0;               //Average Line 
         
         //Get Average Volume 
         BufferH4[i] = avgSubr(i);      
         Avg_Volume= DoubleToStr(BufferH4[i],0);
         
         //Input average and current volume into ShortName
         IndicatorShortName("SonicR Volume  (Period " + Averaging_Period + ", Average " + 
            Avg_Volume + ", Bar "+ DoubleToStr(iVolume(NULL,0,i),0) +")   ");                                   
          
         //Construct histogram with the colors for the three graduations
         nvo = Volume[i]/BufferH4[i];
         nvo = nvo*100 - 100;                                                                         
         if (nvo<0) {BufferH1[i]=Volume[i];}  
         else if (nvo<100) {BufferH2[i]=Volume[i];}  
         else {BufferH3[i]=Volume[i];}  
         } 
                         
      else
         {     
         //Reset buffers to zero.                           
         BufferH5[i] = 0;               //graduation 1 bars, up to average
         BufferH6[i] = 0;               //graduation 2 bars, average to 100% over average
         BufferH7[i] = 0;               //graduation 3 bars, more than 100% over average
         BufferH8[i] = 0;               //Average Line  
      
         //Get Average Volume 
         BufferH8[i] = avgSubr(i);      
         Avg_Volume = DoubleToStr(BufferH8[i],0);
         
         //Input average and current volume into ShortName
         IndicatorShortName("SonicR Volume  (Period " + Averaging_Period + ", Average " + 
            Avg_Volume + ", Bar "+ DoubleToStr(iVolume(NULL,0,i),0) +")   ");                                   
          
         //Construct histogram with the colors for the three graduations
         nvo = Volume[i]/BufferH8[i];
         nvo = nvo*100 - 100;                                                                         
         if (nvo<0) {BufferH5[i]=Volume[i];}  
         else if (nvo<100) {BufferH6[i]=Volume[i];}  
         else {BufferH7[i]=Volume[i];}  
         }                
      } //Finished loop
                                                                      
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Subroutine - compute average volume                                                       |
//+-------------------------------------------------------------------------------------------+    
double avgSubr(int i)
   {
   avg = 0;
   for (j = i; j < (i+Averaging_Period); j++) avg = avg + Volume[j];
   avg = avg / Averaging_Period;
   return (avg);
   }

//+-------------------------------------------------------------------------------------------+
//|Custom indicator end                                                                       |
//+-------------------------------------------------------------------------------------------+     

